within SorpLib.Applications.SummaryRecords;
model TwoBedSystem_N1
  "Record containing most import variables of a two-bed adsorption system based 
   on single-component adsorption"

  //
  // Definition of summary records
  //
  input Components.HeatExchangers.SummaryRecords.SingleEvaporatorCondenser
    evaporator
    "Record sumarizing most important variables of the evaporator"
    annotation (Dialog(tab="General",group="General"));

  input Components.HeatExchangers.SummaryRecords.SingleEvaporatorCondenser
    condenser
    "Record sumarizing most important variables of the condenser"
    annotation (Dialog(tab="General",group="General"));

  input Components.ClosedAdsorbers.SummaryRecords.SingleAdsorptionAdsorber
    adsorberBedOne
    "Record sumarizing most important variables of the first adsorber"
    annotation (Dialog(tab="General",group="General"));

  input Components.ClosedAdsorbers.SummaryRecords.SingleAdsorptionAdsorber
    adsorberBedTwo
    "Record sumarizing most important variables of the second adsorber"
    annotation (Dialog(tab="General",group="General"));

  //
  // Definition of variables to evaluate mass
  //
  Modelica.SIunits.Mass mass_refrigerant=
    evaporator.mass + condenser.mass + adsorberBedOne.mass + adsorberBedTwo.mass
    "Overall mass of refrigerant in the system";
  input Modelica.SIunits.Mass mass_sorbent
    "Overall sorbent mass in the system"
    annotation (Dialog(tab="General", group="Mass Evaluation"));
  input Modelica.SIunits.Mass mass_hx_ads
    "Overall metallic mass of adsorber heat exchangers"
    annotation (Dialog(tab="General", group="Mass Evaluation"));
  input Modelica.SIunits.Mass mass_hx_evap
    "Overall metallic mass of evaporator heat exchangers"
    annotation (Dialog(tab="General", group="Mass Evaluation"));
  input Modelica.SIunits.Mass mass_hx_cond
    "Overall metallic mass of condenser heat exchangers"
    annotation (Dialog(tab="General", group="Mass Evaluation"));

  //
  // Definition of variables to evaluate energy
  //
  input Modelica.SIunits.HeatFlowRate Q_flow_ads1_input
    "Heat flow rate provided for desorption of adsorber bed one"
    annotation (Dialog(tab="General", group="Energetic Evaluation"));
  input Modelica.SIunits.HeatFlowRate Q_flow_ads1_output
    "Heat flow rate released during adsorption of adsorber bed one"
    annotation (Dialog(tab="General", group="Energetic Evaluation"));

  input Modelica.SIunits.HeatFlowRate Q_flow_ads2_input
    "Heat flow rate provided for desorption of adsorber bed two"
    annotation (Dialog(tab="General", group="Energetic Evaluation"));
  input Modelica.SIunits.HeatFlowRate Q_flow_ads2_output
    "Heat flow rate released during adsorption of adsorber bed two"
    annotation (Dialog(tab="General", group="Energetic Evaluation"));

  Modelica.SIunits.Energy Q_E(start=0, fixed=true)
    "Integrated heat flow rate of evaporator";
  Modelica.SIunits.Energy Q_C(start=0, fixed=true)
    "Integrated heat flow rate of condenser";
  Modelica.SIunits.Energy Q_A1(start=0, fixed=true)
    "Integrated heat flow rate of adsorber bed 1";
  Modelica.SIunits.Energy Q_A2(start=0, fixed=true)
    "Integrated heat flow rate of adsorber bed 2";

  Modelica.SIunits.Energy Q_A_ads1_input(start=0, fixed=true)
    "Integrated heat flow rate provided for desorption of adsorber bed one";
  Modelica.SIunits.Energy Q_A_ads1_output(start=0, fixed=true)
    "Integrated heat flow rate released during adsorption of adsorber bed one";

  Modelica.SIunits.Energy Q_A_ads2_input(start=0, fixed=true)
    "Integrated heat flow rate provided for desorption of adsorber bed two";
  Modelica.SIunits.Energy Q_A_ads2_output(start=0, fixed=true)
    "Integrated heat flow rate released during adsorption of adsorber bed two";

  //
  // Definition of characteristic numbers
  //
  input Real COP(unit="J/J")
    "Coefficient of performance"
    annotation (Dialog(tab="General", group="Characteristic Numbers"));

  input Real SP_sor(unit="W/kg")
    "Specific power: Useful energy related to sorbent mass"
    annotation (Dialog(tab="General", group="Characteristic Numbers"));
  input Real SH_sor_ads(unit="W/kg")
    "Specific power: Useful energy related to sorbent mass and mass of adsorber
     heat exchanger"
    annotation (Dialog(tab="General", group="Characteristic Numbers"));
  input Real SP_sor_ads_evp_cond(unit="W/kg")
    "Specific power: Useful energy related to sorbent mass and masses of adsorber,
     evaporator, and condenser heat exchanger"
    annotation (Dialog(tab="General", group="Characteristic Numbers"));

equation
  //
  // Calculation of energies
  //
  der(Q_E) = evaporator.DH_liquid;
  der(Q_A1) = adsorberBedOne.DH_liquid;
  der(Q_A2) = adsorberBedTwo.DH_liquid;
  der(Q_C) = condenser.DH_liquid;

  der(Q_A_ads1_input) = Q_flow_ads1_input;
  der(Q_A_ads1_output) = Q_flow_ads1_output;
  der(Q_A_ads2_input) = Q_flow_ads2_input;
  der(Q_A_ads2_output) = Q_flow_ads2_output;

  //
  // Annotations
  //
  annotation (Icon(graphics={
      Rectangle(
        extent={{-92,70},{88,-70}},
        lineColor={0,0,0},
        fillColor={255,255,255},
        fillPattern=FillPattern.Solid),
      Line(points={{-82,50},{78,50}}, color={0,0,0}),
      Line(points={{-62,30},{58,30}}, color={0,0,0}),
      Line(points={{-62,10},{58,10}}, color={0,0,0}),
      Line(points={{-62,-10},{58,-10}}, color={0,0,0}),
      Line(points={{-62,-30},{58,-30}}, color={0,0,0}),
      Line(points={{-62,-50},{58,-50}}, color={0,0,0}),
      Line(points={{-2,50},{-2,-50}},
                                    color={0,0,0}),
        Text(
          extent={{-100,100},{100,80}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={217,67,180},
          fillPattern=FillPattern.Solid,
          textString="Summary")}), Documentation(revisions="<html>
<ul>
  <li>
  January 26, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end TwoBedSystem_N1;
