within SorpLib.Components.Boundaries.BaseClasses;
partial model PartialBoundary "Base model for all boundary models"

  //
  // Definition of parameters defining the boundary type
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="General", group="Boundary Type"),
                evaluate=true,
                HideResult=true);

  parameter Internals.Types.BoundaryPotentialFlow boundaryType_PF=SorpLib.Internals.Types.BoundaryPotentialFlow.Pressure
    "Boundary type: Pressure, mass flow, or volume flow"
    annotation (Dialog(tab="General", group="Boundary Type"),
                evaluate=true,
                HideResult=true);
  parameter Internals.Types.BoundaryStream boundaryType_S=SorpLib.Internals.Types.BoundaryStream.SpecificEnthalpy
    "Boundary type: Specific enthalpy or temperature"
    annotation (Dialog(tab="General", group="Boundary Type"),
                evaluate=true,
                HideResult=true);

  //
  // Definition of parameters defining potential and flow variables
  //
  parameter Boolean use_pInput = false
    " = true, if p is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.Pressure)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.Pressure p = 1.01325e5
    "Fixed pressure at boundary"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.Pressure)
                and not use_pInput),
                HideResult=use_pInput);

  parameter Boolean use_mFlowInput = false
    "=true, if m_flow is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.MassFlow)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.MassFlowRate m_flow = 0.001
    "Fixed mass flow at boundary"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.MassFlow)
                and not use_mFlowInput),
                HideResult=use_mFlowInput);

  //
  // Definition of parameters defining stream variables
  //
  parameter Boolean use_xiInput = false
    "=true, if xi is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Stream variables"),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.MassFraction[no_components-1] xi=
    fill(1/no_components, no_components-1)
    "Fixed mass fraction at boundary"
    annotation (Dialog(tab="General",group="Stream variables",
                enable=not use_xiInput),
                HideResult=use_xiInput);

  parameter Boolean use_hInput = false
    "=true, if h is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Stream variables",
                enable=(boundaryType_S ==
                Internals.Types.BoundaryStream.SpecificEnthalpy)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.SpecificEnthalpy h = 350e3
    "Fixed specific enthalpy at boundary"
    annotation (Dialog(tab="General",group="Stream variables",
                enable=(boundaryType_S ==
                Internals.Types.BoundaryStream.SpecificEnthalpy
                and not use_hInput)),
                HideResult=use_hInput);

  //
  // Definition of connectors
  //
  Modelica.Blocks.Interfaces.RealInput p_input(final unit="Pa") if
    (boundaryType_PF == Internals.Types.BoundaryPotentialFlow.Pressure and
    use_pInput)
    "Input for pressure"
     annotation (Placement(transformation(extent={{-120,50},{-80,90}}),
       iconTransformation(extent={{-22,60},{-2,80}})));

  Modelica.Blocks.Interfaces.RealInput m_flow_input(final unit="kg/s") if
    boundaryType_PF == Internals.Types.BoundaryPotentialFlow.MassFlow and
    use_mFlowInput
    "Input for mass flow"
    annotation (Placement(transformation(extent={{-120,20},{-80,60}}),
      iconTransformation(extent={{-22,30},{-2,50}})));

  Modelica.Blocks.Interfaces.RealInput[no_components-1] xi_input(final unit="kg/kg") if
    use_xiInput
    "Input for mass fractions"
    annotation (Placement(transformation(extent={{-120,10},{-80,-30}}),
      iconTransformation(extent={{-22,-20},{-2,0}})));

  Modelica.Blocks.Interfaces.RealInput h_input(final unit="J/kg") if
    boundaryType_S == Internals.Types.BoundaryStream.SpecificEnthalpy and
    use_hInput
    "Input for specific enthalpy"
    annotation (Placement(transformation(extent={{-120,-20},{-80,-60}}),
      iconTransformation(extent={{-22,-50},{-2,-30}})));

protected
  Modelica.Blocks.Interfaces.RealInput p_dummy(final unit="Pa") = -1 if
       boundaryType_PF <>Internals.Types.BoundaryPotentialFlow.Pressure
    "Fixed pressure at input";
  Modelica.Blocks.Interfaces.RealInput p_fixed(final unit="Pa") = p if
       (boundaryType_PF == Internals.Types.BoundaryPotentialFlow.Pressure and
    not use_pInput)
    "Fixed pressure at input";

  Modelica.Blocks.Interfaces.RealInput m_flow_dummy(final unit="kg/s") = -1 if
       boundaryType_PF <> Internals.Types.BoundaryPotentialFlow.MassFlow
    "Fixed mass flow at input";
  Modelica.Blocks.Interfaces.RealInput m_flow_fixed(final unit="kg/s") = m_flow if
       (boundaryType_PF == Internals.Types.BoundaryPotentialFlow.MassFlow and
    not use_mFlowInput)
    "Fixed mass flow at input";

  Modelica.Blocks.Interfaces.RealInput[no_components-1] xi_fixed(final unit="kg/kg")=
    xi if not use_xiInput
    "Fixed specific enthalpy input";

  Modelica.Blocks.Interfaces.RealInput h_dummy(final unit="J/kg") = -1 if
       boundaryType_S <> Internals.Types.BoundaryStream.SpecificEnthalpy
    "Fixed specific enthalpy input";
  Modelica.Blocks.Interfaces.RealInput h_fixed(final unit="J/kg") = h if
       (boundaryType_S == Internals.Types.BoundaryStream.SpecificEnthalpy and
    not use_hInput)
    "Fixed specific enthalpy input";

  Modelica.Blocks.Interfaces.RealOutput p_(final unit="Pa")
    "Pressure at boundary";
  Modelica.Blocks.Interfaces.RealOutput m_flow_(final unit="kg/s")
    "Mass flow at boundary";
  Modelica.Blocks.Interfaces.RealOutput[no_components-1] xi_(final unit="kg/kg")
    "Mass fractions at boundary";
  Modelica.Blocks.Interfaces.RealOutput h_(final unit="J/kg")
    "Specific enthalpy at boundary";

  //
  // Definition of ports
  //
public
  replaceable Connectors.BaseClasses.PartialMediaConnector port(final
      no_components=no_components) constrainedby
    Connectors.BaseClasses.PartialMediaConnector "Fluid port"
    annotation (Dialog(tab="General", group="Connectors"), Placement(
        transformation(extent={{-4,-4},{4,4}})));

equation
  //
  // Connectors
  //
  connect(p_dummy,p_);
  connect(p_fixed,p_);
  connect(p_input,p_);

  connect(m_flow_dummy,m_flow_);
  connect(m_flow_fixed,m_flow_);
  connect(m_flow_input,m_flow_);

  connect(xi_fixed,xi_);
  connect(xi_input,xi_);

  connect(h_dummy,h_);
  connect(h_fixed,h_);
  connect(h_input,h_);

  //
  // Fluid properties at port
  //
  port.xi_outflow = xi_
    "Stream variable: Mass fractions";

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false)),
            Diagram(coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 11, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialBoundary;
