within SorpLib.Components.Boundaries.BaseClasses;
partial model PartialMediaLibraryBoundary
  "Base model for all boundary models using a specific media library"
  extends PartialBoundary;

  //
  // Definition of parameters defining potential and flow variables
  //
  parameter Boolean use_VFlowInput = false
    "=true, if V_flow is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.VolumeFlow)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.VolumeFlowRate V_flow = 10/1000/60
    "Fixed volume flow at boundary"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType_PF ==
                Internals.Types.BoundaryPotentialFlow.VolumeFlow)
                and not use_VFlowInput),
                HideResult=use_VFlowInput);

  //
  // Definition of parameters defining stream variables
  //
  parameter Boolean use_TInput = false
    "=true, if T is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Stream variables",
                enable=(boundaryType_S ==
                Internals.Types.BoundaryStream.Temperature)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.Temperature T = 293.15
    "Fixed temperature at boundary"
    annotation (Dialog(tab="General",group="Stream variables",
                enable=(boundaryType_S ==
                Internals.Types.BoundaryStream.Temperature
                and not use_TInput)),
                HideResult=use_TInput);

  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of connectors
  //
  Modelica.Blocks.Interfaces.RealInput V_flow_input(final unit="m3/s") if
    boundaryType_PF == Internals.Types.BoundaryPotentialFlow.VolumeFlow and
    use_VFlowInput
    "Input for volume flow"
    annotation (Placement(transformation(extent={{-120,-10},{-80,30}}),
      iconTransformation(extent={{-22,0},{-2,20}})));

  Modelica.Blocks.Interfaces.RealInput T_input(final unit="K") if
    boundaryType_S == Internals.Types.BoundaryStream.Temperature and
    use_TInput
    "Input for specific enthalpy"
    annotation (Placement(transformation(extent={{-120,-50},{-80,-90}}),
      iconTransformation(extent={{-22,-80},{-2,-60}})));

protected
  Modelica.Blocks.Interfaces.RealInput V_flow_dummy(final unit="m3/s") = -1 if
       boundaryType_PF <> Internals.Types.BoundaryPotentialFlow.VolumeFlow
    "Fixed volume flow at input";
  Modelica.Blocks.Interfaces.RealInput V_flow_fixed(final unit="m3/s") = V_flow if
       (boundaryType_PF == Internals.Types.BoundaryPotentialFlow.VolumeFlow and
    not use_VFlowInput)
    "Fixed volume flow at input";

  Modelica.Blocks.Interfaces.RealInput d_out_dummy(final unit="kg/m3") = -1 if
    boundaryType_PF <> Internals.Types.BoundaryPotentialFlow.VolumeFlow
    "Fixed density";

  Modelica.Blocks.Interfaces.RealInput d_in_dummy(final unit="kg/m3") = -1 if
    boundaryType_PF <> Internals.Types.BoundaryPotentialFlow.VolumeFlow
    "Fixed density";

  Modelica.Blocks.Interfaces.RealInput T_dummy(final unit="K") = -1 if
       boundaryType_S <> Internals.Types.BoundaryStream.Temperature
    "Fixed specific enthalpy input";
  Modelica.Blocks.Interfaces.RealInput T_fixed(final unit="K") = T if
       (boundaryType_S == Internals.Types.BoundaryStream.Temperature and
    not use_TInput)
    "Fixed specific enthalpy input";

  Modelica.Blocks.Interfaces.RealInput h_pT_dummy(final unit="J/kg") = -1 if
    boundaryType_S <> Internals.Types.BoundaryStream.Temperature
    "Fixed calculated specific enthalpy input";

  Modelica.Blocks.Interfaces.RealOutput V_flow_(final unit="m3/s")
    "Volume flow at boundary";
  Modelica.Blocks.Interfaces.RealOutput d_out(final unit="kg/m3")
    "Density at boundary";
  Modelica.Blocks.Interfaces.RealOutput d_in(final unit="kg/m3")
    "Density at boundary";
  Modelica.Blocks.Interfaces.RealOutput T_(final unit="K")
    "Temperature at boundary";
  Modelica.Blocks.Interfaces.RealOutput h_calc(final unit="J/kg")
    "Calculated specific enthalpy at boundary";

equation
  //
  // Connectors
  //
  connect(V_flow_dummy,V_flow_);
  connect(V_flow_fixed,V_flow_);
  connect(V_flow_input,V_flow_);

  connect(d_out_dummy,d_out);

  connect(d_in_dummy,d_in);

  connect(T_dummy,T_);
  connect(T_fixed,T_);
  connect(T_input,T_);

  connect(h_pT_dummy,h_calc);

  //
  // Fluid properties at port
  //
  if boundaryType_PF == Internals.Types.BoundaryPotentialFlow.Pressure then
    port.p = p_
      "Potential variable: Pressure";

  elseif boundaryType_PF == Internals.Types.BoundaryPotentialFlow.MassFlow then
    port.m_flow = m_flow_
      "Flow variable: Mass flow";

  elseif boundaryType_PF == Internals.Types.BoundaryPotentialFlow.VolumeFlow then
    if avoid_events then
      //
      // Use noEvent()-operator
      //
      port.m_flow = if noEvent(V_flow_ > 0) then
        V_flow_ * d_in else V_flow_ * d_out
        "Flow variable: Mass flow";

    else
      //
      // Do not use noEvent()-operator
      //
      port.m_flow = if V_flow_ > 0 then V_flow_ * d_in else V_flow_ * d_out
        "Flow variable: Mass flow";

    end if;

  end if;

  if boundaryType_S == Internals.Types.BoundaryStream.SpecificEnthalpy then
    port.h_outflow = h_
      "Stream variable: Specific enthalpy";

  elseif boundaryType_S == Internals.Types.BoundaryStream.Temperature then
    port.h_outflow = h_calc
      "Stream variable: Specific enthalpy";

  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 11, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialMediaLibraryBoundary;
