within SorpLib.Components.Boundaries.Thermal;
model HeatSource
  "Boundary model of a heat source"

  //
  // Definition of parameters defining the boundary type
  //
  parameter Internals.Types.BoundaryThermal boundaryType=SorpLib.Internals.Types.BoundaryThermal.Temperature
    "Boundary type: Temperature or heat flow rate"
    annotation (Dialog(tab="General", group="Boundary Type"),
                evaluate=true,
                HideResult=true);

  //
  // Definition of parameters defining potential and flow variables
  //
  parameter Boolean use_TInput = false
    " = true, if T is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType ==
                Internals.Types.BoundaryThermal.Temperature)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.Temperature T = 293.15
    "Fixed temperature at boundary"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType ==
                Internals.Types.BoundaryThermal.Temperature)
                and not use_TInput),
                HideResult=use_TInput);

  parameter Boolean use_QFlowInput = false
    "=true, if Q_flow is defined by input; otherwise, fixed value is used"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType ==
                Internals.Types.BoundaryThermal.HeatFlow)),
                evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.HeatFlowRate Q_flow = 25
    "Fixed heat flow at boundary"
    annotation (Dialog(tab="General",group="Potential and Flow Variables",
                enable=(boundaryType ==
                Internals.Types.BoundaryThermal.HeatFlow)
                and not use_QFlowInput),
                HideResult=use_QFlowInput);

  //
  // Definition of connectors
  //
  Modelica.Blocks.Interfaces.RealInput T_input(final unit="K") if
    (boundaryType == Internals.Types.BoundaryThermal.Temperature and
    use_TInput)
    "Input for temperature"
     annotation (Placement(transformation(extent={{-40,30},{0,70}}),
       iconTransformation(extent={{-20,42},{0,62}})));

  Modelica.Blocks.Interfaces.RealInput Q_flow_input(final unit="J/s") if
    boundaryType == Internals.Types.BoundaryThermal.HeatFlow and
    use_QFlowInput
    "Input for heat flow"
    annotation (Placement(transformation(extent={{-40,-70},{0,-30}}),
      iconTransformation(extent={{-20,-60},{0,-40}})));

protected
  Modelica.Blocks.Interfaces.RealInput T_dummy(final unit="K") = -1 if
    boundaryType <>Internals.Types.BoundaryThermal.Temperature
    "Fixed temperature at input";
  Modelica.Blocks.Interfaces.RealInput T_fixed(final unit="K") = T if
    (boundaryType == Internals.Types.BoundaryThermal.Temperature and
    not use_TInput)
    "Fixed temperature at input";

  Modelica.Blocks.Interfaces.RealInput Q_flow_dummy(final unit="J/s") = -1 if
    boundaryType <> Internals.Types.BoundaryThermal.HeatFlow
    "Fixed heat flow at input";
  Modelica.Blocks.Interfaces.RealInput Q_flow_fixed(final unit="J/s") = Q_flow if
    (boundaryType == Internals.Types.BoundaryThermal.HeatFlow and
    not use_QFlowInput)
    "Fixed heat flow at input";

  Modelica.Blocks.Interfaces.RealOutput T_(final unit="K")
    "Temperature at boundary";
  Modelica.Blocks.Interfaces.RealOutput Q_flow_(final unit="J/s")
    "Heat flow at boundary";

public
  Connectors.Thermal.Port_a port
    "Heat port"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

equation
  //
  // Connectors
  //
  connect(T_dummy,T_);
  connect(T_fixed,T_);
  connect(T_input,T_);

  connect(Q_flow_dummy,Q_flow_);
  connect(Q_flow_fixed,Q_flow_);
  connect(Q_flow_input,Q_flow_);

  //
  // Properties at port
  //
  if boundaryType == Internals.Types.BoundaryThermal.Temperature then
    //
    // Temperature boundary
    //
    port.T = T_
      "Temperature in port";

  else
    //
    // Heat flow boundary
    //
    port.Q_flow = Q_flow_
      "Heat flow in port";

  end if;

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
                             Rectangle(
          extent={{-2,80},{2,-80}},
          lineColor={238,46,47},
          lineThickness=0.5,
          fillColor={238,46,47},
          fillPattern=FillPattern.Solid)}),                      Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 11, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end HeatSource;
