within SorpLib.Components.Cells.BaseClasses;
partial model PartialFluidCell "Base model for all fluid cell models"

  //
  // Definition of general parameters
  //
  replaceable parameter Geometry.FluidCell
    cellGeometry constrainedby Geometry.FluidCell
    "Cell geometry"
    annotation (Dialog(tab="General", group="Geometry"),
                choicesAllMatching = true);

  //
  // Definition of parameters describing the medium
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);
  parameter Boolean state_phxi = true
    "= true, if independent states are p, h, and xi; otherwise, independent states
     are p, T, and xi"
    annotation (Dialog(tab="Media", group="General"),
                choices(choice=true "p, h, and xi", choice=false "p, T, and xi"),
                Evaluate=true,
                HideResult = true);
  parameter Boolean calc_propertyRecord = false
    "= true, if property record is calculated for models of transport phenomena"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Boolean singleStateVariable = false
    "= true, if only one state variable exists (e.g., for incompressible media)"
    annotation (Dialog(tab="Initialisation", group="General"),
                Evaluate=true,
                HideResult = true);

  parameter Modelica.SIunits.Pressure pInitial= 1e5
    "Initial value for pressure"
    annotation (Dialog(tab="Initialisation", group="Initial Values"),
                HideResult = true);
  parameter Modelica.SIunits.Temperature TInitial= 293.15
    "Initial value for temperature"
    annotation (Dialog(tab="Initialisation", group="Initial Values",
                       enable=not state_phxi),
                HideResult = true);
  parameter Modelica.SIunits.SpecificEnthalpy hInitial= 84011
    "Initial value for specific enthalpy"
    annotation (Dialog(tab="Initialisation", group="Initial Values",
                       enable=state_phxi),
                HideResult = true);
  parameter Modelica.SIunits.MassFraction[no_components-1] xiInitial=
    fill(1/no_components, no_components-1)
    "Initial values for independent mass fractions"
    annotation (Dialog(tab="Initialisation", group="Initial Values"),
                HideResult = true);

  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Start value for mass flow rate"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);
  parameter Modelica.SIunits.Density rhoStart= 1000
    "Start value for density"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Internals.Types.BalanceEquations type_overallMassBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the overall mass equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.BalanceEquations type_independentMassBalances=
    Internals.Types.BalanceEquations.SteadyState_FreeInitial
    "Handling of the independent mass equations and corresponding initialisations"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.BalanceEquations type_energyBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the energy equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Boolean neglectTermVp = false
    "Neglect term V*p in the energy balance: i.e., U = H"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Definition of variables
  //
public
  Modelica.SIunits.Pressure p(
    start = pInitial,
    stateSelect=StateSelect.prefer)
    "Pressure of fluid in cell";
  Modelica.SIunits.Temperature T(
    start = TInitial,
    stateSelect= if not state_phxi then StateSelect.prefer else
      StateSelect.avoid)
    "Temperature of fluid in cell";
  Modelica.SIunits.Density rho(
    start = rhoStart)
    "Density of fluid in cell";

  Modelica.SIunits.SpecificEnthalpy h(
    start = hInitial,
    stateSelect= if state_phxi then StateSelect.prefer else
      StateSelect.avoid)
    "Specific enthalpy of fluid in cell";
  Modelica.SIunits.SpecificInternalEnergy u(
    start = hInitial + pInitial/rhoStart)
    "Specific internal energy of fluid in cell";

  Modelica.SIunits.MassFraction[no_components-1] xi(
    start = xiInitial,
    stateSelect=StateSelect.prefer)
    "Mass fractions of fluid in cell";

  Modelica.SIunits.Mass mass
    "Mass of overall fluid in cell";
  Modelica.SIunits.Mass[no_components-1] mass_i
    "Masses of independent components of fluid in cell";
  Modelica.SIunits.Energy U
    "Internal energy of fluid in cell";

  Real dmass_dtau(unit="kg/(s)")
    "Derivative of fluid mass in cell with respect to time";
  Real dU_dtau(unit="J/(s)")
    "Derivative of internal energy of fluid in cell with respect to time";

  //
  // Definition of connectors
  //
  Connectors.Thermal.Port_a heatPort "Heat port of cell"
    annotation (Placement(transformation(extent={{-10,50},{10,70}}),
                iconTransformation(extent={{-10,50},{10,70}})));

  replaceable Connectors.BaseClasses.PartialMediaConnector port_a(
    final no_components=no_components,
    m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-80,-10},{-60,10}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector port_b(
    final no_components=no_components,
    m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{42,-10},{62,10}})));

initial equation
  if not singleStateVariable and type_overallMassBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    p = pInitial
      "Fixed initial value";

  elseif not singleStateVariable and type_overallMassBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(p) = 0
      "Steady-state initialisation of overall mass balance";

  end if;

  if type_independentMassBalances ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    xi = xiInitial
      "Fixed initial value";

  elseif type_independentMassBalances ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(xi) = zeros(no_components-1)
      "Steady-state initialisation of independent mass balances";

  end if;

  if type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    if state_phxi then
      h = hInitial
        "Fixed initial value";
    else
      T = TInitial
        "Fixed initial value";
    end if;

  elseif type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    if state_phxi then
      der(h) = 0
        "Steady-state initialisation of energy balance";
    else
      der(T) = 0
        "Steady-state initialisation of energy balance";
    end if;

  end if;

equation
  //
  // Momentum balance
  //
  p = port_a.p
    "No pressure drop within cell (i.e., homogenous volume)";
  p = port_b.p
    "No pressure drop within cell (i.e., homogenous volume)";

  //
  // Mass balance
  //
  mass = rho*cellGeometry.volume
    "Overall fluid mass in cell";

  mass_i = mass * xi
    "Masses of independent components of fluid in cell";

  port_a.xi_outflow = xi
    "Mass fractions leaving port A (i.e., homogenous volume)";
  port_b.xi_outflow = xi
    "Mass fractions leaving port B (i.e., homogenous volume)";

  //
  // Energy balance
  //
  U = mass*u
    "Internal energy of fluid in cell";

  u = if neglectTermVp then h else h - p/rho
    "Specific internal energy of fluid in cell";

  port_a.h_outflow = h
    "Specific enthalpy leaving port A (i.e., homogenous volume)";
  port_b.h_outflow = h
    "Specific enthalpy leaving port B (i.e., homogenous volume)";

  //
  // Thermal boundary
  //
  T = heatPort.T
    "Temperature of fluid in cell";

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
        Ellipse(
          extent={{-100,100},{-80,80}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-100,-20},{-80,-40}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-40,-80},{-20,-100}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{80,-80},{100,-100}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{20,100},{40,80}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-40,40},{-20,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{20,-20},{40,-40}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{80,40},{100,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Line(
          points={{-90,90},{30,90},{90,30},{90,-90},{30,-30},{-90,-30},{-90,90},
              {-30,30},{-30,-90},{90,-90}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-30,30},{92,30}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-90,-30},{-30,-90}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{30,90},{30,-30}},
          color={0,0,0},
          thickness=0.5)}), Diagram(coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialFluidCell;
