within SorpLib.Components.Cells.BaseClasses;
partial model PartialGasCell "Base model for all gas cell models"
  extends PartialFluidCell(
    final singleStateVariable = false,
    state_phxi = true,
    no_components = 2);

  //
  // Definition of parameters describing the medium
  //
  parameter Integer index_adsorptiv = 1
    "Index of adsorptiv that is adsorbed or desorbed in gas mixture"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of variables
  //
  Modelica.SIunits.Pressure p_adsorptiv
    "Partial pressures of adsorptiv in cell";

  Modelica.SIunits.MassFraction[no_components-1] xi_adsorptiv
    "Mass fractions at adsorptive port required for mass balance of independent
     components";

protected
  Modelica.SIunits.SpecificEnthalpy h_adsorptiv_eb
    "Specific enthalpy of adsorptiv required for energy balance (i.e.,
     required due to conversion of different media models)"
     annotation(Hide=true);
  Modelica.SIunits.SpecificEnthalpy h_adsorptiv_gas_in
    "Specific enthalpy of adsorptiv entering cell according to ideal gas media
     model"
     annotation(Hide=true);
  Modelica.SIunits.SpecificEnthalpy h_adsorptiv_gas_out
    "Specific enthalpy of adsorptiv leaving cell according to ideal gas media
     model"
     annotation(Hide=true);
  Modelica.SIunits.SpecificEnthalpy h_adsorptiv_vle_in
    "Specific enthalpy of adsorptiv entering cell according to VLE media model"
     annotation(Hide=true);
  Modelica.SIunits.SpecificEnthalpy h_adsorptiv_vle_out
    "Specific enthalpy of adsorptiv leaving cell according to VLE media model"
     annotation(Hide=true);

  //
  // Definition of connectors
  //
public
  replaceable Connectors.BaseClasses.PartialMediaConnector port_adsorptiv(
    final no_components=1,
    m_flow(start = m_flowInitial))
    "Fluid port of adsorptiv that is adsorbed or desorbed (i.e., VLE port)"
    annotation (Placement(transformation(extent={{-10,-70},{10,-50}}),
        iconTransformation(extent={{-10,-70},{10,-50}})));

  Connectors.Thermal.Port_a heatPort_adsorptiv
    "Heat port of cell"
    annotation (Placement(transformation(extent={{20,-70},{40,-50}}),
                iconTransformation(extent={{20,-70},{40,-50}})));

equation
  //
  // Momentum balance
  //
  p_adsorptiv = port_adsorptiv.p
    "No pressure drop within cell (i.e., homogenous volume)";

  //
  // Mass balance
  //
  if type_overallMassBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = 0
      "No change of fluid mass wrt. time";
    port_a.m_flow + port_b.m_flow + port_adsorptiv.m_flow = 0
      "Steady-state overall mass balance";

  else
    port_a.m_flow + port_b.m_flow + port_adsorptiv.m_flow = dmass_dtau
      "Transient overall mass balance";

  end if;

  if type_independentMassBalances ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if avoid_events then
      zeros(no_components-1) =
        port_a.m_flow*noEvent(actualStream(port_a.xi_outflow)) +
        port_b.m_flow*noEvent(actualStream(port_b.xi_outflow)) +
        port_adsorptiv.m_flow*xi_adsorptiv
        "Steady-state mass balances of independent components";

    else
      zeros(no_components-1) =
        port_a.m_flow*actualStream(port_a.xi_outflow) +
        port_b.m_flow*actualStream(port_b.xi_outflow) +
        port_adsorptiv.m_flow*xi_adsorptiv
        "Steady-state mass balances of independent components";

    end if;

  else
    if avoid_events then
      der(xi) = 1/mass*(
        port_a.m_flow*(noEvent(actualStream(port_a.xi_outflow))-xi) +
        port_b.m_flow*(noEvent(actualStream(port_b.xi_outflow))-xi) +
        port_adsorptiv.m_flow*(xi_adsorptiv-xi))
        "Steady-state mass balances of independent components";

    else
      der(xi) = 1/mass*(
        port_a.m_flow*(actualStream(port_a.xi_outflow)-xi) +
        port_b.m_flow*(actualStream(port_b.xi_outflow)-xi) +
        port_adsorptiv.m_flow*(xi_adsorptiv-xi))
        "Steady-state mass balances of independent components";

    end if;

  end if;

  //
  // Calculate mass fractions at adsorptiv port required for mass balance:
  // Only one component enters or leveas the cell via the port
  //
  for i in 1:no_components-1 loop
    if i == index_adsorptiv then
      xi_adsorptiv[i] = 1
        "Adsorptiv is one of the independent components";
    else
      xi_adsorptiv[i] = 0
        "Component is not the adsorptiv";
    end if;
  end for;

  //
  // Energy balance
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dU_dtau = 0
      "No change of internal energy wrt. time";

    if avoid_events then
      0 = heatPort.Q_flow + heatPort_adsorptiv.Q_flow +
        port_a.m_flow*noEvent(actualStream(port_a.h_outflow)) +
        port_b.m_flow*noEvent(actualStream(port_b.h_outflow)) +
        port_adsorptiv.m_flow*h_adsorptiv_eb
        "Steady-state energy balance";

      h_adsorptiv_eb = noEvent(Modelica.Fluid.Utilities.regStep(
        port_adsorptiv.m_flow, h_adsorptiv_gas_in, h_adsorptiv_gas_out, 1e-6))
        "Specific enthalpy of adsorptiv required for mass balance: During
         adsorption, specific enthalpy of adsorptiv in gas mixture is correct
         as the enthalpy is converted at port; during desorption, specific 
         enthalpy of adsorptive port needs to be converted from VLE to ideal
         gas";

    else
      0 = heatPort.Q_flow + heatPort_adsorptiv.Q_flow +
        port_a.m_flow*actualStream(port_a.h_outflow) +
        port_b.m_flow*actualStream(port_b.h_outflow) +
        port_adsorptiv.m_flow*h_adsorptiv_eb
        "Steady-state energy balance";

      h_adsorptiv_eb = Modelica.Fluid.Utilities.regStep(port_adsorptiv.m_flow,
        h_adsorptiv_gas_in, h_adsorptiv_gas_out, 1e-6)
        "Specific enthalpy of adsorptiv required for mass balance: During
         adsorption, specific enthalpy of adsorptiv in gas mixture is correct
         as the enthalpy is converted at port; during desorption, specific 
         enthalpy of adsorptive port needs to be converted from VLE to ideal
         gas";

    end if;

  else
    if avoid_events then
      dU_dtau = heatPort.Q_flow + heatPort_adsorptiv.Q_flow +
        port_a.m_flow*noEvent(actualStream(port_a.h_outflow)) +
        port_b.m_flow*noEvent(actualStream(port_b.h_outflow)) +
        port_adsorptiv.m_flow*h_adsorptiv_eb
        "Transient energy balance";

      h_adsorptiv_eb = noEvent(Modelica.Fluid.Utilities.regStep(
        port_adsorptiv.m_flow, h_adsorptiv_gas_in, h_adsorptiv_gas_out, 1e-6))
        "Specific enthalpy of adsorptiv required for mass balance: During
         adsorption, specific enthalpy of adsorptiv in gas mixture is correct
         as the enthalpy is converted at port; during desorption, specific 
         enthalpy of adsorptive port needs to be converted from VLE to ideal
         gas";

    else
      dU_dtau = heatPort.Q_flow + heatPort_adsorptiv.Q_flow +
        port_a.m_flow*actualStream(port_a.h_outflow) +
        port_b.m_flow*actualStream(port_b.h_outflow) +
        port_adsorptiv.m_flow*h_adsorptiv_eb
        "Transient energy balance";

      h_adsorptiv_eb = Modelica.Fluid.Utilities.regStep(port_adsorptiv.m_flow,
        h_adsorptiv_gas_in, h_adsorptiv_gas_out, 1e-6)
        "Specific enthalpy of adsorptiv required for mass balance: During
         adsorption, specific enthalpy of adsorptiv in gas mixture is correct
         as the enthalpy is converted at port; during desorption, specific 
         enthalpy of adsorptive port needs to be converted from VLE to ideal
         gas";

    end if;

  end if;

  port_adsorptiv.h_outflow = h_adsorptiv_vle_out
    "Specific enthalpy leaving port of adsorptiv (i.e., homogenous volume)";

  //
  // Thermal boundary
  //
  T = heatPort_adsorptiv.T
    "Temperature of fluid in cell";

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  November 23, 2017, by Andrej Gibelhaus:<br>
  Tidy up implementation and enhance documentation for publication of library.<br>
  </li>
  <li>
  May 26, 2021, by Mirko Engelpracht:<br/>
  Major revision (e.g., object-oriented approach, media models, ...).
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
<p>Mass balance: </p>
<p align=\"center\"><i>V</i> <i>d<code>&rho;</code>/dt</i> = 
<i><code>&#7745;</code></i><sub>gas,in</sub> + <i><code>&#7745;</code></i>
<sub>gas,out</sub> + <i><code>&#7745;</code></i><sub>VLE</sub> </p>
<p>where <i>V</i> is the volume, <i>&rho;</i> is the density of the gas, and
<i><code>&#7745;</code></i><sub>gas,in</sub>, <i><code>&#7745;</code></i><sub>
gas,out</sub> and <i><code>&#7745;</code></i><sub>VLE</sub> are the mass flow 
rates at the gas ports and at the vle port.</p>
<p>The derivative <i>d<code>&rho;</code>/dt</i> can be expressed by the total 
differential: </p>
<p align=\"center\"><i>d<code>&rho;</code>/dt</i> = <i>d<code>&rho;
</code>/dh</i> <i>dh/dt</i> + <i>d<code>&rho;</code>/dp</i> <i>dp/dt</i> + 
<i>d<code>&rho;</code>/dxi</i> <i>dxi/dt</i> </p>
<p>where <i>h</i> is the specific enthalpy, <i>p</i> is the pressure, and 
<i>xi</i> is the mass fraction of the condensing component in the gas cell.</p>
<p align=\"center\"> <i>V <code>&rho;</code></i>  <i>dxi/dt</i> = <i>
<code>&#7745;</code></i><sub>gas,in</sub> (<i>xi</i><sub>gas,in</sub> - 
<i>xi</i>) - <i><code>&#7745;</code></i><sub>gas,out</sub> (<i>xi</i>
<sub>gas,out</sub> - <i>xi</i>) + <i><code>&#7745;</code></i><sub>vle</sub> 
(1 - <i>xi</i>) </p>    
<p>Energy balance: </p>
<p align=\"center\"> <i>V <code>&rho;</code></i>  <i>dh/dt</i> = 
<i><code>&#7745;</code></i><sub>gas,in</sub> (<i>h</i><sub>gas,in</sub> - 
<i>h</i>) - <i><code>&#7745;</code></i><sub>gas,out</sub> (<i>h</i>
<sub>gas,out</sub> - <i>h</i>) + <i><code>&#7745;</code></i><sub>vle</sub> 
(<i>h</i><sub>vle</sub> - <i>h</i>) + <i>V</i> <i>dp/dt</i> + <code>&sum;
</code> <i>Q</i> </p>
<p>where <i>h</i><sub>gas,in</sub>, <i>h</i><sub>gas,out</sub>, and <i>h</i>
<sub>vle</sub> are the specific enthalpies of incoming/outflowing streams. </p>
The pressure at the vle port is set equal to the partial pressure of the 
condensing component in the gas mixture. To consider different reference states
fot the specific enthalpies of gas and vle models the specific enthalpies are 
converted into each other using temperature and pressure.
</p>
<h4>Assumptions and limitations</h4>
<p>
The model uses the following main assumptions:
<ul>
  <li>The gas / gas mixtrue within the cell is ideally mixed with a uniform 
  enthalpy, pressure and composition.</li>
  <li>For the calculations of the gas properties, the ideal gas law 
  is assumed.</li>
  <li>For the simple pressure drop models, such as zero pressure drop and 
  constant pressure drop, there is no correlation between mass flow rate and 
  pressure drop. Thus, a mass flow rate has to be given by a connected 
  model, e.g. by a mass flow boundary.</li>
</ul>
</p>
<h4>Typical use and important parameters</h4>
<p>
For open adsorption systems the gas cell represents the moist air.
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
The model has three states: the specific enthalpy <i>h</i>, 
the pressure <i>p</i> and the mass fraction of the condensing 
component <i>xi</i>.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialGasCell;
