within SorpLib.Components.Cells.BaseClasses;
partial model PartialSingleWPairCell
  "Base model for all working pair cells describing single-component adsorption"
  extends PartialWorkingPairCell(
    redeclare final model WorkingPair = SingleWPair);

  //
  // Definition of general parameters
  //
  parameter Integer nPorts_B=0
    "Number of ports B (e.g., 2 ports are required for MR in closed systems)"
    annotation (Dialog(connectorSizing=true),
                HideResult=true);

  replaceable model SingleWPair =
    Media.WorkingPair.SingleComponent.BaseClasses.PartialSingleWPair
    constrainedby
    Media.WorkingPair.SingleComponent.BaseClasses.PartialSingleWPair(
      final T_adsorpt=T,
      final p_adsorpt=p,
      final x_adsorpt=x)
    "Working pair model"
    annotation (Dialog(tab="General",group="Working Pair"),
                choicesAllMatching = true,
                Evaluate=true,
                HideResult=true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Real xInitial(unit="kg/kg") = 0.2
    "Initial value for equilibrium loading"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                HideResult=true);

  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Start value for mass flow rate"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);
  parameter Modelica.SIunits.Pressure pStart = 100
    "Start value for equilibrium pressure"
    annotation (Dialog(group = "Start Values", tab = "Initialisation"),
                HideResult=true);

  //
  // Definition of outputs
  //
  output Modelica.SIunits.Pressure p(
    start = pStart)
    "Equilibrium pressure of adsorbate"
    annotation (Dialog(tab="General",group="Outputs",enable=false));
  output Real x(
    unit="kg/kg",
    start=xInitial)
    "Equilibrium loading of adsorbate"
    annotation (Dialog(tab="General",group="Outputs",enable=false));

  //
  // Definition of ports
  //
  replaceable Connectors.BaseClasses.PartialMediaConnector port_a(
    final no_components=no_components,
    m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-80,-10},{-60,10}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector[nPorts_B] port_b(
    each final no_components=no_components,
    each m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{42,-10},{62,10}})));

initial equation
  if type_massBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    x = xInitial
      "Equilibrium loading of adsorbate";

  elseif type_massBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(x) = 0
      "Equilibrium loading of adsorbate";

  end if;

equation
  //
  // Momentum balance
  //
  port_a.p = p
    "Homogeneous working pair cell";
  for i in 1:nPorts_B loop
    port_b[i].p = p
      "Homogeneous working pair cell";
  end for;

  //
  // Energy balance
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if avoid_events then
      0 = port_a.m_flow*noEvent(actualStream(port_a.h_outflow)) +
        sum(port_b.m_flow.*noEvent(actualStream(port_b.h_outflow))) +
        heatPort_a.Q_flow + heatPort_b.Q_flow
        "Steady-state energy balance assuming adsorbate = sorbens + adsorpt";

    else
      0 = port_a.m_flow*actualStream(port_a.h_outflow) +
        sum(port_b.m_flow.*actualStream(port_b.h_outflow)) +
        heatPort_a.Q_flow + heatPort_b.Q_flow
        "Steady-state energy balance assuming adsorbate = sorbens + adsorpt";

    end if;

  else
    if avoid_events then
      sum(workingPair.du_avg_dT_x)*der(T) =
        (port_a.m_flow*noEvent(actualStream(port_a.h_outflow)) +
        sum(port_b.m_flow.*noEvent(actualStream(port_b.h_outflow))) +
        heatPort_a.Q_flow + heatPort_b.Q_flow) / m_sorbent -
        sum(workingPair.du_avg_dx_T)*der(x)
        "Transient energy balance assuming adsorbate = sorbens + adsorpt";

    else
      sum(workingPair.du_avg_dT_x)*der(T) =
        (port_a.m_flow*actualStream(port_a.h_outflow) +
        sum(port_b.m_flow.*actualStream(port_b.h_outflow)) +
        heatPort_a.Q_flow + heatPort_b.Q_flow) / m_sorbent -
        sum(workingPair.du_avg_dx_T)*der(x)
        "Transient energy balance assuming adsorbate = sorbens + adsorpt";

    end if;

  end if;

  port_a.h_outflow = workingPair.h_sorptive
    "Homogeneous working pair cell";
  for i in 1:nPorts_B loop
    port_b[i].h_outflow = workingPair.h_sorptive
      "Homogeneous working pair cell";
  end for;

  //
  // Mass balance
  //
  if type_massBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    0 = 1/m_sorbent * (port_a.m_flow + sum(port_b[:].m_flow))
      "Steady-state mass balance";

  else
    der(x) = 1/m_sorbent * (port_a.m_flow + sum(port_b[:].m_flow))
      "Transient mass balance with loading as differential state";
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  Major revision (e.g., object-oriented approach, initialization, working pair) after restructering of the library.
  </li>
  <li>
  November 18, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>This model is a cell model combining adsorbent and adsorbate. This cell model consists of an energy and a mass balance, combined with equilibrium data for the adsorbent/adsorbate working pair (<a href=\"modelica://SorpLib.Media.Functions.CharCurve1Arctan\">SorpLib.Media.AdsorbentAdsorbate.Partial.PartialAdsorbentWater</a>). </p>
<p>For a cell, a lumped model approach is used with equal temperature of adsorbent/adsorbate. By combining more cells with a finite volume approach, a distributed adsorbent model can be built, with a spatial loading and temperature distribution. To create a full adsorber, this adsorbent cell model can be connected to some kind of heat exchanger model. </p>

  <h4>Main equations</h4>
<p>The main equations are an energy and a mass balance.</p>
<p>Mass balance: </p>
<p align=\"center\"><i>dx/dt</i> = 1 / <i>m</i><sub>sor</sub> * (<i><code>&#7745;</code></i><sub>ad,in</sub> - <i><code>&#7745;</code></i><sub>ad,out</sub>) </p>
<p>where <i>x</i> is the loading of the adsorbent defined as <i>x = m</i><sub>ad</sub> / <i>m</i><sub>sor</sub>. <i>m</i><sub>sor</sub> is the mass of dry adsorbent and <i>m</i><sub>ad</sub> is the mass of adsorbate. </p>
<p>Energy balance: </p>
<p align=\"center\">(<i>c</i><sub>sor</sub> + <i>x</i> <i>c</i><sub>ad</sub>) <i>dT/dt</i> + <i>u</i><sub>ad</sub> <i>dx/dt</i> = 1 / <i>m</i><sub>sor</sub> * (<i><code>&#7745;</code></i><sub>ad,in</sub> * <i>h</i><sub>in</sub> - <i><code>&#7745;</code></i><sub>ad,out</sub> * <i>h</i><sub>out</sub> + <code>&sum;</code> <i>Q</i> ) </p>
<p>where <i>c</i> is the specific heat capacity, <i>u</i><sub>ad</sub> is the specific internal energy of the adsorbate at the current loading and temperature, <i>h</i> is the specific enthalpy of the incoming and outflowing adsorbate stream, and <i>Q</i> is the heat flow.</p>
<p>The specific enthalpy of the incoming adsorbate stream is determined by the component, where the adsorbate is coming from (normally the evaporator). The specific enthalpy of the outflowing stream is set the saturation enthalpy of adsorbate in vapour state. </p>

  <h4>Assumptions and limitations</h4>
<p>The model uses the following main assumptions: </p>
<ul>
<li>All adsorbate is in adsorbed state. </li>
<li>Adsorbent and adsorbate have the same temperature. </li>
<li>Enthalpy of outflowing stream is set the saturation enthalpy of adsorbate in vapour state</li>
<li>Note: the pressure at the VLE ports is set to the equilibrium pressure of the equilibrium model. --&GT; A mass transfer model has to be connected to the adsorbent model. </li>
</ul>
<h4>Typical use and important parameters</h4>
<p>The adsorbent cell model is at the heart of all adsorption models: closed adsorber (adsorption heat pumps and chiller) and open adsorber (desiccant). </p>

<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>The adsorbent model has two differential states: the loading <i>x</i> and the temperature <i>T</i>. </p>

<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<ul>
<li>Lanzerath, F.; Bau, U.; Seiler, J.; Bardow, A. Optimal design of adsorption chillers based on a validated dynamic object-oriented model. Science and Technology for the Built Environment, 2015, 21(3), 248-257. </li>
</ul>
</html>"));
end PartialSingleWPairCell;
