within SorpLib.Components.Cells.BaseClasses;
partial model PartialWallCell "Base model of a wall cells"

  //
  // Definition of general parameters
  //
  replaceable parameter Geometry.WallCell
    cellGeometry constrainedby Geometry.WallCell
    "Cell geometry"
    annotation (Dialog(tab="General", group="Geometry"),
                choicesAllMatching = true);

  parameter Internals.Types.BalancingWallElement balancingWallElement=
   Internals.Types.BalancingWallElement.NS
   "Balancing method for wall cell"
    annotation (Dialog(tab = "General", group = "Heat Transfer"),
                Evaluate=true,
                HideResult = true);
  replaceable model HeatTransferModel_NS =
    SorpLib.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaCircular_GeometryBased
    constrainedby
    SorpLib.Components.HeatTransfers.BaseClasses.PartialInternalWallHeatTransfer
    "Heat transfer model for north-south direction (e.g., perpendicular to flow 
     direction of a 1D tube)"
    annotation (Dialog(tab = "General", group = "Heat Transfer"),
                choices(choice(redeclare model HeatTransferModel_NS =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.ConstantR
                         "Wall cell | Constant heat transfer resistance of cell"),
                       choice(redeclare model HeatTransferModel_NS =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaPlain_GeometryBased
                         "Wall cell | Geometry-based heat transfer coefficient for plain heat flow"),
                       choice(redeclare model HeatTransferModel_NS =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaCircular_GeometryBased
                         "Wall cell | Geometry-based heat transfer coefficient for circular heat flow")),
                Evaluate=true,
                HideResult = true);
  replaceable model HeatTransferModel_WE =
    SorpLib.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaPlain_GeometryBased
    constrainedby
    SorpLib.Components.HeatTransfers.BaseClasses.PartialInternalWallHeatTransfer
    "Heat transder model for west-east direction (e.g., in flow direction of a
     1D tube)"
    annotation (Dialog(tab = "General", group = "Heat Transfer"),
                choices(choice(redeclare model HeatTransferModel_WE =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.ConstantR
                         "Wall cell | Constant heat transfer resistance of cell"),
                       choice(redeclare model HeatTransferModel_WE =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaPlain_GeometryBased
                         "Wall cell | Geometry-based heat transfer coefficient for plain heat flow"),
                       choice(redeclare model HeatTransferModel_WE =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.Wall.LambdaCircular_GeometryBased
                         "Wall cell | Geometry-based heat transfer coefficient for circular heat flow")),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Modelica.SIunits.Temperature TInitial = 293.15
    "Initial value for temperature"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                HideResult = true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Internals.Types.BalanceEquations type_energyBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the energy balance and corresponding initialisation"
    annotation (Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  //
  // Instantiation of models
  //
  HeatTransferModel_NS heatTransferModel_NS(
    final cellGeometry=cellGeometry,
    final cellProperties=cellProperties)
    "Heat transfer model for heat transfer between wall and heat transfer fluid"
    annotation(Placement(transformation(extent={{20,40},{40,60}},rotation=0)));

  HeatTransferModel_WE heatTransferModel_WE(
    final cellGeometry=cellGeometry,
    final cellProperties=cellProperties)
    "Heat transfer model for heat transfer between wall and heat transfer fluid"
    annotation(Placement(transformation(extent={{60,20},{80,40}},rotation=0)));

  //
  // Definition of variables
  //
  Modelica.SIunits.Temperature T(
    start = TInitial)
    "Temperature of wall cell";
  Modelica.SIunits.Density rho
    "Density of wall cell";

  Modelica.SIunits.SpecificHeatCapacity c
    "Specific heat capacity of wall cell";

  Modelica.SIunits.ThermalConductivity lambda
    "Thermal conductivity of wall cell";

  Modelica.SIunits.Mass mass = rho*cellGeometry.volume
    "Mass of wall cell";

  PropertyRecord.WallCellProperties cellProperties(
    final T=T,
    final rho=rho,
    final c=c,
    final lambda=lambda)
    "Wall property record for calculation of transport phenomena" annotation (
      Placement(transformation(extent={{-100,-100},{-80,-80}}, rotation=0)),
      HideResult=true);

  //
  // Definition of connectors
  //
  Connectors.Thermal.Port_b heatPort_N
    "Heat port at north"
    annotation (Placement(transformation(extent={{-10,90},{10,110}})));
  Connectors.Thermal.Port_b heatPort_E
    "Heat port at east"
    annotation (Placement(transformation(extent={{90,-10},{110,10}})));
  Connectors.Thermal.Port_a heatPort_S
    "Heat port at south"
    annotation (Placement(transformation(extent={{-10,-110},{10,-90}})));
  Connectors.Thermal.Port_a heatPort_W
    "Heat port at west"
    annotation (Placement(transformation(extent={{-110,-10},{-90,10}})));

initial equation
  if type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    T = TInitial
      "Fixed initial value";

  elseif type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(T) = 0
      "Steady-state initialisation of energy balance";

  end if;

equation
  //
  // Energy balance
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    0 = heatPort_N.Q_flow + heatPort_E.Q_flow +
      heatPort_S.Q_flow + heatPort_W.Q_flow
      "Steady-state energy balance of wall cell";

  else
    mass*c*der(T) = heatPort_N.Q_flow + heatPort_E.Q_flow +
      heatPort_S.Q_flow + heatPort_W.Q_flow
      "Transient energy balance of wall cell";

  end if;

  //
  // Calculation of heat flows
  //
  if balancingWallElement == Internals.Types.BalancingWallElement.NoResistances then
    //
    // No heat resistances: Wall cell has homogeneous temperature
    //
    heatPort_N.T = T;
    heatPort_E.T = T;
    heatPort_S.T = T;
    heatPort_W.T = T;

  elseif balancingWallElement == Internals.Types.BalancingWallElement.NS then
    //
    // Only heat resistance for north-south direction
    //
    heatPort_N.Q_flow = (heatPort_N.T-T)*heatTransferModel_NS.k_CenterPort;
    heatPort_E.T = T;
    heatPort_S.Q_flow = (heatPort_S.T-T)*heatTransferModel_NS.k_PortCenter;
    heatPort_W.T = T;

  elseif balancingWallElement == Internals.Types.BalancingWallElement.WE then
    //
    // Only heat resistance for west-east direction
    //
    heatPort_N.T = T;
    heatPort_E.Q_flow = (heatPort_E.T-T)*heatTransferModel_WE.k_CenterPort;
    heatPort_S.T = T;
    heatPort_W.Q_flow = (heatPort_W.T-T)*heatTransferModel_WE.k_PortCenter;

  elseif balancingWallElement == Internals.Types.BalancingWallElement.NSandWE then
    //
    // No heat resistances: Wall cell has homogeneous temperature
    //
    heatPort_N.Q_flow = (heatPort_N.T-T)*heatTransferModel_NS.k_CenterPort;
    heatPort_E.Q_flow = (heatPort_E.T-T)*heatTransferModel_WE.k_CenterPort;
    heatPort_S.Q_flow = (heatPort_S.T-T)*heatTransferModel_NS.k_PortCenter;
    heatPort_W.Q_flow = (heatPort_W.T-T)*heatTransferModel_WE.k_PortCenter;

  end if;

  //
  // Annotations
  //
  annotation (Diagram(graphics={
        Line(
          points={{-20,-80},{-20,80}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled}),
        Line(
          points={{-80,-20},{80,-20}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{-10,-30},{70,-50}},
          lineColor={0,0,0},
          textString="Design direction"),
        Text(
          extent={{-40,10},{40,-10}},
          lineColor={0,0,0},
          textString="Design direction",
          origin={-40,30},
          rotation=90)}), Icon(graphics={Rectangle(
          extent={{-100,100},{100,-100}},
          lineColor={0,0,0},
          fillColor={255,255,255},
          fillPattern=FillPattern.Forward)}),
    Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  Revisions (e.g., object-oriented approach) after restructering of the library.
  </li>
  <li>
  November 23, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
  <p>
    This model represents a wall cell. For a cell, a lumped model approach is used with an uniform Temperatur. By combining more cells with a finite volume approach, a distributed model can be built, with a spatial and temperature distribution. The model has four heat ports to exchange heat with connected components. <br>
    
  </p>
  <h4>Main equations</h4>
  <p>    
    <p>Energy balance: </p>
    <p align=\"center\"> <i>m</i> <i>c</i> <i>dT/dt</i> = <code>&sum;</code> <i>Q</i> </p>
    <p>where <i>m</i> is the mass, <i>c</i> is the specific heat capacity, <i>T</i> is the temperature and <i>Q</i> is the heat flow. </p>
    The temperature at each heat port is set equal to the wall temperature <i>T</i>.
  </p>

  <h4>Assumptions and limitations</h4>

  <p>
    The model uses the following main assumptions:
    <ul>
      <li>The wall cell has a uniform temperature --> heat conduction is not considered</li>
    </ul>
  </p>
<h4>Typical use and important parameters</h4>
  <p>
    In heat exchangers the wall cell represents the thermal capacity of the metal mass. 
  </p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
  <p>
    The model has one state: the temperature <i>T</i>.
  </p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialWallCell;
