within SorpLib.Components.Cells.BaseClasses;
partial model PartialWorkingPairCell "Base model for all working pair cells"

  //
  // Definition of general parameters
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  parameter Modelica.SIunits.Mass m_sorbent = 2.5
    "Mass of dry sorbent"
    annotation (Dialog(tab="General",group="Sorbent"),
                HideResult=true);

  replaceable model WorkingPair =
    Media.WorkingPair.BaseClasses.PartialWorkingPair
    constrainedby Media.WorkingPair.BaseClasses.PartialWorkingPair
    "Working pair model"
    annotation (Dialog(tab="General",group="Working Pair"),
                choicesAllMatching = true,
                Evaluate=true,
                HideResult=true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Modelica.SIunits.Temperature TInitial = 298.15
    "Initial value for equilibrium temperature"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                HideResult=true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Internals.Types.BalanceEquations type_massBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the mass equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.BalanceEquations type_energyBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the energy equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Instantiation of models
  //
public
  WorkingPair workingPair
    "Working pair model"
    annotation (Placement(transformation(extent={{-10,-30},{10,-10}})));

  //
  // Definition of connectors
  //
  Connectors.Thermal.Port_a heatPort_a
    "Heat port A"
    annotation (Placement(transformation(extent={{-10,-70},{10,-50}})));
  Connectors.Thermal.Port_b heatPort_b
    "Heat port B"
    annotation (Placement(transformation(extent={{-10,50},{10,70}})));

  //
  // Definition of outputs
  //
  output Modelica.SIunits.Temperature T(
    start=TInitial)
    "Equilibrium temperature of adsorbate"
    annotation (Dialog(tab="General",group="Outputs",enable=false));

initial equation
  if type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    T = TInitial
      "Equilibrium temperature of adsorbate";

  elseif type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(T) = 0
      "Equilibrium temperature of adsorbate";

  end if;

equation
  //
  // Heat ports
  //
  heatPort_a.T = T
    "Temperature of adsorbate: Homogenoues working pair cell";
  heatPort_b.T = T
    "Temperature of adsorbate: Homogenoues working pair cell";

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
        Ellipse(
          extent={{-100,100},{-80,80}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-100,-20},{-80,-40}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-40,-80},{-20,-100}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{80,-80},{100,-100}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{20,100},{40,80}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-40,40},{-20,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{20,-20},{40,-40}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{80,40},{100,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Line(
          points={{-90,90},{30,90},{90,30},{90,-90},{30,-30},{-90,-30},{-90,90},
              {-30,30},{-30,-90},{90,-90}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-30,30},{92,30}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-92,-30}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-90,-30},{-30,-90}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{30,90},{30,-30}},
          color={0,0,0},
          thickness=0.5)}),                                      Diagram(
        coordinateSystem(preserveAspectRatio=false), graphics={Line(
          points={{-20,20},{-20,-20}},
          color={0,0,0},
          arrow={Arrow.Open,Arrow.Open}), Text(
          extent={{-20,10},{20,-10}},
          lineColor={0,0,0},
          textString="Equilibrium",
          origin={-30,0},
          rotation=90)}),
    Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialWorkingPairCell;
