within SorpLib.Components.Cells.ModelicaMediaLibrary;
model CondensingGasCell
  "Model of a homogenous gas cell with one adsorptive port; adsorptiv can condense"
  extends BaseClasses.PartialGasCell(
    final type_Media=Internals.Types.MediaModel.Modelica_IG,
    final neglectTermVp = false,
    state_phxi = false,
    redeclare final Connectors.ModelicaMediaLibrary.GasPort_a port_a(
      redeclare final package Medium = MediumGas),
    redeclare final Connectors.ModelicaMediaLibrary.GasPort_b port_b(
      redeclare final package Medium = MediumGas),
    redeclare final Connectors.ModelicaMediaLibrary.VLEPort_a port_adsorptiv(
      redeclare final package Medium = MediumVLE),
    type_independentMassBalances=
      Internals.Types.BalanceEquations.Transient_FixedInitial,
    rhoStart= 1.3);

  //
  // Definition of parameters
  //
  replaceable package MediumGas = Modelica.Media.Air.MoistAir
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Gas model (i.e., ideal gas mixture)"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  replaceable package MediumVLE = Modelica.Media.Water.WaterIF97_ph
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "VLE model (i.e., adsorptiv)"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then MediumGas.thermalConductivity(
        gas_phxi) else -1,
    final eta=if calc_propertyRecord then MediumGas.dynamicViscosity(gas_phxi)
         else -1,
    final Pr=if calc_propertyRecord then MediumGas.prandtlNumber(gas_phxi)
         else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  MediumGas.ThermodynamicState gas_phxi = if state_phxi then
    MediumGas.setState_phX(p, h, xi) else MediumGas.setState_pTX(p, T, xi)
    "Gas media model with independent states p, h or T, and xi"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

protected
  MediumGas.ThermodynamicState gas_pT = MediumGas.setState_pTX(p_adsorptiv,
    MediumVLE.temperature(VLE_ph), 0.9999999999*xi_adsorptiv)
    "Conversion of enthalpy from VLE to ideal gas: Required via desorption"
    annotation (Hide=true,
      Placement(transformation(extent={{20,-40},{40,-20}})));

   MediumVLE.ThermodynamicState VLE_ph=
    MediumVLE.setState_phX(p_adsorptiv, h_adsorptiv_vle_in, {1})
    "Calculation of temperature of instreaming adsorptiv: Required via 
     desorption"
    annotation (Hide=true,
      Placement(transformation(extent={{20,-70},{40,-50}})));

   MediumVLE.ThermodynamicState VLE_pT=
    MediumVLE.setState_pTX(p_adsorptiv, MediumGas.temperature(gas_phxi), {1})
    "Conversion of enthalpy from ideal gas to VLE: Required via adsorption"
    annotation (Hide=true,
      Placement(transformation(extent={{-40,-70},{-20,-50}})));

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance is only allowed in combination with
       steady-state mass balance!");
   end if;

  //
  // Passing state variables
  //
  rho = MediumGas.density(gas_phxi)
    "Density of fluid in cell";
  if state_phxi then
    T = MediumGas.temperature(gas_phxi)
      "Temperature of fluid in cell";
  else
    h = MediumGas.specificEnthalpy(gas_phxi)
      "Specific enthalpy of fluid in cell";
  end if;

  p_adsorptiv = MediumGas.saturationPressure(T) *
    MediumGas.relativeHumidity(gas_phxi)
    "Partial pressure of adsorptive in cell";

  h_adsorptiv_gas_out = MediumGas.enthalpyOfCondensingGas(T)
    "Specific enthalpy of adsorptive in cell according to media model of ideal
     gas mixture";
  h_adsorptiv_gas_in = MediumGas.enthalpyOfCondensingGas(
    MediumGas.temperature(gas_pT))
    "Specific enthalpy of adsorptive entering cell according to media model of 
     ideal gas mixture";
  h_adsorptiv_vle_in = inStream(port_adsorptiv.h_outflow)
    "Specific enthalpy of adsorptive entering cell according to VLE media 
     model";
  h_adsorptiv_vle_out = MediumVLE.specificEnthalpy(VLE_pT)
    "Specific enthalpy of adsorptive in cell according to VLE media model";

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = der(mass)
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = der(mass*h)
        "Derivative of enthalpy of fluid in cell wrt time";

    else
      dU_dtau = der(U)
        "Derivative of enthalpy of fluid in cell wrt time";
    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"), Diagram(graphics={
        Line(
          points={{-20,0},{-30,0},{-30,-44}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{-30,-74},{-30,-80},{-4,-80},{-4,-74}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{4,-74},{4,-80},{30,-80},{30,-74}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{44,-60},{50,-60},{50,-30},{44,-30}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{18,-30},{0,-30},{0,-20},{0,-14}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Text(
          extent={{-42,0},{-32,-44}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="Use
p_i
and
T"),    Text(
          extent={{-30,-82},{-4,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="to calculate h"),
        Text(
          extent={{4,-82},{30,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="Use p_i and h_i"),
        Text(
          extent={{52,-30},{80,-60}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="to
calculate
T")}));
end CondensingGasCell;
