within SorpLib.Components.Cells.ModelicaMediaLibrary;
model LiquidCell
  "Model of a homogenous liquid cell"
  extends BaseClasses.PartialLiquidVLECell(
    p(stateSelect=if neglectDerivativeP then StateSelect.default else
      StateSelect.prefer),
    state_phxi=false,
    final type_Media=Internals.Types.MediaModel.Modelica_IL,
    final type_independentMassBalances=
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
    redeclare final Connectors.ModelicaMediaLibrary.LiquidPort_a port_a(
      redeclare final package Medium = Medium),
    redeclare final Connectors.ModelicaMediaLibrary.LiquidPort_b port_b(
      redeclare final package Medium = Medium));

  //
  // Definition of parameters
  //
  replaceable package Medium = Modelica.Media.Water.WaterIF97_R1pT
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Liquid media"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  parameter Boolean neglectDerivativeP = false
    "= true, if derivative der(p) is neglectected, conservation equations are
    explicitly written for incompressibles fluids, ands states are p, T, and xi"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations",
                      enable = use_incompressibleEquations and not state_phxi),
                Evaluate=true,
                HideResult = true);
  parameter Boolean use_incompressibleEquations = false
    "= true, if conservation equations are explicitly written for
    incompressible fluids if states are p, T, and xi"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations",
                      enable = not state_phxi),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then Medium.thermalConductivity(
        stateProperties) else -1,
    final eta=if calc_propertyRecord then Medium.dynamicViscosity(
        stateProperties) else -1,
    final Pr=if calc_propertyRecord then Medium.prandtlNumber(stateProperties)
         else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  Medium.ThermodynamicState stateProperties = if state_phxi then
    Medium.setState_phX(p,h, xi) else Medium.setState_pTX(p, T, xi)
    "Record describing current thermodynamic state of homogenous volume"
    annotation (HideResult = true);

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance for incompressible liquid fluids is only
       reasonable if also using steady-state mass balance!");
  end if;

  //
  // Passing state variables
  //
  rho = Medium.density(stateProperties)
    "Density of fluid in cell";

  if state_phxi then
    T = Medium.temperature(stateProperties)
      "Temperature of fluid in cell";
  else
    h = Medium.specificEnthalpy(stateProperties)
      "Specific enthalpy of fluid in cell";
  end if;

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = if (not use_incompressibleEquations and not state_phxi) then
      der(mass) else -rho*cellGeometry.volume *
      Medium.isobaricExpansionCoefficient(stateProperties) * der(T)
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = if (not use_incompressibleEquations and not state_phxi) then
        der(mass*h) else (if neglectDerivativeP then rho*cellGeometry.volume *
        Medium.specificHeatCapacityCp(stateProperties)*der(T) +
        u*dmass_dtau else rho*cellGeometry.volume *
        Medium.specificHeatCapacityCp(stateProperties)*der(T) +
        cellGeometry.volume*(1 -
        Medium.isobaricExpansionCoefficient(stateProperties)*T)*der(p) +
        u*dmass_dtau)
        "Derivative of enthalpy of fluid in cell wrt time";

    else
      dU_dtau = if (not use_incompressibleEquations and not state_phxi) then
        der(U) else (if neglectDerivativeP then cellGeometry.volume*(rho *
        Medium.specificHeatCapacityCp(stateProperties)-p*
        Medium.isobaricExpansionCoefficient(stateProperties))*der(T) +
        u*dmass_dtau else
        cellGeometry.volume*(rho*Medium.specificHeatCapacityCp(stateProperties) -
        p*Medium.isobaricExpansionCoefficient(stateProperties))*der(T) -
        cellGeometry.volume *
        (Medium.isobaricExpansionCoefficient(stateProperties)*T)*der(p) +
        u*dmass_dtau)
        "Derivative of enthalpy of fluid in cell wrt time";

    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end LiquidCell;
