within SorpLib.Components.Cells.ModelicaMediaLibrary;
model VLECell "Model of a homogenous VLE cell"
  extends BaseClasses.PartialLiquidVLECell(
    final state_phxi=true,
    final type_Media=Internals.Types.MediaModel.Modelica_VLE,
    final type_independentMassBalances=
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
    redeclare final Connectors.ModelicaMediaLibrary.VLEPort_a port_a(
      redeclare final package Medium = Medium),
    redeclare final Connectors.ModelicaMediaLibrary.VLEPort_b port_b(
      redeclare final package Medium = Medium));

  //
  // Definition of parameters
  //
  replaceable package Medium = Modelica.Media.Water.WaterIF97_ph
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "VLE media"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then Medium.thermalConductivity(
        stateProperties) else -1,
    final eta=if calc_propertyRecord then Medium.dynamicViscosity(
        stateProperties) else -1,
    final Pr=if calc_propertyRecord then Medium.prandtlNumber(stateProperties)
         else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  Medium.ThermodynamicState stateProperties=
    Medium.setState_phX(p, h, xi)
    "Record describing current thermodynamic state of homogenous volume"
    annotation (HideResult = true);

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance is only allowed in combination with
       steady-state mass balance!");
   end if;

  //
  // Passing state variables
  //
  T = Medium.temperature(stateProperties)
    "Temperature of fluid in cell";
  rho = Medium.density(stateProperties)
    "Density of fluid in cell";

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = der(mass)
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = der(mass*h)
        "Derivative of enthalpy of fluid in cell wrt time";
    else
      dU_dtau = der(U)
        "Derivative of enthalpy of fluid in cell wrt time";
    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end VLECell;
