within SorpLib.Components.Cells.TILMediaLibrary;
model CondensingGasCell
  "Model of a homogenous gas cell with one adsorptive port; adsorptiv can condense"
  extends BaseClasses.PartialGasCell(
    final type_Media=Internals.Types.MediaModel.TIL_IG,
    final state_phxi = true,
    redeclare final Connectors.TILMediaLibrary.GasPort_a port_a(
      final gasType=gasType),
    redeclare final Connectors.TILMediaLibrary.GasPort_b port_b(
      final gasType=gasType),
    redeclare final Connectors.TILMediaLibrary.VLEPort_a port_adsorptiv(
      final vleFluidType=vleFluidType),
    type_independentMassBalances=
      Internals.Types.BalanceEquations.Transient_FixedInitial,
    rhoStart= 1.3);

  //
  // Definition of parameters
  //
  replaceable parameter TILMedia.GasTypes.TILMedia_MoistAir gasType
    constrainedby TILMedia.GasTypes.BaseGas
    "Gas model (i.e., ideal gas mixture)"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  replaceable parameter TILMedia.VLEFluidTypes.TILMedia_Water vleFluidType
    constrainedby TILMedia.VLEFluidTypes.BaseVLEFluid
    "VLE model (i.e., adsorptiv)"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  parameter Boolean use_explicitlyWrittenDerivitives = true
    "= true, if conservation equations are explicitly written"
    annotation (Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then gas_phxi.transp.lambda else -1,
    final eta=if calc_propertyRecord then gas_phxi.transp.eta else -1,
    final Pr=if calc_propertyRecord then gas_phxi.transp.Pr else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  TILMedia.Gas_ph gas_phxi(
    final gasType=gasType,
    final computeTransportProperties=calc_propertyRecord,
    final p=p,
    final h=h,
    final xi=xi)
    "Gas media model with independent states p, h, and xi"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

protected
   TILMedia.Gas_pT gas_pT(
    final gasType=gasType,
    final p=p_adsorptiv,
    final T=VLE_ph.T,
    final xi=0.9999999999*xi_adsorptiv)
    "Conversion of enthalpy from VLE to ideal gas: Required via desorption"
    annotation (Hide=true,
      Placement(transformation(extent={{20,-40},{40,-20}})));

   TILMedia.VLEFluid_ph VLE_ph(
    final vleFluidType=vleFluidType,
    final p=p_adsorptiv,
    final h=h_adsorptiv_vle_in)
    "Calculation of temperature of instreaming adsorptiv: Required via 
     desorption"
    annotation (Hide=true,
      Placement(transformation(extent={{20,-70},{40,-50}})));

   TILMedia.VLEFluid_pT VLE_pT(
    final vleFluidType=vleFluidType,
    final p=p_adsorptiv,
    final T=gas_phxi.T)
    "Conversion of enthalpy from ideal gas to VLE: Required via adsorption"
    annotation (Hide=true,
      Placement(transformation(extent={{-40,-70},{-20,-50}})));

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance is only allowed in combination with
       steady-state mass balance!");
   end if;

  //
  // Passing state variables
  //
  rho = gas_phxi.d
    "Density of fluid in cell";
  T = gas_phxi.T
    "Specific enthalpy of fluid in cell";

  p_adsorptiv = gas_phxi.p_i[index_adsorptiv]
    "Partial pressure of adsorptive in cell";

  h_adsorptiv_gas_out = gas_phxi.h_i[index_adsorptiv]
    "Specific enthalpy of adsorptive in cell according to media model of ideal
     gas mixture";
  h_adsorptiv_gas_in = gas_pT.h_i[index_adsorptiv]
    "Specific enthalpy of adsorptive entering cell according to media model of 
     ideal gas mixture";
  h_adsorptiv_vle_in = inStream(port_adsorptiv.h_outflow)
    "Specific enthalpy of adsorptive entering cell according to VLE media 
     model";
  h_adsorptiv_vle_out = VLE_pT.h
    "Specific enthalpy of adsorptive in cell according to VLE media model";

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = if not use_explicitlyWrittenDerivitives then der(mass) else
      cellGeometry.volume * (gas_phxi.drhodh_pxi*der(h) +
      gas_phxi.drhodp_hxi*der(p) + gas_phxi.drhodxi_ph*der(xi))
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = if not use_explicitlyWrittenDerivitives then der(mass*h) else
        mass*der(h) + h*dmass_dtau
        "Derivative of enthalpy of fluid in cell wrt time";

    else
      dU_dtau = if not use_explicitlyWrittenDerivitives then der(U) else
         mass*der(h) + h*dmass_dtau - cellGeometry.volume*der(p)
        "Derivative of enthalpy of fluid in cell wrt time";
    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"), Diagram(graphics={
        Line(
          points={{-20,0},{-30,0},{-30,-44}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{-30,-74},{-30,-80},{-4,-80},{-4,-74}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{4,-74},{4,-80},{30,-80},{30,-74}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{44,-60},{50,-60},{50,-30},{44,-30}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Line(
          points={{18,-30},{0,-30},{0,-20},{0,-14}},
          color={0,0,0},
          arrow={Arrow.None,Arrow.Filled},
          thickness=0.5),
        Text(
          extent={{-42,0},{-32,-44}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="Use
p_i
and
T"),    Text(
          extent={{-30,-82},{-4,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="to calculate h"),
        Text(
          extent={{4,-82},{30,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="Use p_i and h_i"),
        Text(
          extent={{52,-30},{80,-60}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="to
calculate
T")}));
end CondensingGasCell;
