within SorpLib.Components.Cells.TILMediaLibrary;
model LiquidCell
  "Model of a homogenous liquid cell"
  extends BaseClasses.PartialLiquidVLECell(
    p(stateSelect=if neglectDerivativeP then StateSelect.default else
      StateSelect.prefer),
    final state_phxi=false,
    final type_Media=Internals.Types.MediaModel.TIL_IL,
    final type_independentMassBalances=
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
    redeclare final Connectors.TILMediaLibrary.LiquidPort_a port_a(
      final liquidType=liquidType),
    redeclare final Connectors.TILMediaLibrary.LiquidPort_b port_b(
      final liquidType=liquidType));

  //
  // Definition of parameters
  //
  replaceable parameter TILMedia.LiquidTypes.TILMedia_Water liquidType
    constrainedby TILMedia.LiquidTypes.BaseLiquid
    "Medium modelLiquid media"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  parameter Boolean neglectDerivativeP = false
    "= true, if derivative der(p) is neglectected and states are p, T, and xi"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations",
                      enable = not state_phxi),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then liquid_pT.transp.lambda else -1,
    final eta=if calc_propertyRecord then liquid_pT.transp.eta else -1,
    final Pr=if calc_propertyRecord then liquid_pT.transp.Pr else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  TILMedia.Liquid_pT liquid_pT(
    final liquidType=liquidType,
    final computeTransportProperties=calc_propertyRecord,
    final p=p,
    final T=T,
    final xi=xi)
    "Liquid media model with independent states p, T, and xi"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance for incompressible liquid fluids is only
       reasonable if also using steady-state mass balance!");
  end if;

  //
  // Passing state variables
  //
  rho = liquid_pT.d
    "Density of fluid in cell";
  h = liquid_pT.h
    "Specific enthalpy of fluid in cell";

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = -rho*cellGeometry.volume*liquid_pT.beta * der(T)
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = if neglectDerivativeP then
        rho*cellGeometry.volume*liquid_pT.cp*der(T) + u*dmass_dtau else
        rho*cellGeometry.volume*liquid_pT.cp*der(T) +
        cellGeometry.volume*(1-liquid_pT.beta*T)*der(p) + u*dmass_dtau
        "Derivative of enthalpy of fluid in cell wrt time";

    else
      dU_dtau = if neglectDerivativeP then u*dmass_dtau +
        cellGeometry.volume*(rho*liquid_pT.cp-p*liquid_pT.beta)*der(T) else
        cellGeometry.volume*(rho*liquid_pT.cp-p*liquid_pT.beta)*der(T) -
         cellGeometry.volume*(liquid_pT.beta*T)*der(p) + u*dmass_dtau
        "Derivative of enthalpy of fluid in cell wrt time";

    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end LiquidCell;
