within SorpLib.Components.Cells.TILMediaLibrary;
model VLECell "Model of a homogenous VLE cell"
  extends BaseClasses.PartialLiquidVLECell(
    final state_phxi=true,
    final type_Media=Internals.Types.MediaModel.TIL_VLE,
    final type_independentMassBalances=
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
    redeclare final Connectors.TILMediaLibrary.VLEPort_a port_a(
      final vleFluidType=vleFluidType),
    redeclare final Connectors.TILMediaLibrary.VLEPort_b port_b(
      final vleFluidType=vleFluidType));

  //
  // Definition of parameters
  //
  replaceable parameter TILMedia.VLEFluidTypes.TILMedia_Water vleFluidType
    constrainedby TILMedia.VLEFluidTypes.BaseVLEFluid
    "VLE media"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true,
                Hide=true);

  parameter Boolean use_explicitlyWrittenDerivitives = false
    "= true, if conservation equations are explicitly written"
    annotation (Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of outputs
  //
  output PropertyRecord.FluidCellProperties fluidProperties(
    final T=T,
    final rho=rho,
    final lambda=if calc_propertyRecord then vleFluid_ph.transp.lambda else -1,
    final eta=if calc_propertyRecord then vleFluid_ph.transp.eta else -1,
    final Pr=if calc_propertyRecord then vleFluid_ph.transp.Pr else -1,
    final m_flow_portA=port_a.m_flow,
    final m_flow_portB=port_b.m_flow)
    "Record containing fluid properties required by models of transport phenomena"
    annotation (Dialog(
      tab="General",
      group="Outputs",
      enable=false));

  //
  // Definition of media models
  //
  TILMedia.VLEFluid_ph vleFluid_ph(
    final vleFluidType=vleFluidType,
    final computeTransportProperties=calc_propertyRecord,
    final p=p,
    final h=h,
    final xi=xi)
    "VLE media model calculate with independent states p, h, and xi"
    annotation (Placement(transformation(extent={{-10,-12},{10,8}})));

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance is only allowed in combination with
       steady-state mass balance!");
   end if;

  //
  // Passing state variables
  //
  T = vleFluid_ph.T
    "Temperature of fluid in cell";
  rho = vleFluid_ph.d
    "Density of fluid in cell";

  //
  // Mass balance
  //
  if type_overallMassBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    dmass_dtau = if not use_explicitlyWrittenDerivitives then der(mass) else
      cellGeometry.volume *(vleFluid_ph.drhodh_pxi*der(h) +
      vleFluid_ph.drhodp_hxi*der(p) + vleFluid_ph.drhodxi_ph*der(xi))
      "Derivative of overall mass wrt time";
  end if;

  //
  // Energy balance
  //
  if type_energyBalance <>
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if neglectTermVp then
      dU_dtau = if not use_explicitlyWrittenDerivitives then der(mass*h) else
        mass*der(h) + h*dmass_dtau
        "Derivative of enthalpy of fluid in cell wrt time";
    else
      dU_dtau = if not use_explicitlyWrittenDerivitives then der(U) else
        u*dmass_dtau + mass*(der(h) - (1/rho*der(p) -
        p/rho^2*(vleFluid_ph.drhodh_pxi*der(h) + vleFluid_ph.drhodp_hxi*der(p) +
        vleFluid_ph.drhodxi_ph*der(xi))))
        "Derivative of enthalpy of fluid in cell wrt time";
    end if;
  end if;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 12, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end VLECell;
