within SorpLib.Components.Connectors.Converter;
model Modelica_VLE__TIL_Liquid
  "Converter between fluid ports: Modelica VLE medium to TIL liquid medium"
  extends BaseClasses.PartialConverter(
    redeclare final ModelicaMediaLibrary.VLEPort_a port_mediaLibraryA(
      final no_components=no_components,
      redeclare final package Medium = Medium_LibraryA),
    redeclare final TILMediaLibrary.LiquidPort_a port_mediaLibraryB(
      final no_components=no_components,
      final liquidType=Medium_LibraryB));

  //
  // Definition of parameters
  //
  replaceable package Medium_LibraryA =
    Modelica.Media.Water.StandardWater
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Medium model A (i.e., Modelica VLE medium)"
    annotation (Dialog(tab="General", group="Medium"),
                evaluate=true,
                HideResult=true,
                choicesAllMatching=true);

  replaceable parameter TILMedia.LiquidTypes.TILMedia_Water Medium_LibraryB
    constrainedby TILMedia.LiquidTypes.BaseLiquid
    "Medium model B (i.e., TIL liquid medium)"
    annotation (Dialog(tab="General", group="Medium"),
                evaluate=true,
                HideResult=true,
                choicesAllMatching=true);

  //
  // Definition of media models
  //
  Medium_LibraryA.ThermodynamicState mediumA_ph = Medium_LibraryA.setState_phX(
    p=port_mediaLibraryA.p,
    h=inStream(port_mediaLibraryA.h_outflow),
    X=if no_components==1 then {1} else
      cat(1,
          inStream(port_mediaLibraryA.xi_outflow),
          {1-sum(inStream(port_mediaLibraryA.xi_outflow))}))
    "Thermodynamic state according to instreaming properties at port port_mediaLibraryA"
    annotation (Placement(transformation(extent={{-40,16},{-20,36}})));

  TILMedia.Liquid_pT mediumB_pT(
    final liquidType=Medium_LibraryB,
    final p=Medium_LibraryA.pressure(mediumA_ph),
    final T=Medium_LibraryA.temperature(mediumA_ph),
    final xi=inStream(port_mediaLibraryA.xi_outflow))
    "Thermodynamic state according to outstreaming properties at port port_mediaLibraryB"
    annotation (Placement(transformation(extent={{20,16},{40,36}})));

  TILMedia.Liquid_ph mediumB_ph(
    final liquidType=Medium_LibraryB,
    final p=port_mediaLibraryB.p,
    final h=inStream(port_mediaLibraryB.h_outflow),
    final xi=inStream(port_mediaLibraryB.xi_outflow))
    "Thermodynamic state according to instreaming properties at port port_mediaLibraryB"
    annotation (Placement(transformation(extent={{20,-36},{40,-16}})));

  Medium_LibraryA.ThermodynamicState mediumA_pT = Medium_LibraryA.setState_pTX(
    p=mediumB_ph.p,
    T=mediumB_ph.T,
    X=if no_components==1 then {1} else
      cat(1,
          mediumB_ph.xi,
          {1-sum(mediumB_ph.xi)}))
    "Thermodynamic state according to outstreaming properties at port port_mediaLibraryA"
    annotation (Placement(transformation(extent={{-40,-16},{-20,-36}})));

equation
  //
  // Energy balance
  //
  port_mediaLibraryA.h_outflow = Medium_LibraryA.specificEnthalpy(mediumA_pT)
    "No change in energy in converter model";
  port_mediaLibraryB.h_outflow =mediumB_pT.h
    "No change in energy in converter model";

  //
  // Annotations
  //
  annotation (Diagram(graphics={Text(
          extent={{-18,28},{18,24}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillPattern=FillPattern.Sphere,
          fillColor={228,228,228},
          textString="Use p and T"), Text(
          extent={{-18,-24},{18,-28}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillPattern=FillPattern.Sphere,
          fillColor={228,228,228},
          textString="Use p and T")}), Documentation(revisions="<html>
<ul>
  <li>
  January 11, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end Modelica_VLE__TIL_Liquid;
