within SorpLib.Components.Fittings.HydraulicResistors.BaseClasses;
partial model PartialHydraulicResistor
  "Base model for all hydraulic resistors"

  //
  // Definition of parameters describing the pressure drop model
  //
  replaceable model PressureDropModel =
    SorpLib.Components.Fittings.HydraulicResistors.BaseClasses.PartialPressureDrop
    constrainedby
    SorpLib.Components.Fittings.HydraulicResistors.BaseClasses.PartialPressureDrop
    "Pressure drop model"
    annotation (Dialog(tab="General", group="Pressure Drop Correlation"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing the medium
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);
  final parameter Boolean calc_propertyRecord=
    pressureDropModel.calc_propertyRecord
    "= true, if property record is calculated for models of transport phenomena"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.PositionProperties propertiesPosition=SorpLib.Internals.Types.PositionProperties.PortA
    "Tye defining calculation approach for fluid properties if needed"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Start value for mass flow rate"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);

  //
  // Definition of advanced parameters
  //
  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of inputs
  //
  input Cells.PropertyRecord.FluidCellProperties fluidProperties_portA
    "Record containing fluid properties at port A"
    annotation (Dialog(tab="General", group="Inputs"));

  input Cells.PropertyRecord.FluidCellProperties fluidProperties_portB
    "Record containing fluid properties at port B"
    annotation (Dialog(tab="General", group="Inputs"));

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Definition of variables
  //
public
  Modelica.SIunits.PressureDifference dp = pressureDropModel.pressureDrop
    "Pressure drop between port A and port B";

protected
  Cells.PropertyRecord.FluidCellProperties actualFluidProperties
    "Record containing actual fluid properties" annotation (HideResult=true);

  //
  // Instanziation of models
  //
public
  PressureDropModel pressureDropModel
    "Model to calculate the pressure drop"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

  //
  // Definition of connectors
  //
  replaceable Connectors.BaseClasses.PartialMediaConnector port_a(
    final no_components=no_components,
    m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-110,-10},{-90,10}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector port_b(
    final no_components=no_components,
    m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{88,-10},{108,10}})));

equation
  //
  // Assertations
  //
  assert(propertiesPosition <> Internals.Types.PositionProperties.Average,
    "Average fluid properties currently are not supported for hydraulic
     resistor!");

  //
  // Momentum balance
  //
  dp = port_a.p - port_b.p
    "Pressure drop between port A and port B";

  //
  // Mass balance
  //
  0 = port_a.m_flow + port_b.m_flow
    "Steady state mass balance";

  port_a.xi_outflow = inStream(port_b.xi_outflow)
    "Stream variable: Trivial equation since no change of mass fractions";
  port_b.xi_outflow = inStream(port_a.xi_outflow)
    "Stream variable: Trivial equation since no change of mass fractions";

  //
  // Energy balance
  //
  port_a.h_outflow = inStream(port_b.h_outflow)
    "Stream variable: Trivial equation since no change of energy";
  port_b.h_outflow = inStream(port_a.h_outflow)
    "Stream variable: Trivial equation since no change of energy";

  //
  // Calculate fluid properties
  //
  if propertiesPosition == Internals.Types.PositionProperties.ActualInlet then
   actualFluidProperties = if avoid_events then
       (if noEvent(port_a.m_flow > 0) then fluidProperties_portA else
         fluidProperties_portB) else
       (if port_a.m_flow > 0 then fluidProperties_portA else
         fluidProperties_portB);

  elseif propertiesPosition == Internals.Types.PositionProperties.PortA then
   actualFluidProperties = fluidProperties_portA;

  elseif propertiesPosition == Internals.Types.PositionProperties.PortB then
   actualFluidProperties = fluidProperties_portB;

  end if;

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-40},
            {100,40}}), graphics={
        Rectangle(
          extent={{-100,20},{100,-20}},
          lineColor={0,0,0},
          lineThickness=1,
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid), Line(
          points={{-50,-20},{-50,20},{50,-20},{50,20}},
          color={0,0,0},
          thickness=1)}),                                        Diagram(
        coordinateSystem(preserveAspectRatio=false, extent={{-100,-40},{100,40}})),
    Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialHydraulicResistor;
