within SorpLib.Components.Fittings.HydraulicResistors.TransportPhenomena.OpenAdsorber;
model PackedBed_Kast1988
  "Open adsorber | Packed bed according to Kast"
  extends
    Fittings.HydraulicResistors.BaseClasses.PartialPressureDropOpenAdsorbers(
      final calc_propertyRecord = true);

  //
  // Definition of variables
  //
  Modelica.SIunits.Velocity v
    "Velocity";
  Modelica.SIunits.ReynoldsNumber Re
    "Reynolds-Number";

equation
  //
  // Calculation of characteristic numbers
  //
  v = m_flow / (fluidProperties.rho * cellGeometry.innerCrossSectionalArea *
    cellGeometry.psi)
    "Velocity of fluid";

  Re = abs(v) * cellGeometry.d_inner / fluidProperties.eta * fluidProperties.rho
    "Reynolds number of fluid";

  //
  // Calculation of pressure drop
  //
  pressureDrop = 2.2*(64/(Re + 1) + 1.8/(Re + 1)^0.1)/cellGeometry.d_inner*
    cellGeometry.length*fluidProperties.rho/2*
    SorpLib.Internals.Numerics.squareFunction_noEvent(v, 0.001)
    "Pressure drop according to Kast correlation";

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructering of the library.
  </li>
  <li>
  December 11, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
This model calculates the pressure drop for packed beds according to Kast 
(1988). The sign of the pressure drop is automatically determined by the sign 
of the hydraulic mass flow rate. The hydraulic mass flow rate, the fluid
properties record, the adsorber geometry and discretization number are 
defined as outer objects and thus taken from the overlying model level. 
</p>
<h4>Main equations</h4>
<p>
<p align=\"center\"> <i><code>&Delta;</code>p</i> = sgn(<i>
<code>&#7745;</code></i><sub>hydraulic</sub>) <i>L</i> / <i>D</i> <i>f</i> / 
(2 <i><code>&rho;</code></i> <i>A</i><sup>2</sup> <i><code>&psi;</code></i>
<sup>2</sup>) <i><code>&#7745;</code></i><sub>hydraulic</sub><sup>2</sup> </p>
<p>where <i><code>&Delta;</code>p</i> is the pressure drop, <i><code>&#7745;
</code></i><sub>hydraulic</sub> is the hydraulic mass flow rate, <i>L</i> is
the hydraulic length, <i>D</i> is the hydraulic diameter, <i>f</i> is the 
resistance coefficient, <i><code>&rho;</code></i> is the fluid density, 
<i>A</i> is the cross sectional area and <i><code>&psi;</code></i> is the
void percentage of packed bed. </p>
The resistance coefficient <i>f</i> is calculated by
<p align=\"center\"> <i>f</i> = 2.2 (64/Re + 1.8/Re<sup>0.1</sup>)</p>
where Re is the Reynolds number.
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>Adsorption aus der Gasphase, Werner Kast, Darmstadt, 1988. </li>
</ul>
</p>
</html>"));
end PackedBed_Kast1988;
