within SorpLib.Components.HeatTransfers;
model HeatTransfer
  "Model of heat transfer resistance"

  //
  // Definition of parameters
  //
  parameter Integer n_A(min=1) = 1
    "Number of heat ports A"
    annotation (Dialog(tab = "General", group = "Discretisation"),
                Evaluate=true,
                HideResulte=true);
  parameter Integer n_B(min=1) = 1
    "Number of heat ports B"
    annotation (Dialog(tab = "General", group = "Discretisation"),
                Evaluate=true,
                HideResulte=true);

  parameter Boolean use_alphaA_input = false
    "= true, if alphaA is given by input; otherwise, "
    annotation (Dialog(tab = "General", group = "Heat transfer"),
                Evaluate=true,
                HideResulte=true);

  //
  // Definition of models
  //
  replaceable model HeatTransfer =
    SorpLib.Components.HeatTransfers.TransportPhenomena.Internal.General.ConstantAlpha
    constrainedby
    SorpLib.Components.HeatTransfers.BaseClasses.PartialInternalHeatTransfer
    "Heat transfer model"
    annotation (Dialog(tab = "General", group = "Heat transfer",
               enable = not use_alphaA_input),
               choices(choice(redeclare model HeatTransfer =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.General.ConstantAlpha
                         "Heat transfer | Heat transfer model with constant heat transfer coefficient alpha"),
                       choice(redeclare model HeatTransfer =
            TransHeat.Components.HeatTransfers.TransportPhenomena.Internal.General.ConstantAlphaA
                         "Heat transfer | Heat transfer model with constant heat transfer coefficient alphaA")),
                Evaluate=true,
                HideResulte=true);

  HeatTransfer heatTransfer if not use_alphaA_input
    "Overall Heat transfer resistance"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

  //
  // Definition of connectors
  //
  Modelica.Blocks.Interfaces.RealInput alphaA_input(final unit="W/K") if
    use_alphaA_input
    "AlphaA given by input"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}},
        rotation=270,
        origin={0,40}),
      iconTransformation(extent={{-10,-10},{10,10}},
        rotation=270,
        origin={0,40})));

  Connectors.Thermal.Port_a heatPort_a[n_A]
    "Heat port A"
    annotation (Placement(transformation(extent={{-110,-10},{-90,10}})));
  Connectors.Thermal.Port_b heatPort_b[n_B]
    "Heat port B"
    annotation (Placement(transformation(extent={{90,-10},{110,10}})));

protected
  Modelica.Blocks.Interfaces.RealInput alphaA_model(final unit="W/K")=
    heatTransfer.alphaA if not use_alphaA_input
    "AlphaA calculated by heat transfer model"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealOutput alphaA_internal(final unit="W/K")
    "AlphaA used for calculations"
    annotation (HideResult = true);

equation
  //
  // Assertions
  //
  if n_A < n_B then
    assert(rem(n_B, n_A) == 0,
      "Number of heat ports A must be factor of number of heat ports B!");

  elseif n_A > n_B then
    assert(rem(n_A, n_B) == 0,
      "Number of heat ports A must be factor of number of heat ports B!");

  end if;

  //
  // Calculation of heat ports depending on discretisation approach
  //
  if n_A < n_B then
    //
    // Number of heat ports A is less than number of heat ports B:
    // Connect multiple heat ports B with one heat port A!
    //
    0 =sum(heatPort_a.Q_flow) + sum(heatPort_b.Q_flow)
      "Overall energy balance";

    for i in 1:n_A-1 loop
      0 =heatPort_a[i].Q_flow + sum(heatPort_b[1 + (i - 1)*integer(n_B/n_A):i*
        integer(n_B/n_A)].Q_flow)
        "N-1 energy balances";
    end for;

    for i in 1:n_A loop
      for j in 1:integer(n_B/n_A) loop
        heatPort_b[j + (i - 1)*integer(n_B/n_A)].Q_flow = alphaA_internal/n_B*(
          heatPort_b[j + (i - 1)*integer(n_B/n_A)].T - heatPort_a[i].T)
          "Heat flow calculation depending on discretisation";
      end for;
    end for;

  elseif n_A > n_B then
    //
    // Number of heat ports A is greater than number of heat ports B:
    // Connect multiple heat ports A with one heat port B!
    //
    0 =sum(heatPort_a.Q_flow) + sum(heatPort_b.Q_flow)
      "Overall energy balance";

    for i in 1:n_B-1 loop
      0 =heatPort_b[i].Q_flow + sum(heatPort_a[1 + (i - 1)*integer(n_A/n_B):i*
        integer(n_A/n_B)].Q_flow)
        "N-1 energy balances";
    end for;

    for i in 1:n_B loop
      for j in 1:integer(n_A/n_B) loop
        heatPort_a[j+(i-1)*integer(n_A/n_B)].Q_flow =alphaA_internal/n_A*(
          heatPort_a[j + (i - 1)*integer(n_A/n_B)].T - heatPort_b[i].T)
          "Heat flow calculation depending on discretisation";
      end for;
    end for;
  else
    //
    // Number of heat ports A equals number of heat ports B:
    // Connect one heat port A with one heat port B!
    //
    0 =sum(heatPort_a.Q_flow) + sum(heatPort_b.Q_flow)
      "Overall energy balance";

    for i in 1:n_A-1 loop
      0 =heatPort_a[i].Q_flow + heatPort_b[i].Q_flow
        "N-1 energy balances";
    end for;

    for i in 1:n_A loop
      heatPort_b[i].Q_flow = alphaA_internal/n_A*(heatPort_b[i].T - heatPort_a[
        i].T) "Heat flow calculation depending on discretisation";
    end for;

  end if;

  //
  // Connections
  //
  connect(alphaA_internal, alphaA_input);
  connect(alphaA_internal, alphaA_model);

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,
            -40},{100,40}}),
                        graphics={Rectangle(
          extent={{-100,40},{100,-40}},
          lineColor={238,46,47},
          lineThickness=0.5), Text(
          extent={{-80,20},{80,-20}},
          lineColor={0,0,0},
          lineThickness=1,
          fillColor={244,125,35},
          fillPattern=FillPattern.None,
          textString="k * (T_A-T_b)")}),
                         Diagram(coordinateSystem(preserveAspectRatio=false,
          extent={{-100,-40},{100,40}})),
    Documentation(revisions="<html>
<ul>
  <li>
  January 13, 2021, by Mirko Engelpracht:<br/>
  Major revision allowing different discretizations for both ports.
  </li>
  <li>
  December 06, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
  <p>
    The heat transfer model is used to thermally connect models which can exchange heat. 
    The heat transfer model represents a thermal resistance betweed two models. 
    Depending on the attached temperatures (attached potential) and the chosen transport phenomena, the heat transfer model determines the heat flow rate between the connected models. 
    The heat transfer model itself does not have a thermal capacity.<br>

    The model has two heat ports, whereby one heat port (B) is scalable. 
    The number of heat ports B is automatically adapted depending on connected models. 
    The scalable heat port B enables the connection of various models to a single heat source or sink. 
    For example, this enables to connect a discretized model to a lumped model.
    If two discretized models are connected, the heat transfer model itself can be disctretized in 1-1 configuration.
  </p>
<h4>Main equations</h4>
  <p>
    The heat transfer model is modelled as thermal resistance with no thermal capacity. 
    Thus, the energy balance reads:
    <p align=\"center\"><i>Q</i><sub>in</sub> = <i>Q</i><sub>out</sub> </p>
    where <i>Q</i> ist the heat flow. 
    The heat flow is proportional to the attached temperature difference (potential) according to the following equation:
    <p align=\"center\"><i>Q</i><sub>in</sub> = <i><code>&alpha;</code> A</i> (<i>T</i><sub>in</sub> - <i>T</i><sub>out</sub>) </p>
    where <i><code>&alpha;</code> A</i> is the heat transfer coefficient multiplied with the heat transfer area, calculated in the internal heat transfer model, and <i>T</i> is the temperature.
  </p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end HeatTransfer;
