within SorpLib.Components.HeatTransfers.TransportPhenomena.External;
package LiquidTube
  "Package containing heat transfer models for tube-side heat transfer coefficients of liquid tubes"
  extends Internals.ClassTypes.ModelPackage;

  model ConstantAlpha
    "Liquid heat exchanger | Constant alpha for each cell"
    extends
      HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientLiquidTube(
        final calc_propertyRecord = false);

    //
    // Definition of parameters
    //
    parameter Modelica.SIunits.CoefficientOfHeatTransfer constantAlpha = 25
      "Constant value for alpha"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

  equation
    alphaA = constantAlpha*cellGeometry.innerHeatTransferArea;

    //
    // Annotations
    //
    annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>",   info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
  end ConstantAlpha;

  model ConstantAlphaA
    "Liquid heat exchanger | Constant alphaA for each cell"
    extends
      HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientLiquidTube(
        final calc_propertyRecord = false);

    //
    // Definition of parameters
    //
    parameter Modelica.SIunits.ThermalConductance constantAlphaA = 25
      "Constant value for alphaA for entire tube"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

  equation
    alphaA = constantAlphaA/no_cells_liquid
      "Correct value according to discretization number";

    //
    // Annotations
    //
    annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>",   info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
  end ConstantAlphaA;

  model SiederTate
    "Liquid heat exchanger | Sieder-Tate correlation"
    extends
      HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientLiquidTube(
        final calc_propertyRecord = true);

    //
    // Definition of parameters
    //
    parameter Real SiederTateConstant = 0.027
      "Sieder-Tate constant"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));
    parameter Boolean use_portA = true
      " = true, if port A is used for hydraulic mass flow rate; otherwise, port B
    is used"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

    //
    // Definition of variables
    //
    Modelica.SIunits.ReynoldsNumber Re
      "Reynolds-Number";
    Modelica.SIunits.NusseltNumber Nu
      "Nusselt-Number";

    Modelica.SIunits.CoefficientOfHeatTransfer alpha
      "Heat tranfer coefficient";

  equation
    //
    // Calculation of characteristic numbers
    //
    if use_portA then
      Re = abs(fluidProperties.m_flow_portA) /
        cellGeometry.innerCrossSectionalArea*cellGeometry.d_inner /
        fluidProperties.eta
        "Reynolds-Number";

    else
      Re = abs(fluidProperties.m_flow_portB) /
        cellGeometry.innerCrossSectionalArea*cellGeometry.d_inner /
        fluidProperties.eta
        "Reynolds-Number";

    end if;

    Nu = SiederTateConstant*max(Re,1e-12)^0.8 * fluidProperties.Pr^(1/3)
      "Nusselt-Number";

    //
    // Calculation of heat transfer coefficients
    //
    alpha = Nu*fluidProperties.lambda/cellGeometry.d_inner;
    alphaA = alpha*cellGeometry.innerHeatTransferArea;

    //
    // Annotations
    //
    annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
  <li>
  December 6, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>",   info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
  end SiederTate;

  model Schmidt
    "Liquid heat exchanger | Schmidt-Korrelation"
    extends
      HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientLiquidTube(
        final calc_propertyRecord = true);

    //
    // Definition of parameters
    //
    parameter Modelica.SIunits.Diameter coilDiameter=0.05
      "Diameter of tube coil"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));
    parameter Boolean use_portA = true
      " = true, if port A is used for hydraulic mass flow rate; otherwise, port B
    is used"
      annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

    //
    // Definition of variables
    //
    Real zeta
      "Resistance coefficient";

    Modelica.SIunits.ReynoldsNumber Re
      "Reynolds-Number";
    Modelica.SIunits.NusseltNumber Nu
      "Nusselt-Number";

    Modelica.SIunits.CoefficientOfHeatTransfer alpha
      "Heat tranfer coefficient";

  equation
    //
    // Calculation of characteristic numbers
    //
    if use_portA then
      Re = abs(fluidProperties.m_flow_portA) /
        cellGeometry.innerCrossSectionalArea*cellGeometry.d_inner /
        fluidProperties.eta
        "Reynolds-Number";

    else
      Re = abs(fluidProperties.m_flow_portB) /
        cellGeometry.innerCrossSectionalArea*cellGeometry.d_inner /
        fluidProperties.eta
        "Reynolds-Number";

    end if;

    Nu = (zeta/8*max(Re,1e-12)*fluidProperties.Pr) /
      (1 + 12.7*sqrt(abs(zeta/8))*(fluidProperties.Pr^(2/3) - 1));
    zeta = 0.3164/max(Re,1e-12)^0.25+0.03 *
      (cellGeometry.d_inner/coilDiameter)^0.5;

    //
    // Calculation of heat transfer coefficients
    //
    alpha = Nu*fluidProperties.lambda/cellGeometry.d_inner;
    alphaA = alpha*cellGeometry.innerHeatTransferArea;

    //
    // Annotations
    //
    annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
  <li>
  December 6, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>",   info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
  end Schmidt;
annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end LiquidTube;
