within SorpLib.Components.HeatTransfers.TransportPhenomena.External.OpenAdsorber;
model Gas_PackedBed_Kast1988
  "Open adsorbers | Packed bed: sorbent <-> gas according to Kast"
  extends
    HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientOpenAdsorber(
      final calc_propertyRecord = true);

  //
  // Definition of parameters
  //
  parameter Real Nu_min_particle = 2
    "Minimum Nusselt-Number depending on particle type (e.g., 2 = spherical)"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));
  parameter Boolean use_portA = true
    " = true, if port A is used for hydraulic mass flow rate; otherwise, port B
    is used"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

  //
  // Definition of variables
  //
  Modelica.SIunits.ReynoldsNumber Re
    "Reynolds-Number";
  Modelica.SIunits.NusseltNumber Nu
    "Nusselt-Number of whole bed";
  Modelica.SIunits.NusseltNumber Nu_particle
    "Nusselt-Number of one particle";
  Modelica.SIunits.NusseltNumber Nu_particle_lam
    "Nusselt-Number of one particle for laminar regime";
  Modelica.SIunits.NusseltNumber Nu_particle_turb
    "Nusselt-Number of one particle for turbulent regime";

  Modelica.SIunits.CoefficientOfHeatTransfer alpha
    "Heat tranfer coefficient";

protected
  Modelica.SIunits.Length formFactor=
    1 + 1.5*(1-cellGeometry.psi)
    "Form factor of bed for spherical particles";

equation
  //
  // Calculation of characteristic numbers
  //
  if use_portA then
    Re = abs(fluidProperties.m_flow_portA) /
      (cellGeometry.innerCrossSectionalArea * cellGeometry.psi) *
      cellGeometry.d_particle / fluidProperties.eta
      "Reynolds-Number";

  else
    Re = abs(fluidProperties.m_flow_portB) /
      (cellGeometry.innerCrossSectionalArea * cellGeometry.psi) *
      cellGeometry.d_particle / fluidProperties.eta
      "Reynolds-Number";

  end if;

  Nu = formFactor * Nu_particle
    "Nusselt-Number of whole bed";

  Nu_particle = Nu_min_particle + (Nu_particle_lam^2 + Nu_particle_turb^2)^0.5
    "Nusselt-Number of one particle";

  Nu_particle_lam = 0.664*(Re+1)^0.5 * fluidProperties.Pr^0.5
    "Nusselt-Number of one particle for laminar regime";

  Nu_particle_turb = (0.037*(Re+1)^0.8*fluidProperties.Pr) /
    (1 + 2.443*(Re+1)^(-0.8)*(fluidProperties.Pr^(2/3) - 1))
    "Nusselt-Number of one particle for turbulent regime";

  //
  // Calculation of heat transfer coefficients
  //
  alpha = Nu*fluidProperties.lambda/cellGeometry.d_particle;
  alphaA = alpha*cellGeometry.A_particle_surface / no_cells_gas;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructering of the library.
  </li>
  <li>
  December 11, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
  This model calculates heat transfer coefficient from gas to adsorbent for packed beds according to Kast (1988). The hydraulic mass flow rate, the fluid properties record, the adsorber geometry and discretization number are defined as outer objects and thus taken from the overlying model level.  
</p>
<h4>Main equations</h4>
<p>
  <p align=\"center\"> Nu = <i>f</i><sub>a</sub> (Nu<sub>min</sub> + <code>&radic;</code>(Nu<sub>lam</sub><sup>2</sup> + Nu<sub>turb</sub><sup>2</sup>)) </p>
<p>where Nu is the Nusselt number and <i>f</i><sub>a</sub> is the form factor. </p>
Nu<sub>lam</sub> and Nu<sub>turb</sub> are calculated by following correlations:
<p align=\"center\"> Nu<sub>lam</sub> = 0.664 <code>&radic;</code>Re <code>&radic;</code>Pr </p>
<p align=\"center\"> Nu<sub>turb</sub> = 0.037 Re<sup>0.8</sup> Pr / (1 + 2.443 Re<sup>-0.8</sup> (Pr<sup>2/3</sup> - 1)) </p>
<p>where Re is the Reynolds number and Pr is the Prandtl number. </p>
<p>For detailed information we refer to Kast (1988). </p>
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>Adsorption aus der Gasphase, Werner Kast, Darmstadt, 1988. </li>
</ul>
</p>
</html>"));
end Gas_PackedBed_Kast1988;
