within SorpLib.Components.HeatTransfers.TransportPhenomena.External.OpenAdsorber;
model Wall_PackedBed_Kast1988
  "Open adsorbers | Packed bed: wall <-> gas according to Kast"
  extends
    HeatTransfers.BaseClasses.PartialExternalHeatTransferCoefficientOpenAdsorber(
      final calc_propertyRecord = true);

  //
  // Definition of parameters
  //
  parameter Real type_particle = 12
    "Parameter describing type of particle (e.g., 12 = spherical)"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));
  parameter Boolean use_portA = true
    " = true, if port A is used for hydraulic mass flow rate; otherwise, port B
    is used"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

  //
  // Definition of variables
  //
  Modelica.SIunits.Velocity v
    "Velocity of fluid";
  Modelica.SIunits.ReynoldsNumber Pe
    "Peclet-Number";
  Modelica.SIunits.NusseltNumber Nu
    "Nusselt-Number";
  Modelica.SIunits.NusseltNumber Nu_length
    "Nusselt-Number: Correction of length";

  Modelica.SIunits.CoefficientOfHeatTransfer alpha
    "Heat tranfer coefficient";

protected
  Modelica.SIunits.Length l_mixture=
    1.15 * cellGeometry.d_particle
    "Mixture length of spherical particles";

equation
  //
  // Calculation of characteristic numbers
  //
  if use_portA then
    v = abs(fluidProperties.m_flow_portA) / (fluidProperties.rho *
      cellGeometry.innerCrossSectionalArea * cellGeometry.psi)
      "Reynolds-Number";

  else
    v = abs(fluidProperties.m_flow_portB) / (fluidProperties.rho *
      cellGeometry.innerCrossSectionalArea * cellGeometry.psi)
      "Reynolds-Number";

  end if;

  Pe = v*l_mixture * (fluidProperties.eta/fluidProperties.Pr) /
    fluidProperties.rho
    "Peclet-Number";

  Nu = Nu_length / (1+type_particle*cellGeometry.length /
    cellGeometry.d_inner / (Pe+0.1))
    "Nusselt-Number";

  Nu_length = -1.55e-6*Pe^2 + 0.04285*Pe + 24.57
    "Nusselt-Number: Own fit for spherical particles from Kast (1988) p. 136";

  //
  // Calculation of heat transfer coefficients
  //
  alpha = Nu*fluidProperties.lambda/l_mixture;
  alphaA = alpha*cellGeometry.innerHeatTransferArea;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructering of the library.
  </li>
  <li>
  December 11, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
This model calculates heat transfer coefficient from gas to wall for packed 
beds according to Kast (1988). The hydraulic mass flow rate, the fluid
properties record, the adsorber geometry and discretization number are defined 
as outer objects and thus taken from the overlying model level. 
</p>
<h4>Main equations</h4>
<p>
<p align=\"center\"> Nu (1 + <i>c</i> <i>L / D</i> / Pe) = <i>f</i>(Pe)</p>
  <p>where Nu is the Nusselt number, <i>c</i> is a constant for the particle 
  form, <i>L</i> is the hydraulic length, <i>D</i> is the hydraulic diameter 
  and Pe is the Peclet number. The correlation <i>f</i>(Pe) is fitted with a 
  second order polynom according to the diagram in Kast (1988), p. 136. </p>
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>Adsorption aus der Gasphase, Werner Kast, Darmstadt, 1988. </li>
</ul>
</p>
</html>"));
end Wall_PackedBed_Kast1988;
