within SorpLib.Components.Machines.Pumps.BaseClasses;
partial model PartialSimplePump "Base model for all simple pump models"

  //
  // Definition of general parameters
  //
  parameter Modelica.SIunits.Efficiency eta_pump = 0.6
    "Efficiency of pump that is used to calculate shaft power"
    annotation (Dialog(tab="General", group="Pump Characteristics"));
  parameter Modelica.SIunits.Efficiency eta_drive = 0.9
    "Efficiency of drive that is used to calculate drive power"
    annotation (Dialog(tab="General", group="Pump Characteristics"));

  //
  // Definition of parameters describing the medium
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Start value for mass flow rate"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Boolean assumeIsenthalpicPump = false
    "=true, if internal losses due not affect energy balance"
    annotation (Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of inputs
  //
  parameter Boolean use_m_flow = false
    "= true, if mass flow rate is given by input; otherwise, volume flow is given"
    annotation (Dialog(tab="General", group="Inputs"),
                Evaluate=true,
                HideResult=true);

  parameter Boolean use_m_flow_input = false
    "= true, if mass flow rate is given by user input"
    annotation (Dialog(tab="General", group="Inputs",
                enable=use_m_flow),
                Evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.MassFlowRate m_flow_fixed = 0.15
    "Fixed prescribed mass flow rate"
    annotation (Dialog(tab="General", group="Inputs",
                enable = not use_m_flow_input and use_m_flow),
                Evaluate=true,
                HideResult=true);

  parameter Boolean use_V_flow_input = false
    "= true, if volume flow rate is given by user input"
    annotation (Dialog(tab="General", group="Inputs",
                enable= not use_m_flow),
                Evaluate=true,
                HideResult=true);
  parameter Modelica.SIunits.VolumeFlowRate V_flow_fixed = 10/60/1000
    "Fixed prescribed volume flow rate"
    annotation (Dialog(tab="General", group="Inputs",
                enable = not use_V_flow_input and not use_m_flow),
                Evaluate=true,
                HideResult=true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Definition of variables
  //
public
  Modelica.SIunits.PressureDifference dp
    "Pressure increase of pump";

  Modelica.SIunits.VolumeFlowRate V_flow
    "Volume flow rate of pump";

  Modelica.SIunits.Power P_hydraulic
    "Hydraulic power consumption of pump";
  Modelica.SIunits.Power P_shaft
    "Shaft power consumption of pump";
  Modelica.SIunits.Power P_drive
    "Drive power consumption of pump";
  Modelica.SIunits.Power P_loss_internal
    "Internal power losses of pump (i.e., increase outflowing specific enthalpy)";

protected
  Modelica.SIunits.Density rho_in
    "Density at inlet of liquid media";

  //
  // Definition of connectors
  //
public
  replaceable Connectors.BaseClasses.PartialMediaConnector port_a(
    final no_components=no_components,
    m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-110,-10},{-90,10}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector port_b(
    final no_components=no_components,
    m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{88,-10},{108,10}})));

  Modelica.Blocks.Interfaces.RealInput m_flow_input if use_m_flow_input and
    use_m_flow
    "Input for prescribed mass flow rate" annotation (Placement(transformation(
        extent={{-20,-20},{20,20}},
        rotation=90,
        origin={0,-100}), iconTransformation(
        extent={{-10,-10},{10,10}},
        rotation=90,
        origin={0,-100})));
  Modelica.Blocks.Interfaces.RealInput V_flow_input if use_V_flow_input and
    not use_m_flow
    "Input for prescribed volume flow rate" annotation (Placement(transformation(
        extent={{-20,-20},{20,20}},
        rotation=90,
        origin={0,-100}), iconTransformation(
        extent={{-10,-10},{10,10}},
        rotation=90,
        origin={0,-100})));

  //
  // Definition of connectors for conditional removed variables / models
  //
protected
  Modelica.Blocks.Interfaces.RealInput m_flow_dummy(final unit="kg/s")=
    0 if not use_m_flow
    "Fixed mass flow given by parameter input";
  Modelica.Blocks.Interfaces.RealInput m_flow_parameter(final unit="kg/s")=
    m_flow_fixed if not use_m_flow_input and use_m_flow
    "Fixed mass flow given by parameter input";

  Modelica.Blocks.Interfaces.RealInput V_flow_dummy(final unit="m3/s")=
    0 if use_m_flow
    "Fixed volume flow given by parameter input";
  Modelica.Blocks.Interfaces.RealInput V_flow_parameter(final unit="m3/s")=
    V_flow_fixed if not use_V_flow_input and not use_m_flow
    "Fixed volume flow given by parameter input";

  Modelica.Blocks.Interfaces.RealOutput m_flow_(final unit="kg/s")
    "Prescribed volume flow rate";
  Modelica.Blocks.Interfaces.RealOutput V_flow_(final unit="m3/s")
    "Prescribed volume flow rate";

equation
  //
  // Connectors
  //
  connect(m_flow_, m_flow_dummy);
  connect(m_flow_, m_flow_input);
  connect(m_flow_, m_flow_parameter);

  connect(V_flow_, V_flow_dummy);
  connect(V_flow_, V_flow_input);
  connect(V_flow_, V_flow_parameter);

  //
  // Momentum balance
  //
  dp = port_a.p - port_b.p
    "Pressure difference between port a and b";

  //
  // Mass balance
  //
  0 = port_a.m_flow + port_b.m_flow
    "Steady-state mass flow balance";

  port_a.xi_outflow = inStream(port_b.xi_outflow)
    "Stream variable: Trivial equation since no change of mass fractions";
  port_b.xi_outflow = inStream(port_a.xi_outflow)
    "Stream variable: Trivial equation since no change of mass fractions";

  port_a.m_flow = if use_m_flow then m_flow_ else V_flow_ * rho_in
    "Prescribed mass flow rate";

  //
  // Energy balance
  //
  V_flow = if use_m_flow then V_flow_ else port_a.m_flow / rho_in
    "Volume flow of fluid";

  P_hydraulic = - V_flow * dp
    "Hydraulic power consumption of pump";
  P_shaft = P_hydraulic / eta_pump
    "Shaft power consumption of pump";
  P_drive = P_shaft / eta_drive
    "Drive power consumption of pump";
  P_loss_internal = P_shaft - P_hydraulic
    "Internal power losses of pump (i.e., increase outflowing specific enthalpy)";

  port_b.h_outflow = if assumeIsenthalpicPump then inStream(port_a.h_outflow) else
    inStream(port_a.h_outflow) - (1/eta_pump - 1) * dp / rho_in
    "Increase in specific enthalpy due to internal losses of pump";
  port_a.h_outflow = inStream(port_b.h_outflow)
    "Stream variable: Trivial equation since flow is only allowd from port A
     to port B; otherwise, nothing happens";

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
        Ellipse(
          extent={{-100,100},{100,-100}},
          lineColor={0,0,0},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid),
        Line(points={{-60,80},{98,20}}, color={0,0,0}),
        Line(points={{-60,-80},{98,-20}}, color={0,0,0}),
        Line(
          points={{-68,0},{72,0}},
          color={0,0,0},
          thickness=1,
          arrow={Arrow.None,Arrow.Filled})}),                    Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 20, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSimplePump;
