within SorpLib.Components.MassTransfers.BaseClasses;
partial model PartialMassTransfer
  "Base model for all mass transfer models"

  //
  // Definition of parameters describing the kind of valve
  //
  parameter Boolean is_activelyCloseable = false
    "= true, if valve is actively closeable; otherwise, valve cannot be closed
     actively"
    annotation (Dialog(tab="General",group="Valve Characteristics"),
                Evaluate = true,
                HideResult=true);
  parameter Boolean is_flapValve = false
    "= true, if valve is flap valve; otherwise, valve is not a flap valve"
    annotation (Dialog(tab="General",group="Valve Characteristics"),
                Evaluate = true,
                HideResult=true);
  parameter Boolean is_flowAB = true
    "= true, if flow direction is A->B; otherwise, flow direction is B->A"
    annotation (Dialog(tab="General",group="Valve Characteristics",
                enable = is_flapValve),
                Evaluate = true,
                HideResult=true);

  //
  // Definition of parameters describing the mass transfer model
  //
  parameter Integer no_cells(min=1)=1
    "Discretization number of working pair cells (i.e, required for 
    discretization of working pair cell (i.e., vapor port))"
    annotation (Dialog(tab="General",group="Mass Transfer"),
                Evaluate=true,
                HideResult=true);
  parameter Boolean calculate_FluidProperties = false
    "= true, to calculate actual fluid properties at port A and B"
    annotation (Dialog(tab="General",group="Mass Transfer"),
                Evaluate = true,
                HideResult=true);
  parameter Boolean use_beta_input = false
    "= true, if mass transport resistance is given by input"
    annotation (Dialog(tab="General",group="Mass Transfer"),
                Evaluate = true,
                HideResult=true);
  parameter Internals.Types.PositionProperties propertiesPosition=
    Internals.Types.PositionProperties.Average
    "Tye defining calculation approach of fluid properties if needed"
    annotation (Dialog(tab="General",group="Mass Transfer",
                enable = not use_beta_input),
                Evaluate = true,
                HideResult=true);

  //
  // Definition of parameters describing the medium
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Definition of connectors
  //
public
  Modelica.Blocks.Interfaces.RealInput open_input if is_activelyCloseable
    "> 0, if valve is open; otherwise, valve is actively closed"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}},
                rotation=90,origin={0,-100})),
                HideResult=not is_activelyCloseable);

  //
  // Definition of variables
  //
  Modelica.SIunits.Pressure dp
    "Total pressure difference";

  //
  // Definition of connectors for conditional removed models / variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput open_fixed = 1 if not is_activelyCloseable
    "If valve is not actively closeable, set its status to 'open'"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput open_
    "Status of valve: > 0, if valve is open; otherwise, valve is closed"
    annotation (HideResult=true);

equation
  //
  // Connections
  //
  connect(open_, open_fixed);
  connect(open_, open_input);

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  Major revisions (e.g., object-oriented approach) after restructuring of the library.
  </li>
  <li>
  November 30, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialMassTransfer;
