within SorpLib.Components.MassTransfers.BaseClasses;
partial model PartialSingleMassTransfer
  "Base model for all mass transfer models of pure fluids"
  extends PartialMassTransfer;

  //
  // Definition of parameters describing the mass transfer model
  //
  replaceable model MassTransfer =
    TransportPhenomena.BaseClasses.PartialInternalMassTransfer
    constrainedby TransportPhenomena.BaseClasses.PartialInternalMassTransfer
    "Mass transfer model"
    annotation (Dialog(tab="General",group="Mass Transfer",
                enable=not use_beta_input),
                choicesAllMatching = true,
                Evaluate=true,
                HideResult=true);

  //
  // Definition of parameters describing the initialisation
  //
  parameter Modelica.SIunits.Pressure pInitial = 1e3
    "Initial value for pressures at ports"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                HideResult=true);
  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Initial value for mass flow rate"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                HideResult=true);

  //
  // Instanziation of models
  //
  MassTransfer massTransfer
    "Mass transfer model"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})),
                Hide=use_beta_input);

  //
  // Definition of variables
  //
  Real drivingForce
    "Driving force for mass transfer depending on kind of valve: dp or dx";
  Real drivingForce_status
    "Status of valve according to driving force and flap valve behaivior: < 0,
     if valve is open; otherwise, valve is closed";

  Real open = open_ * drivingForce_status
    "Current status of valve accounting for driving force and external signal:
     < 0, if valve is open; otherwise, valve is closed";

protected
  Real drivingForce_
    "Internal value of driving force: Required to pass either pressure or
    loading difference in extended models"
    annotation (HideResult=true);

  Modelica.SIunits.Temperature T_adsorptive(min=-1)
    "Temperature of adsorptive if required for transport phenomena"
    annotation (HideResult=true);
  Modelica.SIunits.Density d_adsorptive(min=-1)
    "Density of adsorptive if required for transport phenomena"
    annotation (HideResult=true);
  Modelica.SIunits.DynamicViscosity eta_adsorptive(min=-1)
    "Dynamic viscosity of adsorptive if required for transport phenomena"
    annotation (HideResult=true);

  //
  // Definition of connectors
  //
public
  replaceable Connectors.BaseClasses.PartialMediaConnector port_a(
    final no_components=no_components,
    m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-110,-10},{-90,10}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector port_b(
    final no_components=no_components,
    m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{90,-10},{110,10}})));

  Modelica.Blocks.Interfaces.RealInput beta_input if
       use_beta_input
    "Mass transport coefficient calculated externally"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}},
                rotation=90,origin={30,-100})),
                HideResult=not beta_input);

  //
  // Definition of connectors for conditional removed models / variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput beta_calculated = massTransfer.beta if
       not use_beta_input
    "Mass transport coefficient calculated internally"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealInput T_adsorptive_dummy(final unit="K")=
    -1 if not massTransfer.calculate_FluidProperties
    "Dummy temperature of adsorptiv"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealInput d_adsorptive_dummy(final unit="kg/m3")=
    -1 if not massTransfer.calculate_FluidProperties
    "Dummy density of adsorptiv"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealInput eta_adsorptive_dummy(final unit="Pa.s")=
    -1 if not massTransfer.calculate_FluidProperties
    "Dummy dynamic viscosity of adsorptiv"
    annotation (HideResult=true);

  Modelica.Blocks.Interfaces.RealOutput beta_internal
    "Mass transport coefficient"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput T_adsorptive_A_internal(final unit="K")
    "Temperature of adsorptive at port A"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput T_adsorptive_B_internal(final unit="K")
    "Temperature of adsorptive at port B"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput d_adsorptive_A_internal(final unit="kg/m3")
    "Density of adsorptive at port A"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput d_adsorptive_B_internal(final unit="kg/m3")
    "Density of adsorptive at port B"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput eta_adsorptive_A_internal(final unit="Pa.s")
    "Dynamic viscosity of adsorptive at port A"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput eta_adsorptive_B_internal(final unit="Pa.s")
    "Dynamic viscosity of adsorptive at port B"
    annotation (HideResult=true);

equation
  //
  // Assertations
  //
  assert(no_components == 1, "Mass transfer model is only valid for pure
   component!");

  //
  // Momentum balance
  //
  dp = port_a.p - port_b.p
    "Pressure drop dp required for flap valve behaiviour and/or pressure driven
     mass transfer";

  //
  // Energy balance
  //
  port_a.h_outflow = inStream(port_b.h_outflow)
    "No change of energy";
  port_b.h_outflow = inStream(port_a.h_outflow)
    "No change of energy";

  //
  // Mass balance
  //
  0 = port_a.m_flow + port_b.m_flow
    "No change of mass";

  port_a.xi_outflow = inStream(port_b.xi_outflow)
    "No change of transported substances";
  port_b.xi_outflow = inStream(port_a.xi_outflow)
    "No change of transported substances";

  //
  // Determine mass flow
  //
  if avoid_events then
    port_a.m_flow = if noEvent(open > 0.5) then
      beta_internal*drivingForce/no_cells else 0
      "Mass flow depends on status of driving force (i.e., open or closed)";

  else
    port_a.m_flow = if open > 0.5 then
      beta_internal*drivingForce/no_cells else 0
      "Mass flow depends on status of driving force (i.e., open or closed)";

  end if;

  if is_flapValve then
    //
    // Flap valve: Driving force depends on flow direction
    //
    drivingForce = if is_flowAB then max(drivingForce_, 0)
      else min(drivingForce_, 0)
      "A->B: Values at port A must be greater than at port B; B->A: vice versa";

    if is_flowAB then
      if avoid_events then
        drivingForce_status = if noEvent(dp > 0) then 1 else 0
          "A->B: Values at port A must be greater than at port B";

      else
        drivingForce_status = if dp > 0 then 1 else 0
          "A->B: Values at port A must be greater than at port B";

      end if;

    else
      if avoid_events then
        drivingForce_status = if noEvent(dp < 0) then 1 else 0
          "B->A: Values at port B must be greater than at port A";

      else
        drivingForce_status = if dp < 0 then 1 else 0
          "B->A: Values at port B must be greater than at port A";

      end if;

    end if;

  else
    //
    // No flap valve: Use driving force as given
    //
    drivingForce_status = 1
      "No flap valve: Valve is always open w.r.t. driving force, i.e., valve is
       only closed when using a actively closeable valve";
    drivingForce = drivingForce_
      "Driving force depends on kind of valve: Pressure- or loading-driven valve";

  end if;

  //
  // Calculate fluid properties if required for mass transport phenomena
  //
  if massTransfer.calculate_FluidProperties and not use_beta_input then
    //
    // Calculate fluid properties
    //
    if propertiesPosition == Internals.Types.PositionProperties.ActualInlet then
      if avoid_events then
        if noEvent(port_a.m_flow > 0) then
          T_adsorptive = T_adsorptive_A_internal;
          d_adsorptive = d_adsorptive_A_internal;
          eta_adsorptive = eta_adsorptive_A_internal;

        else
          T_adsorptive = T_adsorptive_B_internal;
          d_adsorptive = d_adsorptive_B_internal;
          eta_adsorptive = eta_adsorptive_B_internal;

        end if;

     else
        if port_a.m_flow > 0 then
          T_adsorptive = T_adsorptive_A_internal;
          d_adsorptive = d_adsorptive_A_internal;
          eta_adsorptive = eta_adsorptive_A_internal;

        else
          T_adsorptive = T_adsorptive_B_internal;
          d_adsorptive = d_adsorptive_B_internal;
          eta_adsorptive = eta_adsorptive_B_internal;

        end if;

      end if;

    elseif propertiesPosition == Internals.Types.PositionProperties.PortA then
      T_adsorptive = T_adsorptive_A_internal;
      d_adsorptive = d_adsorptive_A_internal;
      eta_adsorptive = eta_adsorptive_A_internal;

    elseif propertiesPosition == Internals.Types.PositionProperties.PortB then
      T_adsorptive = T_adsorptive_B_internal;
      d_adsorptive = d_adsorptive_B_internal;
      eta_adsorptive = eta_adsorptive_B_internal;

    else
      T_adsorptive = (T_adsorptive_A_internal+T_adsorptive_B_internal)/2;
      d_adsorptive = (d_adsorptive_A_internal+d_adsorptive_B_internal)/2;
      eta_adsorptive = (eta_adsorptive_A_internal+eta_adsorptive_B_internal)/2;

    end if;

  else
    //
    // Set dummy values
    //
    T_adsorptive = -1;
    d_adsorptive = -1;
    eta_adsorptive = -1;

  end if;

  //
  // Connections
  //
  connect(beta_internal, beta_calculated);
  connect(beta_internal, beta_input);
  connect(T_adsorptive_A_internal, T_adsorptive_dummy);
  connect(T_adsorptive_B_internal, T_adsorptive_dummy);
  connect(d_adsorptive_A_internal, d_adsorptive_dummy);
  connect(d_adsorptive_B_internal, d_adsorptive_dummy);
  connect(eta_adsorptive_A_internal, eta_adsorptive_dummy);
  connect(eta_adsorptive_B_internal, eta_adsorptive_dummy);

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  Major revisions (e.g., object-oriented approach) after restructuring of the library.
  </li>
  <li>
  November 30, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
  <p>
    The mass transfer models are used to connect the adsorbent cell to an evaporator, a condenser, or a moist air cell (or any other adsorbate source or sink). 
    The mass transfer models are modeled as an isenthalpic valve with no volume. 
    Depending on the chosen Transport Phenomena, the Mass Transfer model represents diffusion in the particle and also pressure drop due to friction in pipes, connceting the adsorbent to other models.
    The partial model includes the main equations, used for all Mass Transfer models including:
    <ul>
      <li>2 vle ports</li>
      <li>Parameters and connectors to model an activly closable and a flap valve (opens only in one direction)</li>
      <li>Mass balance</li>
      <li>Energy balance</li>
    </ul>
  </p>
<h4>Main equations</h4>
  <p>
    The mass transfer model is modelled as an isenthalpic valve with no storage volume. 
    Thus, the mass balance reads:
    <p align=\"center\"><i><code>m&#775;</code></i><sub>in</sub> = <i><code>m&#775;</code></i><sub>out</sub> </p>
    The energy balance boils down to equal specific enthalpies:
    <p align=\"center\"><i>h</i><sub>in</sub> = <i>h</i><sub>out</sub> </p>
  </p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
  <p>
    For more information see:
    <ul>
    <li>Lanzerath, F.; Bau, U.; Seiler, J.; Bardow, A. Optimal design of adsorption chillers based on a validated dynamic object-oriented model. Science and Technology for the Built Environment, 2015, 21(3), 248-257. </li>
    </ul>
  </p>
</html>"));
end PartialSingleMassTransfer;
