﻿within SorpLib.Components.MassTransfers.BaseClasses;
partial model PartialSingleMassTransfer_dx
  "Base model of loading-driven mass transfer"
  extends PartialSingleMassTransfer(
    redeclare final model MassTransfer = MassTransfer_dx);

  //
  // Definition of models describing the mass transfer
  //
  replaceable model MassTransfer_dx =
    SorpLib.Components.MassTransfers.TransportPhenomena.LoadingDriven.ConstantCoefficient
    constrainedby
    SorpLib.Components.MassTransfers.TransportPhenomena.BaseClasses.PartiallInternalMassTransfer_dx(
      final massTransferProperties=massTransferProperties)
    "Mass transfer model"
    annotation (Dialog(tab="General",group="Mass Transfer", enable=not use_beta_input),
                choices(choice(redeclare model MassTransfer_dx =
            TransHeat.Components.MassTransfers.TransportPhenomena.LoadingDriven.ConstantCoefficient
                         "Mass Transfer | Constant mass transfer coefficient for loading-driven mass transfer"),
                       choice(redeclare model MassTransfer_dx =
            TransHeat.Components.MassTransfers.TransportPhenomena.LoadingDriven.ConstantSpecificCoefficient
                         "Mass Transfer | Constant specific mass transfer coefficient for loading-driven mass transfe"),
                       choice(redeclare model MassTransfer_dx =
            TransHeat.Components.MassTransfers.TransportPhenomena.LoadingDriven.Glueckauf
                         "Mass Transfer | Glückauf-approach for loading-driven mass transfer"),
                       choice(redeclare model MassTransfer_dx =
            TransHeat.Components.MassTransfers.TransportPhenomena.LoadingDriven.GlueckaufArrhenius
                         "Mass Transfer | Glückauf-approach with temperature-dependent diffusion coefficient for loading-driven mass transfer")),
                 Evaluate=true,
                 HideResult=true);

  replaceable model WorkingPair =
    Media.WorkingPair.SingleComponent.BaseClasses.PartialSingleWPair
    constrainedby
    Media.WorkingPair.SingleComponent.BaseClasses.PartialSingleWPair(
      final use_pInitial= false,
      final pInitial=pInitial,
      final c_sorbent=1000,
      final calculate_caloric_properties=false,
      final type_isotherm=Internals.Types.Isotherm.Classical)
    "Working pair model"
    annotation (Dialog(tab="General",group="Mass Transfer"),
                choicesAllMatching = true,
                 Evaluate=true,
                 HideResult=true);

  //
  // Definition of inputs
  //
  parameter Boolean calculate_equilibriumPropertiesInternally = false
    "= true, if equilibrium properties at port B are calculated internally;
    otherwise, properties are given as inputs"
    annotation (Dialog(tab="General",group="Inputs"),
                Evaluate=true,
                Hide=true);

  input Real T_B_theoEqu(unit="K") if
    not calculate_equilibriumPropertiesInternally
    "Actual (i.e., theoretical) equilibrium temperature of adsorbate"
    annotation (Dialog(tab="General",group="Inputs",
                enable=not calculate_equilibriumPropertiesInternally));
  input Real x_B_act(unit="kg/kg") if
    not calculate_equilibriumPropertiesInternally
    "Actual equilibrium loading of adsorbate"
    annotation (Dialog(tab="General",group="Inputs",
                enable=not calculate_equilibriumPropertiesInternally));

  //
  // Instantiation of models
  //
  WorkingPair workingPair_A_theoEqu(
    final p_adsorpt=port_a.p,
    final T_adsorpt=T_act_internal,
    final x_adsorpt=x_A_theEqu)
    "Working pair model to calculate equilibrium properties at port A"
    annotation (Placement(transformation(extent={{-70,-10},{-50,10}})));
  WorkingPair workingPair_B_act(
    final p_adsorpt=port_b.p,
    final T_adsorpt=T_act_internal,
    final x_adsorpt=x_B_act_calc) if
    calculate_equilibriumPropertiesInternally
    "Working pair model to calculate equilibrium properties at port B"
    annotation (Placement(transformation(extent={{50,-10},{70,10}})));

  //
  // Definition of property record
  //
  TransportPhenomena.PropertyRecord.MassTransferProperties
    massTransferProperties(
      T_adsorbate_theoEqu=T_act_internal,
      T_adsorptive=T_adsorptive,
      d_adsorptive=d_adsorptive,
      eta_adsorptive=eta_adsorptive)
    "Record containing properties of mass transfer model if required";

  //
  // Definition of variables
  //
protected
  Real x_A_theEqu(unit="kg/kg")
    "Theoretical equilibrium loading of adsorbate at port A"
    annotation (HideResult=true);

  //
  // Definition of connectors for conditional removed models / variables
  //
  Modelica.Blocks.Interfaces.RealInput T_B_theoEqu_input(final unit="K")=
    T_B_theoEqu if not calculate_equilibriumPropertiesInternally
    "Equilibrium temperature of adsorbate at port B"
    annotation (HideResult=true);

  Modelica.Blocks.Interfaces.RealInput x_B_act_input(final unit="kg/kg")=
    x_B_act if not calculate_equilibriumPropertiesInternally
    "Actual equilibrium loading of adsorbate at port B"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealInput x_B_act_calc(final unit="kg/kg") if
       calculate_equilibriumPropertiesInternally
    "Actual equilibrium loading of adsorbate at port B"
    annotation (HideResult=true);

  Modelica.Blocks.Interfaces.RealOutput T_act_internal(final unit="K")
    "Equilibrium temperature of adsorbate at port B"
    annotation (HideResult=true);
  Modelica.Blocks.Interfaces.RealOutput x_B_act_internal(final unit="kg/kg")
    "Actual equilibrium loading of adsorbate at port B"
    annotation (HideResult=true);

equation
  //
  // Set driving potential
  //
  drivingForce_ = x_A_theEqu - x_B_act_internal
    "Loading driven mass flow rate";

  //
  // Connections
  //
  connect(T_act_internal, T_B_theoEqu_input);

  connect(x_B_act_internal, x_B_act_input);
  connect(x_B_act_internal, x_B_act_calc);

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSingleMassTransfer_dx;
