﻿within SorpLib.Components.MassTransfers.TransportPhenomena.LoadingDriven;
model GlueckaufArrhenius
  "Mass Transfer | Glückauf-approach with temperature-dependent diffusion coefficient for loading-driven mass transfer"
  extends BaseClasses.PartiallInternalMassTransfer_dx(
    final calculate_FluidProperties=true);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.Mass m_sorbent(start=2) = 2.5
    "Mass of dry sorbent material"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);
  parameter Modelica.SIunits.Mass M_sorbent = 0.018
    "Molar mass of sorbent material"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);

  parameter Modelica.SIunits.Radius r_sorbent = 0.001
    "Radius of sorbent material"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);

  parameter Real D(final unit="m2/s", start=1e-10) = 1e-10
    "Diffusion coefficient"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);
  parameter Modelica.SIunits.SpecificEnergy Ea(start=2.33e6) = 2.33e6
    "Activation energy"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);

equation
  //
  // Calculation of mass transfer coefficient
  //
  beta = 15*m_sorbent*D/r_sorbent^2 *
    exp(-Ea/((Modelica.Constants.R/M_sorbent) *
    massTransferProperties.T_adsorbate_theoEqu));

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructuring of the library.
  </li>
  <li>
  November 30, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
  This mass transfer model derived by Glueckauf (1955) for spheres is enhanced by an Arrhenius approach to determine the mass diffusion coefficient <i>D</i>.
</p>
<h4>Main equations</h4>
<p>
  The mass transfer according to the Linear Driving Force apporach is determined by the following equation:
  <p align=\"center\"> <i><code>m&#775;</code></i> = 15 <i>D</i> / <i>r</i><sup>2</sup> * <i>m</i><sub>sor</sub> (<i>x</i> - <i>x</i><sub>eq</sub>) </p>
  where <i>D</i> is the diffusion coefficient and <i>r</i> is the particle radius.<br>
  Using the Arrhenius approach, the diffusion coefficient can be determined by the following temperature dependent correlation:
  <p align=\"center\"> <i>D</i> = <i>D</i><sub>s,0</sub>  exp ( -<i>E</i><sub>a</sub> / (<i>R T</i>) ) </p>
  where <i>D</i><sub>s,0</sub> is a pre-exponent constant, <i>E</i><sub>a</sub> is the activation energy, <i>R</i> is the gas constant, and <i>T</i> is the temperature.
</p>
<h4>Assumptions and limitations</h4>
  <p>
    This correlation can be used when Knudsen diffusion is the dominant mass transfer regime Ruthven (1984). <br>
    For silica-gel, this approach is used by Sakoda and Suzuki (1984) and by many studies referring to it. 
  </p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<ul>
  <li>Ruthven, D.M. Principles of adsorption and adsorption processes. New York: Wiley, 1984. ISBN 0471866067.</li>
  <li>Sakoda, A.; Suzuki, M. Fundamental study on a solar powered adsorption cooling system. Journal of Chemical Engineering of Japan, 1984, 17(1), 52-57.</li>
</ul>
</html>"));
end GlueckaufArrhenius;
