within SorpLib.Components.MassTransfers.TransportPhenomena.OpenAdsorber;
model PackedBed_Kast1988
  "Open adsorber | Packed bed according to Kast"
  extends BaseClasses.PartialExternalMassTransferOpenAdsorber_dp(
    final calculate_FluidProperties=false);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.MolarMass M_adsorptive = 0.018
    "Molar mass of adsorptive"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);

  //
  // Definition of parameters
  //
  parameter Real Sh_min_particle = 2
    "Minimum Sherwood-Number depending on particle type (e.g., 2 = spherical)"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));
  parameter Boolean use_portA = true
    " = true, if port A is used for hydraulic mass flow rate; otherwise, port B
    is used"
    annotation (Dialog(tab="General",group="Heat Transfer Correlation"));

  //
  // Definition of variables
  //
  Modelica.SIunits.ReynoldsNumber Re
    "Reynolds-Number";
  Real Sh
    "Sherwood-Number of whole bed";
  Real Sh_particle
    "Sherwood-Number of one particle";
  Real Sh_particle_lam
    "Sherwood-Number of one particle for laminar regime";
  Real Sh_particle_turb
    "Sherwood-Number of one particle for turbulent regime";
  Modelica.SIunits.SchmidtNumber Sc
    "Schmidt-Number";

  Modelica.SIunits.DiffusionCoefficient D
    "Diffusion coefficient in binary mixture";

protected
  Modelica.SIunits.Length formFactor=
    1 + 1.5*(1-cellGeometry.psi)
    "Form factor of bed for spherical particles";

equation
  //
  // Calculation of characteristic numbers
  //
  if use_portA then
    Re = abs(fluidProperties.m_flow_portA) /
      (cellGeometry.innerCrossSectionalArea * cellGeometry.psi) *
      cellGeometry.d_particle / fluidProperties.eta
      "Reynolds-Number";

  else
    Re = abs(fluidProperties.m_flow_portB) /
      (cellGeometry.innerCrossSectionalArea * cellGeometry.psi) *
      cellGeometry.d_particle / fluidProperties.eta
      "Reynolds-Number";

  end if;

  D = 1.72e-7*fluidProperties.T-2.438e-5
    "Diffusion coefficient";

  Sc = fluidProperties.eta/fluidProperties.rho/D
    "Schmidt-Number";

  Sh = formFactor * Sh_particle
    "Sherwood-Number of whole bed";

  Sh_particle = Sh_min_particle + (Sh_particle_lam^2 + Sh_particle_turb^2)^0.5
    "Sherwood-Number of one particle";

  Sh_particle_lam = 0.664*(Re+1)^0.5*Sc^0.5
    "Sherwood-Number of one particle for laminar regime";

  Sh_particle_turb = (0.037*(Re+1)^0.8*Sc) /
    (1 + 2.443*(Re+1)^(-0.8)*(Sc^(2/3) - 1))
    "Sherwood-Number of one particle for turbulent regime";

  //
  // Calculation of heat transfer coefficients
  //
  beta = Sh * D / cellGeometry.d_particle * M_adsorptive /
    (fluidProperties.T*Modelica.Constants.R) *
    cellGeometry.A_particle_surface / no_cells_gas;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 14, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructering of the library.
  </li>
  <li>
  December 11, 2017, by Andrej Gibelhaus:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
  This model calculates heat mass coefficient from gas to adsorbent for packed beds according to Kast (1988). The hydraulic mass flow rate, the fluid properties record, the adsorber geometry and discretization number are defined as outer objects and thus taken from the overlying model level.  
</p>
<h4>Main equations</h4>
<p>
  <p align=\"center\"> Sh = <i>f</i><sub>a</sub> (Sh<sub>min</sub> + <code>&radic;</code>(Sh<sub>lam</sub><sup>2</sup> + Sh<sub>turb</sub><sup>2</sup>)) </p>
<p>where Sh is the Sherwood number and <i>f</i><sub>a</sub> is the form factor. </p>
Sh<sub>lam</sub> and Sh<sub>turb</sub> are calculated by following correlations:
<p align=\"center\"> Sh<sub>lam</sub> = 0.664 <code>&radic;</code>Re <code>&radic;</code>Sc </p>
<p align=\"center\"> Sh<sub>turb</sub> = 0.037 Re<sup>0.8</sup> Sc / (1 + 2.443 Re<sup>-0.8</sup> (Sc<sup>2/3</sup> - 1)) </p>
<p>where Re is the Reynolds number and Sc is the Schmidt number. </p>
<p>For detailed information we refer to Kast (1988). </p>
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>Adsorption aus der Gasphase, Werner Kast, Darmstadt, 1988. </li>
</ul>
</p>
</html>"));
end PackedBed_Kast1988;
