within SorpLib.Components.MassTransfers.TransportPhenomena.PressureDriven;
model DarcyPorousMedia
  "Mass Transfer | Porous media according to Darcy's law for pressure-driven mass transfer"
  extends BaseClasses.PartiallInternalMassTransfer_dp(
    final calculate_FluidProperties=true);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.Length l_bed = 0.05
    "Thickness of bed"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);
  parameter Modelica.SIunits.Area A_bed = 0.5
    "Cross-sectional area of bed that adsorptive has to pass"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);
  parameter Real permeability(final unit="m2") = 1
    "Permeability of bed"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);

equation
  //
  // Calculation of mass transfer coefficient
  //
  beta = permeability/massTransferProperties.eta_adsorptive * l_bed/2 *
    A_bed/massTransferProperties.d_adsorptive;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructuring of the library.
  </li>
  <li>
  November 30, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
  The convective flow in a porous media can be described by Darcy's law (1856).
</p>
<h4>Main equations</h4>
<p>
  The convective flow can be expressed as:
  <p align=\"center\"> <i>v</i> = - <i>K</i> / <code>&eta;</code> d<i>p</i>
  /d<i>x</i> </p>
  where <i>v</i> is the flow velocity, <i>K</i> is the permeability, 
  <code>&eta;</code> is the dynamic viscosity and d<i>p</i>/d<i>x</i> is the 
  pressure gradient. <br>
  This can be translated in the following equation for the mass flow:
  <p align=\"center\"> <i><code>m&#775;</code></i> = <i>K</i> / 
  <code>&eta;</code> * <i>A</i> * <code>&rho;</code> / (<code>&Delta;</code>
  <i>x</i> / 2) * <code>&Delta;</code><i>p</i> </p>
  where <i>A</i> is the cross-sectional area, <code>&rho;</code> is the density, 
  and <code>&Delta;</code> <i>x</i> is the total bed length.<br>
  For more information see also Bejan (2013).
</p>
<h4>Assumptions and limitations</h4>
  <p>
  xxx
  </p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>Darcy, H.P.G. Les Fontaines Publiques de la villa de Dijon. 
Paris, 1856.</li>
<li>Bejan, A. Convection heat transfer. 4th ed. Hoboken, N.J.: Wiley, 2013. 
ISBN 1118519760.</li>
</ul>
</p>
</html>"));
end DarcyPorousMedia;
