﻿within SorpLib.Components.MassTransfers.TransportPhenomena.PressureDriven;
model LaminarConnections
  "Mass Transfer | Laminar flow trough connection for pressure-driven mass transfer"
  extends BaseClasses.PartiallInternalMassTransfer_dp(
    final calculate_FluidProperties=true);

  //
  // Definition of parameters
  //
  parameter Real zeta_lam(min=1) = 1
    "Additional factor to account for nonideal tubes"
    annotation (Dialog(tab="General", group="Mass transfer"),
                HideResult=true);

  parameter Modelica.SIunits.Length l_connection = 0.5
    "Length of connection"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);
  parameter Modelica.SIunits.Diameter d_connection = 0.4
    "Hydraulic diameter of connection"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);
  parameter Modelica.SIunits.Area A_connection=
    Modelica.Constants.pi*(d_connection/2)^2
    "Inner cross-sectional area of connection"
    annotation (Dialog(tab="General", group="Geometry"),
                HideResult=true);

equation
  //
  // Calculation of mass transfer coefficient
  //
  beta = A_connection^2/(8*Modelica.Constants.pi*l_connection) *
    massTransferProperties.d_adsorptive/massTransferProperties.eta_adsorptive *
    1/zeta_lam;

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 18, 2021, by Mirko Engelpracht:<br/>
  Minor revisions after restructuring of the library.
  </li>
  <li>
  November 30, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
This laminar transfer model calculates the mass flow based on the pressure 
differnce according to the VDI Wärmeatlas (2013).
</p>
<h4>Main equations</h4>
<p>
For a laminar flow, the correlation between mass flow and pressure drop can be 
expressed as:
<p align=\"center\"> <i><code>&Delta;</code>p</i> = <code>&zeta;</code></i>
<sub>lam</sub> * 8 <code>&pi;</code> <i>L</i> / <i>A</i><sup>2</sup> * 
<code>&eta;</code> / <code>&rho;</code> <i><code>m&#775;</code></i> </p>
where <i>A</i> is the cross-sectional area, <code>&eta;</code> is the 
viscosity, <code>&rho;</code> is the density and <code>&zeta;</code></i>
<sub>lam</sub> is an additional factor to account for a non-ideal tube.<br>
  This correlation is implemented as:
  <p align=\"center\"><i><code>m&#775;</code></i> = <i>A</i><sup>2</sup> / 
  (8 <code>&pi;</code> <i>L</i>) * <code>&rho;</code> / <code>&eta;</code> * 1 / 
  <code>&zeta;</code></i><sub>lam</sub> <i><code>&Delta;</code>p</i></p>
</p>
<h4>Assumptions and limitations</h4>
  <p>
  The additional factor <code>&zeta;</code></i><sub>lam</sub> can be 
  experimentally determined as done by Lanzerath (2015).
  </p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
<ul>
<li>VDI e.V., VDI Wärmeatlas 11., bearbeitete und erweiterte Auflage, 
Chapter L1, Darmstadt: Springer Berlin Heidelberg, 2011. </li>
<li>Lanzerath, F.; Bau, U.; Seiler, J.; Bardow, A. Optimal design of 
adsorption chillers based on a validated dynamic object-oriented model. 
Science and Technology for the Built Environment, 2015, 21(3), 248-257. </li>
</ul>
</p>
</html>"));
end LaminarConnections;
