within SorpLib.Components.PhaseSeperators.BaseClasses;
partial model PartialPhaseSeperator "Base model for all phase seperators"

  //
  // Definition of general parameters
  //
  parameter Integer nPorts_Vapor = 0
    "Number of vapor ports"
    annotation (Dialog(connectorSizing=true),
                HideResult = true);
  parameter Integer nPorts_Liquid = 0
    "Number of liquid ports"
    annotation (Dialog(connectorSizing=true),
                HideResult = true);

  parameter Modelica.SIunits.Volume V = 0.005
    "Overall volume of phase seperator"
    annotation (Dialog(group = "Geometry", tab = "General"),
                Evaluate = true);
  parameter Modelica.SIunits.Area A = 0.03
    "Base area of phase seperator"
    annotation (Dialog(group = "Geometry", tab = "General"),
                Evaluate = true,
                HideResult = true);

  //
  // Definition of parameters describing the medium
  //
  parameter Integer no_components = 1
    "Number of components"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate = true,
                HideResult = true);

  //
  // Definition of parameters deschribing the initialisation
  //
  parameter Modelica.SIunits.MassFlowRate m_flowInitial = 1e-4
    "Guess value for mass flow rate"
    annotation (Dialog(tab="Initialisation", group="Start Values"),
                HideResult = true);

  //
  // Definition of parameters describing advanced options
  //
  parameter Internals.Types.BalanceEquations type_overallMassBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the overall mass equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.BalanceEquations type_energyBalance=
    Internals.Types.BalanceEquations.Transient_FixedInitial
    "Handling of the energy equation and corresponding initialisation"
    annotation(Dialog(tab = "Advanced", group = "Conservation Equations"),
                Evaluate=true,
                HideResult = true);

  parameter Boolean use_stateLimiter = true
    "= true, if specific enthalpy of vapour or liquid is limited at ports (i.e.,
    vapour port is slightly overheated and liquid port is slightly supercooled)"
    annotation (Dialog(group = "Limits", tab = "Advanced"),
                Evaluate=true,
                HideResult = true);
  parameter Modelica.SIunits.SpecificEnthalpy dh = 10
    "Specific enthalpy used for superheating and supercooling"
    annotation (Dialog(tab = "Advanced", group = "Limits",
                enable = use_stateLimiter),
                HideResult = true);

  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);

  //
  // Definition of variables
  //
public
  Modelica.SIunits.Pressure p
    "Pressure of fluid in phase seperator";
  Modelica.SIunits.Temperature T
    "Temperature of fluid in phase seperator";
  Modelica.SIunits.Density rho
    "Density of fluid in phase seperator";

  Modelica.SIunits.SpecificEnthalpy h
    "Specific enthalpy of fluid in phase seperator";
  Modelica.SIunits.SpecificEnergy u
    "Specific internal energy of fluid in phase seperator";

  Real q(unit="kg/kg")
    "Vapour mass fraction in phase seperator";

  Modelica.SIunits.Mass mass
    "Overall mass of fluid in phase seperator";
  Modelica.SIunits.Mass mass_liq
    "Liquid mass in phase seperator";
  Modelica.SIunits.Mass mass_vap
    "Vapor mass in phase seperator";

  Modelica.SIunits.InternalEnergy U
    "Internal energy of fluid in phase seperator";

  Modelica.SIunits.Length l_liq
    "Height of liquid in phase seperator";
  Real l_liq_rel
    "Relative height of liquid in phase seperator";

  //
  // Definition of connectors
  //
  Connectors.Thermal.Port_a heatPort
    "Heat port of cell"
    annotation (Placement(transformation(extent={{50,-10},{70,10}}),
                iconTransformation(extent={{50,-10},{70,10}})));

  replaceable Connectors.BaseClasses.PartialMediaConnector[nPorts_Liquid] liquidPort(
    each final no_components=no_components,
    each m_flow(start = m_flowInitial))
    "Inlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-10,-100},{10,-80}}),
        iconTransformation(extent={{-10,-100},{10,-80}})));
  replaceable Connectors.BaseClasses.PartialMediaConnector[nPorts_Vapor] vaporPort(
    each final no_components=no_components,
    each m_flow(start = -m_flowInitial))
    "Outlet fluid port at design direction"
    annotation (Placement(transformation(extent={{-10,78},{10,98}}),
        iconTransformation(extent={{-10,78},{10,98}})));

equation
  //
  // Assertations
  //
  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    assert(type_overallMassBalance ==
      Internals.Types.BalanceEquations.SteadyState_FreeInitial,
      "Steady-state energy balance is only allowed in combination with
       steady-state mass balance!");
  end if;

  //
  // Momentum balance
  //
  for i in 1:nPorts_Liquid loop
    liquidPort[i].p = p
      "Homogeneous phase seperator: No pressure loss within model";
  end for;

  for i in 1:nPorts_Vapor loop
    vaporPort[i].p = p
      "Homogeneous phase seperator: No pressure loss within model";
  end for;

  //
  // Energy balance
  //
  heatPort.T = T
    "Set temperature of heat port";

  //
  // Calculation of further variables
  //
  l_liq_rel = l_liq / (V/A)
    "Relativ liquid filling height";

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={Polygon(
          points={{-60,-14},{-60,6},{-40,-6},{-20,-6},{-6,0},{14,-10},{36,0},{50,
              -2},{60,-10},{-60,-14}},
          lineColor={28,108,200},
          lineThickness=0.5,
          smooth=Smooth.Bezier,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{-60,-4},{60,-90}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{-60,90},{60,-90}},
          lineColor={0,0,0},
          lineThickness=0.5),
        Ellipse(
          extent={{-36,84},{-48,72}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-26,64},{-38,52}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-10,82},{-22,70}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-2,62},{-14,50}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{18,84},{6,72}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{26,66},{14,54}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{42,84},{30,72}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{52,70},{40,58}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{-44,66},{-56,54}},
          lineColor={28,108,200},
          lineThickness=0.5,
          fillColor={28,108,200},
          fillPattern=FillPattern.Solid),
        Polygon(
          points={{-40,-4},{-40,-4},{-52,10},{-28,10},{-40,-4}},
          lineColor={0,0,0},
          lineThickness=0.5)}),                                  Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  January 13, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
xxx
</p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
<p>
xxx
</p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialPhaseSeperator;
