﻿within SorpLib.Components.PhaseSeperators.BaseClasses;
partial model PartialSinglePhaseSeperator
  "Base model for all phase seperators of a pure fluid"
  extends PartialPhaseSeperator(
    T(start=TInitial, stateSelect=StateSelect.prefer),
    rho(start=rhoInitial, stateSelect=StateSelect.prefer));

  //
  // Definition of parameters deschribing the initialisation
  //
  parameter Modelica.SIunits.Temperature TInitial= 293.15
    "Initial value for temperature"
    annotation (Dialog(tab="Initialisation", group="Initial Values"),
                HideResult = true);
  parameter Modelica.SIunits.Density rhoInitial= 500
    "Initial value for density"
    annotation (Dialog(tab="Initialisation", group="Initial Values"),
                HideResult = true);

  //
  // Definition of variables
  //
protected
  Real dudrho(unit="J.m3/kg2")
    "Derivative of internal energy of fluid with respect to density"
    annotation (HideResult=true);

  Modelica.SIunits.SpecificHeatCapacity cv
    "Specific heat capacity of fluid in phase seperator"
    annotation (HideResult=true);

  Modelica.SIunits.Density rho_liq
    "Density at saturated liquid line"
    annotation (HideResult=true);
  Modelica.SIunits.Density rho_vap
    "Density at saturated vapor line"
    annotation (HideResult=true);

public
  Modelica.SIunits.SpecificEnthalpy h_liq
    "Specific enthalpy at saturated liquid line";
  Modelica.SIunits.SpecificEnthalpy h_vap
    "Specific enthalpy at saturated vapor line";

initial equation
  if type_overallMassBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    rho = rhoInitial
      "Fixed initial value";

  elseif type_overallMassBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(rho) = 0
      "Steady-state initialisation of mass balance";

  end if;

  if type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_FixedInitial then
    T = TInitial
      "Fixed initial value";

  elseif type_energyBalance ==
    Internals.Types.BalanceEquations.Transient_SteadyStateInitial then
    der(T) = 0
      "Steady-state initialisation of energy balance";

  end if;

equation
  //
  // Mass balance
  //
  mass = rho*V
    "Overall mass in phase seperator";

  mass_liq = (1-q) * mass
    "Liquid mass in phase seperator";
  mass_vap = q * mass
    "Vapor mass in phase seperator";

  if type_overallMassBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    0 = sum(liquidPort[:].m_flow) + sum(vaporPort[:].m_flow)
      "Steady-state mass balance";

  else
    V*der(rho) = sum(liquidPort[:].m_flow) + sum(vaporPort[:].m_flow)
      "Transient mass balance";

  end if;

  //
  // Energy balance
  //
  u = h - p/rho
    "Specific enthalpy of fluid in phase seperator";
  U = mass*u
    "Internal energy of fluid in phase seperator";

  if type_energyBalance ==
    Internals.Types.BalanceEquations.SteadyState_FreeInitial then
    if avoid_events then
      0 = sum(vaporPort.m_flow.*noEvent(actualStream(vaporPort.h_outflow))) +
        sum(liquidPort.m_flow.*noEvent(actualStream(liquidPort.h_outflow))) +
        heatPort.Q_flow
        "Steady-state energy balance";

    else
      0 = sum(vaporPort.m_flow.*actualStream(vaporPort.h_outflow)) +
        sum(liquidPort.m_flow.*actualStream(liquidPort.h_outflow)) +
        heatPort.Q_flow
        "Steady-state energy balance";

    end if;

  else
    if avoid_events then
      u*V*der(rho) + mass*(dudrho*der(rho) + cv*der(T)) =
        sum(vaporPort.m_flow.*noEvent(actualStream(vaporPort.h_outflow))) +
        sum(liquidPort.m_flow.*noEvent(actualStream(liquidPort.h_outflow))) +
        heatPort.Q_flow
        "Transient energy balance";

    else
      u*V*der(rho) + mass*(dudrho*der(rho) + cv*der(T)) =
        sum(vaporPort.m_flow.*actualStream(vaporPort.h_outflow)) +
        sum(liquidPort.m_flow.*actualStream(liquidPort.h_outflow)) +
        heatPort.Q_flow
        "Transient energy balance";

    end if;

  end if;

  dudrho = 1/rho^2 * (-p + (h_vap - h_liq) / (1/rho_vap - 1/rho_liq))
    "Calculation of du/drho using Bridgeman table and Clausius-Clayperon equation";

  for i in 1:nPorts_Liquid loop
    liquidPort[i].h_outflow = if use_stateLimiter then
      h_liq - dh else h_liq
      "Homogeneous phase seperator";
  end for;

  for i in 1:nPorts_Vapor loop
    vaporPort[i].h_outflow = if use_stateLimiter then
      h_vap + dh else h_vap
      "Homogeneous phase seperator";
  end for;

  //
  // Calculation of further variables
  //
  q = (1/rho - 1/rho_liq) / (1/rho_vap - 1/rho_liq)
    "Vapor mass fraction";

  l_liq = mass_liq / (rho_liq*A)
    "Liquid filling height";

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  January 13, 2021, by Mirko Engelpracht:<br/>
  Major revisions (e.g., object-oriented approach, options) after restructuring of the library.
  </li>
  <li>
  November 22, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
  <p>
    This model represents an ideal vapour liquid equilibrium (VLE) phase separator. The model has three ports: a VLE vapour port, a VLE liquid port, and a heat port. <br>
    
  </p>
<h4>Main equations</h4>
  <p>
    <p>Mass balance: </p>
    <p align=\"center\"><i>V</i> <i>d<code>&rho;</code>/dt</i> = <code>&sum;</code> <i><code>&#7745;</code></i> </p>
    <p>where <i>V</i> is the volume, <i>&rho;</i> is the density of the VLE fluid, and <i><code>&#7745;</code></i> are the mass flow rates at the VLE vapour ports, and the VLE liquid ports.</p>
    <p>Energy balance: </p>
    <p align=\"center\"><i>V u</i> <i>d<code>&rho;</code>/dt</i> + <i>m</i> <i>c</i><sub>v</sub> <i>dT/dt</i> + <i>m du/d<code>&rho;</code> d<code>&rho;</code>/dt</i> = 
    <code>&sum;</code> <i><code>&#7745;</code> h </i> + <code>&sum;</code> <i>Q</i> </p>
    <p>where <i>c</i><sub>v</sub> is the specific heat capacity at constant volume, <i>u</i> is the specific internal energy at the current density and temperature, <i>h</i> is the specific enthalpy of the incoming and outflowing VLE streams, and <i>Q</i> is the heat flow.</p>
    <p>The partial derivative <i>du/d<code>&rho;</code></i> can be expressed as:</p>
    <p align=\"center\"><i>du/d<code>&rho;</code></i> = 1/<i><code>&rho;</code></i><sup>2</sup> <i>( -p + T dpdT )</i> </p>
    <p>where <i>dpdT</i> can be substituted with the equation of Clausius-Clayperon: <i>dpdT = (h<sup>v</sup> - h<sup>l</sup>) / (T (v<sup>v</sup> - v<sup>l</sup>)</i>. <br>
    For more information see Gräber (2011) and Tummescheit (2002). <br></p>
    The VLE phase seperator has two VLE ports, a liquid and a vapour VLE port. At the liquid VLE port, the specific enthalpy of the leaving fluid is slightly subccoled, at the vapour VLE port, the specific enthalpy of the leaving fluid is slightly superheated.
  </p>
<h4>Assumptions and limitations</h4>
  <p>
    The model uses the following main assumptions:
    <ul>
      <li>Liquid and vapour phase are in chemical equilibrium --> same pressure and temperature</li>
    </ul>
  </p>
<h4>Typical use and important parameters</h4>
  <p>
    For adsorption systems, the VLE phase separator is a core part of the the evaporator/condenser component.
  </p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
  <p>
    The model has two states: the temperature <i>T</i> and the density <i><code>&rho;</code></i>.
  </p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
  <p>
    <ul>
      <li>Tummescheit, H. Design and Implementation of Object-Oriented Model Libraries using Modelica. PhD Thesis. Lund, 2002.</li>
      <li>Gräber, M.; Kirches, C.; Bock, H.G.; Schlöder, J.P.; Tegethoff, W.; Köhler, J. Determining the optimum cyclic operation of adsorption chillers by a direct method for periodic optimal control. Int. J. Refrig., 2011, 34(4), 902-913.</li>
    </ul>
  </p>
</html>"));
end PartialSinglePhaseSeperator;
