within SorpLib.Components.Valves.BaseClasses;
partial model PartialValve_constantFillingLevel
  "Base model for all valves ensuring a constant filling level at port B"
  extends PartialValve;

  //
  // Definition of parameters
  //
  parameter Real kp(final unit="kg/s") = 0.15
    "Linear valve coefficient for mass transfer"
    annotation (Dialog(tab="General", group="Valve characteristics"),
                HideResult=true);
  parameter Real setPoint_fillingLevel(final unit="1") = 0.25
    "Set point for disired relative filling level of component at port B"
    annotation (Dialog(tab="General", group="Valve characteristics"),
                HideResult=true);
  parameter Boolean restrictFlow_BA = false
    "= true, if mass flow rate is only allowed from port B to A"
    annotation (Dialog(tab="General", group="Valve characteristics"),
                evaluate=true,
                HideResult=true);

  //
  // Definition of inputs
  //
  input Real fillingLevel(final unit="1")
    "Current relative filling level of component at port B"
    annotation(Dialog(tab="General", group="Inputs"));

equation
  //
  // Calculation of mass flow rate
  //
  if restrictFlow_BA then
    port_a.m_flow = smooth(1, min(setPoint_fillingLevel-fillingLevel, 0))*kp
      "Linear change of mass flow rate: Only flow from B to A allowed!";

  else
    port_a.m_flow = (setPoint_fillingLevel-fillingLevel)*kp
      "Linear change of mass flow rate";

  end if;

  //
  // Annotations
  //
  annotation (Diagram(graphics={Text(
          extent={{-80,60},{-40,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          textString="Desired relative
filling level at
port B"),     Line(
          points={{-60,28},{-80,0}},
          color={0,0,0},
          thickness=0.5,
          arrow={Arrow.None,Arrow.Filled})}), Documentation(revisions="<html>
<ul>
  <li>
  January 20, 2021, by Mirko Engelpracht:<br/>
  Smaller revisions after restructuring of the library.
  </li>
  <li>
  December 11, 2017, by Uwe Bau:<br/>
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
  <p>
    The VLE valve with constant filling level model extends the <a href=\"modelica://SorpLib.Components.Valves.VLE_Valves.Partial.PartialValve\">SorpLib.Components.Valves.VLE_Valves.Partial.PartialValve</a>.<br>
    The VLE valve with constant filling level model determines the mass flow, so that the filling level in one VLE volume model is constant.<br>
  </p>
<h4>Main equations</h4>
  <p>
    The mass flow <code>m&#775;</code> is proportional to the difference between current filling level <i>l</i> and set point filling level <i>l</i><sub>set</sub>:
    <p align=\"center\"><i><code>m&#775;</code></i><sub>in</sub> = <i>k</i> ( <i>l</i> - <i>l</i><sub>set</sub> ).</p>
    with a linear mass transfer coefficient <i>k</i>.
  </p>
<h4>Assumptions and limitations</h4>
<p>
xxx
</p>
<h4>Typical use and important parameters</h4>
  <p>
    The VLE valve with constant filling level is mainly used to connect the evaporator and the condenser in an adsorption chiller model.
  </p>
<h4>Options</h4>
<p>
xxx
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>Implementation</h4>
<p>
xxx
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialValve_constantFillingLevel;
