within SorpLib.Media.Function.SingleComponent.AdsorptionEnthalpy.Tester;
model TestDhAds_ClausiusClapeyron
  "Model to test Clausius-Clapeyron approach for calculating adsorption enthalpy"
  extends Modelica.Icons.Example;

  //
  // Definition of temperature
  //
  final parameter Real[3] c={7.94*44.0098/1000,3.22e-10,-2.35e4}
    "Coefficients for Langmuir isotherm (qm, b0, dH)";
  final parameter Modelica.SIunits.Pressure M = 0.018015
    "Molar mass of sorptive";

  //
  // Definition of variables
  //
protected
  Real[2] c_Langmuir
    "Temperature-dependent coefficients of Langmuir isotherm";
  Real[2] dc_Langmuir_dT
    "Derivative of temperature-dependent coefficients of Langmuir isotherm wrt.
     temperature";

public
  Modelica.SIunits.Pressure p_adsorpt
    "Adsorption pressure";
  Modelica.SIunits.Temperature T_adsorpt
    "Adsorption temperature";
  Real x_adsorpt(unit="1")
    "Equilibrium loading";

  Real dx_adsorpt_dp(unit="kg/(kg.Pa)")
    "Derivative of loading wrt. pressure";
  Real dx_adsorpt_dT(unit="kg/(kg.K)")
    "Derivative of loading wrt. temperature";

  Modelica.SIunits.SpecificEnthalpy dhAds
    "Specific enthalpy of adsorption";

equation
  //
  // Calculate temperature-dependent coefficients
  //
  c_Langmuir = {c[1],
                c[2]*Modelica.Math.exp(-c[3]/Modelica.Constants.R/T_adsorpt)}
    "Temperature-dependent coefficients of Langmuir isotherm";
  dc_Langmuir_dT = {0,
                    c[3]/(Modelica.Constants.R*T_adsorpt^2) * c_Langmuir[2]}
    "Temperature-dependent coefficients of Langmuir isotherm";

  //
  // Calculate equilibrium properties
  //
  p_adsorpt = time*0.85e4
    "Define equilibrium pressure";
  T_adsorpt = 293.15 + 0.5 * time
    "Equilibrium temperature";
  x_adsorpt =
    Equilibrium.SurfaceApproach.LangmuirIsotherm(p_adsorpt, c_Langmuir)
    "Calculate equilibrium loading";

  dx_adsorpt_dp =
    Equilibrium.SurfaceApproach.LangmuirIsotherm__dxdp(
    p_adsorpt, c_Langmuir)
    "Derivative of loading wrt. pressure";
  dx_adsorpt_dT =
    Equilibrium.SurfaceApproach.LangmuirIsotherm__dxdT(
    p_adsorpt, c_Langmuir, dc_Langmuir_dT)
    "Derivative of loading wrt. temperature";

  //
  // Calculation of specific enthalpy of adsorption
  //
  dhAds =dhAds_ClausiusClapeyron(
        Modelica.Constants.R/M,
        p_adsorpt,
        T_adsorpt,
        dx_adsorpt_dT,
        dx_adsorpt_dp)
    "Calculating adsorption enthalpy";

  //
  // Annotations
  //
  annotation (Documentation(info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>", revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>"));
end TestDhAds_ClausiusClapeyron;
