within SorpLib.Media.Function.SingleComponent.AdsorptionEnthalpy.Tester;
model TestDhAds_ClausiusClapeyron_ddhadsdT_num "Model to test Clausius-Clapeyron approach for calculating derivative of 
   adsorption enthalpy with respect to temperature"
  extends Modelica.Icons.Example;

  //
  // Definition of temperature
  //
  final parameter Real[5] c={1.106e-11*0.01802,4.714e-13,9.955e3,
    3.548e-1,-5.114e1}
    "Coefficients for Toth isotherm (a0, b0, E, t0, c)";
  final parameter Modelica.SIunits.Pressure M = 0.01802
    "Molar mass of sorptive";

  //
  // Definition of variables
  //
  Modelica.SIunits.Pressure p_adsorpt
    "Equilibrium pressure";

  Modelica.SIunits.Temperature T_adsorpt = if noEvent((time<=50)) then
    273.15 + 1*time else 273.15+100 - 1*time
    "Equilibrium temperature";
protected
  Modelica.SIunits.Temperature T_adsorpt_pdT = T_adsorpt + 1e-3
    "Equilibrium temperature + dT";
  Modelica.SIunits.Temperature T_adsorpt_mdT = T_adsorpt - 1e-3
    "Equilibrium temperature - dT";

  Modelica.SIunits.Temperature T_adsorpt_pdT_pdT = T_adsorpt + 2e-3
    "Equilibrium temperature + dT + dT";
  Modelica.SIunits.Temperature T_adsorpt_mdT_mdT = T_adsorpt - 2e-3
    "Equilibrium temperature - dT";

public
  Real x_adsorpt(unit="1")
    "Equilibrium loading";

protected
  Real a(unit="kg/(kg.Pa)")
    "Coefficient a of Toth isotherm";
  Real b(unit="1/Pa")
    "Coefficient b of Toth isotherm";
  Real t(unit="1")
    "Coefficient t of Toth isotherm";

public
  Modelica.SIunits.SpecificEnthalpy dhAds
    "Specific enthalpy of adsorption";
protected
  Modelica.SIunits.SpecificEnthalpy dhAds_pdT
    "Specific enthalpy of adsorption at T+dT";
  Modelica.SIunits.SpecificEnthalpy dhAds_mdT
    "Specific enthalpy of adsorption at T-dT";

public
  Real ddh_ads_dT(unit="J/(kg.K)")
    "Derivative of adsorption enthalpy wrt temperature";
  Real ddh_ads_dT_num(unit="J/(kg.K)")
    "Numerical derivative of adsorption enthalpy wrt temperature";

protected
  Modelica.SIunits.Pressure p_adsorpt_mdT
    "Equilibrium pressure at T-dT";
  Modelica.SIunits.Pressure p_adsorpt_mdT_mdT
    "Equilibrium pressure at T-dT-dT";

  Modelica.SIunits.Pressure p_adsorpt_pdT
    "Equilibrium pressure at T+dT";
  Modelica.SIunits.Pressure p_adsorpt_pdT_pdT
    "Equilibrium pressure at T+dT+dT";

  Real a_pdT(unit="kg/(kg.Pa)")
    "Coefficient a of Toth isotherm at T+dT";
  Real b_pdT(unit="1/Pa")
    "Coefficient b of Toth isotherm at T+dT";
  Real t_pdT(unit="1")
    "Coefficient t of Toth isotherm at T+dT";

  Real a_pdT_pdT(unit="kg/(kg.Pa)")
    "Coefficient a of Toth isotherm at T+dT+dT";
  Real b_pdT_pdT(unit="1/Pa")
    "Coefficient b of Toth isotherm at T+dT+dT";
  Real t_pdT_pdT(unit="1")
    "Coefficient t of Toth isotherm at T+dT+dT";

  Real a_mdT(unit="kg/(kg.Pa)")
    "Coefficient a of Toth isotherm at T-dT";
  Real b_mdT(unit="1/Pa")
    "Coefficient b of Toth isotherm at T-dT";
  Real t_mdT(unit="1")
    "Coefficient t of Toth isotherm at T-dT";

  Real a_mdT_mdT(unit="kg/(kg.Pa)")
    "Coefficient a of Toth isotherm at T-dT-dT";
  Real b_mdT_mdT(unit="1/Pa")
    "Coefficient b of Toth isotherm at T-dT-dT";
  Real t_mdT_mdT(unit="1")
    "Coefficient t of Toth isotherm at T-dT-dT";

  Real dadT_mdT(unit="kg/(kg.Pa.K)")
    "Derivative of coefficient a of Toth isotherm wrt temperature_mdT";
  Real dbdT_mdT(unit="1/(Pa.K)")
    "Derivative of coefficient b of Toth isotherm wrt temperature_mdT";
  Real dtdT_mdT(unit="1/K")
    "Derivative of coefficient t of Toth isotherm wrt temperature_mdT";

  Real dadT_pdT(unit="kg/(kg.Pa.K)")
    "Derivative of coefficient a of Toth isotherm wrt temperature_pdT";
  Real dbdT_pdT(unit="1/(Pa.K)")
    "Derivative of coefficient b of Toth isotherm wrt temperature_pdT";
  Real dtdT_pdT(unit="1/K")
    "Derivative of coefficient t of Toth isotherm wrt temperature_pdT";

equation
  //
  // Calculate equilibrium properties
  //
  p_adsorpt = 1e-12 + time*2930
    "Define equilibrium pressure";

  a = c[1] * Modelica.Math.exp(c[3]/T_adsorpt)
    "Calculate first coefficient of Toth isotherm";
  b = c[2] * Modelica.Math.exp(c[3]/T_adsorpt)
    "Calculate second coefficient of Toth isotherm";
  t = c[4] + c[5]/T_adsorpt
    "Calculate third coefficient of Toth isotherm";

  a_pdT = c[1] * Modelica.Math.exp(c[3]/T_adsorpt_pdT)
    "Calculate first coefficient of Toth isotherm at T+dT";
  b_pdT = c[2] * Modelica.Math.exp(c[3]/T_adsorpt_pdT)
    "Calculate second coefficient of Toth isotherm at T+dT";
  t_pdT = c[4] + c[5]/T_adsorpt_pdT
    "Calculate third coefficient of Toth isotherm at T+dT";

  a_pdT_pdT = c[1] * Modelica.Math.exp(c[3]/T_adsorpt_pdT_pdT)
    "Calculate first coefficient of Toth isotherm at T+dT+dT";
  b_pdT_pdT = c[2] * Modelica.Math.exp(c[3]/T_adsorpt_pdT_pdT)
    "Calculate second coefficient of Toth isotherm at T+dT+dT";
  t_pdT_pdT = c[4] + c[5]/T_adsorpt_pdT_pdT
    "Calculate third coefficient of Toth isotherm at T+dT+dT";

  a_mdT = c[1] * Modelica.Math.exp(c[3]/T_adsorpt_mdT)
    "Calculate first coefficient of Toth isotherm at T-dT";
  b_mdT = c[2] * Modelica.Math.exp(c[3]/T_adsorpt_mdT)
    "Calculate second coefficient of Toth isotherm at T-dT";
  t_mdT = c[4] + c[5]/T_adsorpt_mdT
    "Calculate third coefficient of Toth isotherm at T-dT";

  a_mdT_mdT = c[1] * Modelica.Math.exp(c[3]/T_adsorpt_mdT_mdT)
    "Calculate first coefficient of Toth isotherm at T-dT-dT";
  b_mdT_mdT = c[2] * Modelica.Math.exp(c[3]/T_adsorpt_mdT_mdT)
    "Calculate second coefficient of Toth isotherm at T-dT-dT";
  t_mdT_mdT = c[4] + c[5]/T_adsorpt_mdT_mdT
    "Calculate third coefficient of Toth isotherm at T-dT-dT";

  x_adsorpt =SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm(
    p_adsorpt, {a,b,t})
    "Calculate equilibrium loading";

  p_adsorpt_mdT =
    SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm_inverse(
    x_adsorpt, {a_mdT,b_mdT,t_mdT})
    "Calculate equilibrium pressure with inverse of Toth isotherm at T-dT";
  p_adsorpt_mdT_mdT =
    SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm_inverse(
    x_adsorpt, {a_mdT_mdT,b_mdT_mdT,t_mdT_mdT})
    "Calculate equilibrium pressure with inverse of Toth isotherm at T-dT";

  p_adsorpt_pdT =
    SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm_inverse(
    x_adsorpt, {a_pdT,b_pdT,t_pdT})
    "Calculate equilibrium pressure with inverse of Toth isotherm at T+dT";
  p_adsorpt_pdT_pdT =
    SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm_inverse(
    x_adsorpt, {a_pdT_pdT,b_pdT_pdT,t_pdT_pdT})
    "Calculate equilibrium pressure with inverse of Toth isotherm at T+dT";

  dhAds =dhAds_ClausiusClapeyron_num(
    Modelica.Constants.R/M,
    p_adsorpt_mdT,
    p_adsorpt_pdT,
    T_adsorpt_mdT,
    T_adsorpt_pdT)
    "Calculating adsorption enthalpy";

  dhAds_mdT =dhAds_ClausiusClapeyron_num(
    Modelica.Constants.R/M,
    p_adsorpt_mdT_mdT,
    p_adsorpt,
    T_adsorpt_mdT_mdT,
    T_adsorpt)
    "Calculating adsorption enthalpy at T-dT";

  dhAds_pdT =dhAds_ClausiusClapeyron_num(
    Modelica.Constants.R/M,
    p_adsorpt,
    p_adsorpt_pdT_pdT,
    T_adsorpt,
    T_adsorpt_pdT_pdT)
    "Calculating adsorption enthalpy at T+dT";

  //
  // Calculation derivative of specific enthalpy of adsorption wrt temperature
  //
  dadT_mdT = -a_mdT * c[3]/T_adsorpt_mdT^2
    "Calculate derivative of coefficient a of Toth isotherm wrt temperature_mdT";
  dbdT_mdT =  -b_mdT * c[3]/T_adsorpt_mdT^2
    "Calculate derivative of coefficient b of Toth isotherm wrt temperature_mdT";
  dtdT_mdT = -c[5]/T_adsorpt_mdT^2
    "Calculate derivative of coefficient t of Toth isotherm wrt temperature_mdT";

  dadT_pdT = -a_pdT * c[3]/T_adsorpt_pdT^2
    "Calculate derivative of coefficient a of Toth isotherm wrt temperature_pdT";
  dbdT_pdT = -b_pdT * c[3]/T_adsorpt_pdT^2
    "Calculate derivative of coefficient b of Toth isotherm wrt temperature_pdT";
  dtdT_pdT = -c[5]/T_adsorpt_pdT^2
    "Calculate derivative of coefficient t of Toth isotherm wrt temperature_pdT";

  ddh_ads_dT =dhAds_ClausiusClapeyron_ddhAdsdT_num(
        Modelica.Constants.R/M,
        p_adsorpt_mdT,
        p_adsorpt_pdT,
        T_adsorpt_mdT,
        T_adsorpt_pdT,
        Equilibrium.SurfaceApproach.TothIsotherm_inverse__dpdT(
          x_adsorpt,
          {a_mdT,b_mdT,t_mdT},
          {dadT_mdT,dbdT_mdT,dtdT_mdT})*1,
        Equilibrium.SurfaceApproach.TothIsotherm_inverse__dpdT(
          x_adsorpt,
          {a_pdT,b_pdT,t_pdT},
          {dadT_pdT,dbdT_pdT,dtdT_pdT})*1,
        1,
        1)
    "Calculate derivative of adsorption enthalpy wrt temperature";

  ddh_ads_dT_num = (dhAds_pdT-dhAds_mdT) / 2e-3
    "Calculate numerical derivative of adsorption enthalpy wrt temperature";

  //
  // Annotations
  //
  annotation (experiment(StopTime=100), Documentation(info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>", revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>"));
end TestDhAds_ClausiusClapeyron_ddhadsdT_num;
