within SorpLib.Media.Function.SingleComponent.AdsorptionEnthalpy;
function dhAds_ClausiusClapeyron_ddhAdsdT_num
  "Function that calculates derivative of adsorption enthalpy using 
   Clausius-Clapeyron approach and method of finite differences with respect to
   temperature"
  extends Modelica.Icons.Function;

  //
  // Definition of inputs
  //
  input Modelica.SIunits.SpecificHeatCapacity R_adsorpt
    "Specific gas constant of adsorpt"
    annotation (Dialog(tab="General", group="Inputs"));

  input Modelica.SIunits.Pressure p_mdT
    "Pressure of adsorpt for x_adsorpt and T_mdT"
    annotation (Dialog(tab="General", group="Inputs"));
  input Modelica.SIunits.Pressure p_pdT
    "Pressure of adsorpt for x_adsorpt and T_pdT"
    annotation (Dialog(tab="General", group="Inputs"));
  input Modelica.SIunits.Temperature T_mdT
    "Temperature of adsorpt: (1-dT)*T_adsorpt"
    annotation (Dialog(tab="General", group="Inputs"));
  input Modelica.SIunits.Temperature T_pdT
    "Temperature of adsorpt: (1+dT)*T_adsorpt"
    annotation (Dialog(tab="General", group="Inputs"));

  input Real dp_mdT_dT(unit="Pa/K")
    "Derivative of pressure of adsorpt for x_adsorpt and T_mdT wrt temperature"
    annotation (Dialog(tab="General", group="Inputs"));
  input Real dp_pdT_dT(unit="Pa/K")
    "Derivative of pressure of adsorpt for x_adsorpt and T_pdT wrt temperature"
    annotation (Dialog(tab="General", group="Inputs"));
  input Real dT_mdT_dT(unit="1")
    "Derivative of pemperature of adsorpt: (1-dT)*T_adsorpt wrt temperature"
    annotation (Dialog(tab="General", group="Inputs"));
  input Real dT_pdT_dT(unit="1")
    "Derivative of pemperature of adsorpt: (1+dT)*T_adsorpt wrt temperature"
    annotation (Dialog(tab="General", group="Inputs"));

  //
  // Definition of outputs
  //
  output Real ddh_ads_dT(unit="J/(kg.K)")
    "Derivative of adsorption enthalpy wrt temperature"
    annotation (Dialog(tab="General", group="Outputs", enable=false));

  //
  // Definition of variables
  //
protected
  Real ddh_ads_dp_mdT(unit="K/Pa")
    "Derivative of adsorption enthalpy wrt p_mdT";
  Real ddh_ads_dp_pdT(unit="K/Pa")
    "Derivative of adsorption enthalpy wrt p_pdT";
  Real ddh_ads_dT_mdT(unit="1")
    "Derivative of adsorption enthalpy wrt T_mdT";
  Real ddh_ads_dT_pdT(unit="1")
    "Derivative of adsorption enthalpy wrt T_pdT";

algorithm
  //
  // Calculate partial derivatives
  //
  ddh_ads_dp_mdT := T_mdT*T_pdT / ((T_pdT-T_mdT) * p_mdT)
    "Calculate derivative of adsorption enthalpy wrt p_mdT";
  ddh_ads_dp_pdT := -T_mdT*T_pdT / ((T_pdT-T_mdT) * p_pdT)
    "Calculate derivative of adsorption enthalpy wrt p_pdT";
  ddh_ads_dT_mdT := log(p_mdT/p_pdT) * T_pdT^2 / (T_mdT-T_pdT)^2
    "Calculate derivative of adsorption enthalpy wrt T_mdT";
  ddh_ads_dT_pdT := -log(p_mdT/p_pdT) * T_mdT^2 / (T_pdT-T_mdT)^2
    "Calculate derivative of adsorption enthalpy wrt T_pdT";

  //
  // Calculate derivative of adsorption enthalpy wrt temperature
  //
  ddh_ads_dT := -R_adsorpt *(ddh_ads_dp_mdT*dp_mdT_dT +
    ddh_ads_dp_pdT*dp_pdT_dT + ddh_ads_dT_mdT*dT_mdT_dT +
    ddh_ads_dT_pdT*dT_pdT_dT)
   "Calculation of derivative of adsorption enthalpy wrt temperature";

  //
  // Annotations
  //
  annotation (Documentation(info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>", revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>"));
end dhAds_ClausiusClapeyron_ddhAdsdT_num;
