within SorpLib.Media.Function.SingleComponent.BaseClasses;
partial model PartialCharCurve_Summary
  "Base models for all models that summarise functions of the characteristic curve"
  extends Modelica.Icons.MaterialProperty;

  //
  // Definition of inputs
  //
  input Modelica.SIunits.Temperature T_adsorpt
    "Temperature of adsorpt"
    annotation(Dialog(group = "Inputs", tab = "Interfaces"));
  input Real beta_adsorpt(final unit="1/K")
    "Liquid isobaric expansion coefficient"
    annotation(Dialog(group = "Inputs", tab = "Interfaces"));

  input Modelica.SIunits.SpecificEnergy A_input
    "Adsorption potential"
    annotation(Dialog(group = "Inputs", tab = "Interfaces",
               enable=type_charCurve==Internals.Types.Isotherm.Classical));
  input Modelica.SIunits.SpecificVolume W_input
    "Specific volume of adsorbate"
    annotation(Dialog(group = "Inputs", tab = "Interfaces",
               enable=not type_charCurve==Internals.Types.Isotherm.Classical));

  //
  // Definition of parameters
  //
  parameter Internals.Types.Isotherm type_charCurve=
    Internals.Types.Isotherm.Classical
    "Form of characteristic curve"
    annotation (Dialog(tab="General", group="General"),
                Evaluate=true,
                HideResult = true);
  parameter Real[:] c={3.466e-4, 3.094e5, -9.765e4, 7.312e-7}
    "Coefficients for characteristic curve"
    annotation(Dialog(tab = "General", group = "General"),
               HideResult=true);

  parameter Modelica.SIunits.SpecificEnergy AInitial = 1e5
    "Initial value for adsorption potential"
    annotation(Dialog(group = "Initialisation", tab = "Initialisation"),
               HideResult=true);
  parameter Modelica.SIunits.SpecificVolume WInitial = 5e-5
    "Initial value for specific volume of adsorbate"
    annotation(Dialog(group = "Initialisation", tab = "Initialisation"),
               HideResult=true);

  parameter Boolean calculate_dh_bond = true
    "=true, if specific bonding enthalpy is calculated"
    annotation(Dialog(group = "Calculations", tab = "Advanced"),
               HideResult=true);

  //
  // Definition of outputs
  //
  output Modelica.SIunits.SpecificEnergy A_output(start=AInitial)
    "Adsorption potential"
    annotation(Dialog(group = "Outputs", tab = "Interfaces", enable = false),
               HideResult=type_charCurve==Internals.Types.Isotherm.Classical);
  output Real W_output(final quantity="SpecificVolume", final unit="m3/kg",
    min=-1, start=WInitial)
    "Specific volume of adsorbate"
    annotation(Dialog(group = "Outputs", tab = "Interfaces", enable = false),
               HideResult=not type_charCurve==
               Internals.Types.Isotherm.Classical);
  output Modelica.SIunits.SpecificEnthalpy dh_bond
    "Bonding enthalpy"
    annotation(Dialog(group = "Outputs", tab = "Interfaces", enable=false),
               HideResult=not calculate_dh_bond);

  //
  // Definition of internal variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput A_input_(final unit="J/kg") = A_input if
    type_charCurve == Internals.Types.Isotherm.Classical
    "Adsorption potential used for internal calculations";
  Modelica.Blocks.Interfaces.RealInput A_int_(final unit="J/kg") = -1 if not
    type_charCurve == Internals.Types.Isotherm.Classical
    "Adsorption potential used for internal calculations";
  Modelica.Blocks.Interfaces.RealOutput A_int(final unit="J/kg")
    "Adsorption potential used for internal calculations";

  Modelica.Blocks.Interfaces.RealInput W_input_(final unit="m3/kg") = W_input if not
    type_charCurve == Internals.Types.Isotherm.Classical
    "Specific volume of adsorbate used for internal calculations";
  Modelica.Blocks.Interfaces.RealInput W_int_(final unit="m3/kg") = -1 if
    type_charCurve == Internals.Types.Isotherm.Classical
    "Specific volume of adsorbate used for internal calculations";
  Modelica.Blocks.Interfaces.RealOutput W_int(final unit="m3/kg")
    "Specific volume of adsorbate used for internal calculations";

equation
  //
  // Connections required for correct usesage of conditional removed variables
  //
  connect(A_int, A_input_);
  connect(A_int, A_int_);
  connect(W_int, W_input_);
  connect(W_int, W_int_);

  //
  // Annotations
  //
  annotation (Icon(graphics={Text(
          extent={{-80,80},{80,-80}},
          lineColor={0,0,0},
          textString="W
A
dh_bond"), Text(
          extent={{-100,-120},{100,-140}},
          lineColor={0,0,0},
          textString="%name")}), Documentation(revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialCharCurve_Summary;
