within SorpLib.Media.Function.SingleComponent.Equilibrium.SurfaceApproach.Tester;
model TestLangmuir "Tester for Langmuir isotherm"
  extends Modelica.Icons.Example;

  //
  // Definition of parameters
  //
  final parameter Real[3] c={7.94*44.0098/1000,3.22e-10,-2.35e4}
    "Coefficients for Langmuir isotherm (qm, b0, dH)";

  //
  // Definition of variables
  //
protected
  Real[2] c_Langmuir
    "Temperature-dependent coefficients of Langmuir isotherm";
  Real[2] c_Langmuir_pdT
    "Temperature-dependent coefficients of Langmuir isotherm: T + 1e-4K";
  Real[2] c_Langmuir_mdT
    "Temperature-dependent coefficients of Langmuir isotherm: T - 1e-4K";

  Real[2] dc_Langmuir_dT
    "Derivative of temperature-dependent coefficients of Langmuir isotherm wrt.
     temperature";

public
  Modelica.SIunits.Pressure p_adsorpt
    "Equilibrium pressure";
  Modelica.SIunits.Pressure p_adsorpt_inv
    "Equilibrium pressure calculates with inverse of Langmuir isotherm";

  Modelica.SIunits.Temperature T_adsorpt
    "Equilibrium temperature";

  Real x_adsorpt(unit="1")
    "Equilibrium loading";

  Real dx_adsorpt_dp(unit="kg/(kg.Pa)")
    "Derivative of loading wrt. pressure";
  Real dx_adsorpt_dp_num(unit="kg/(kg.Pa)")
    "Derivative of loading wrt. pressure";

  Real dx_adsorpt_dT(unit="kg/(kg.K)")
    "Derivative of loading wrt. temperature";
  Real dx_adsorpt_dT_num(unit="kg/(kg.K)")
    "Derivative of loading wrt. temperature";

  Real dp_adsorpt_dx(unit="kg.Pa/kg")
    "Derivative of equilibrium pressure of adsorpt wrt. loading";
  Real dp_adsorpt_dx_num(unit="kg.Pa/kg")
    "Derivative of equilibrium pressure of adsorpt wrt. loading";

  Real dp_adsorpt_dT(unit="Pa/K")
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";
  Real dp_adsorpt_dT_num(unit="Pa/K")
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";

equation
  //
  // Calculate temperature-dependent coefficients
  //
  c_Langmuir = {c[1],
                c[2]*Modelica.Math.exp(-c[3]/Modelica.Constants.R/T_adsorpt)}
    "Temperature-dependent coefficients of Langmuir isotherm";
  c_Langmuir_pdT = {c[1],
                    c[2]*Modelica.Math.exp(-c[3]/Modelica.Constants.R/(
                    T_adsorpt+1e-4))}
    "Temperature-dependent coefficients of Langmuir isotherm: T + 1e-4K";
  c_Langmuir_mdT = {c[1],
                    c[2]*Modelica.Math.exp(-c[3]/Modelica.Constants.R/(
                    T_adsorpt-1e-4))}
    "Temperature-dependent coefficients of Langmuir isotherm: T - 1e-4K";

  dc_Langmuir_dT = {0,
                    c[3]/(Modelica.Constants.R*T_adsorpt^2) * c_Langmuir[2]}
    "Temperature-dependent coefficients of Langmuir isotherm";

  //
  // Calculate equilibrium properties
  //
  p_adsorpt = time*0.85e4
    "Define equilibrium pressure";
  T_adsorpt = 293.15 + 0.5 * time
    "Equilibrium temperature";
  x_adsorpt = SurfaceApproach.LangmuirIsotherm(p_adsorpt, c_Langmuir)
    "Calculate equilibrium loading";

  p_adsorpt_inv = SurfaceApproach.LangmuirIsotherm_inverse(x_adsorpt,
    c_Langmuir)
    "Calculate equilibrium pressure with inverse of Langmuir isotherm";

  //
  // Calculate equilibrium properties
  //
  dx_adsorpt_dp =
    SurfaceApproach.LangmuirIsotherm__dxdp(
    p_adsorpt, c_Langmuir)
    "Derivative of loading wrt. pressure";
  dx_adsorpt_dp_num =
    (SurfaceApproach.LangmuirIsotherm(p_adsorpt+1, c_Langmuir) -
     SurfaceApproach.LangmuirIsotherm(p_adsorpt-1, c_Langmuir)) / 2
    "Derivative of loading wrt. pressure";

  dx_adsorpt_dT =
    SurfaceApproach.LangmuirIsotherm__dxdT(
    p_adsorpt, c_Langmuir, dc_Langmuir_dT)
    "Derivative of loading wrt. temperature";
  dx_adsorpt_dT_num =
    (SurfaceApproach.LangmuirIsotherm(p_adsorpt, c_Langmuir_pdT) -
     SurfaceApproach.LangmuirIsotherm(p_adsorpt, c_Langmuir_mdT)) / (2e-4)
    "Derivative of loading wrt. temperature";

  dp_adsorpt_dx =
    SurfaceApproach.LangmuirIsotherm_inverse__dpdx(
    x_adsorpt, c_Langmuir)
    "Derivative of equilibrium pressure of adsorpt wrt. loading";
  dp_adsorpt_dx_num =
    (SurfaceApproach.LangmuirIsotherm_inverse(x_adsorpt+1e-6, c_Langmuir) -
     SurfaceApproach.LangmuirIsotherm_inverse(x_adsorpt-1e-6, c_Langmuir)) /
     (2e-6)
    "Derivative of equilibrium pressure of adsorpt wrt. loading";

  dp_adsorpt_dT =
    SurfaceApproach.LangmuirIsotherm_inverse__dpdT(
    x_adsorpt, c_Langmuir, dc_Langmuir_dT)
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";
  dp_adsorpt_dT_num =
    (SurfaceApproach.LangmuirIsotherm_inverse(x_adsorpt, c_Langmuir_pdT) -
     SurfaceApproach.LangmuirIsotherm_inverse(x_adsorpt, c_Langmuir_mdT)) /
     (2e-4)
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";

  //
  // Annotations
  //
  annotation (experiment(StopTime=100), Documentation(revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end TestLangmuir;
