within SorpLib.Media.Function.SingleComponent.Equilibrium.SurfaceApproach.Tester;
model TestToth "Tester for Toth isotherm"
  extends Modelica.Icons.Example;

  //
  // Definition of parameters
  //
  final parameter Real[5] c={1.106e-11*0.01802,4.714e-13,9.955e3,
    3.548e-1,-5.114e1}
    "Coefficients for Toth isotherm (a0, b0, E, t0, c)";

  //
  // Definition of variables
  //
protected
  Real[3] c_Toth
    "Temperature-dependent coefficients of Toth isotherm";
  Real[3] c_Toth_pdT
    "Temperature-dependent coefficients of Toth isotherm: T + 1e-4K";
  Real[3] c_Toth_mdT
    "Temperature-dependent coefficients of Toth isotherm: T - 1e-4K";

  Real[3] dc_Toth_dT
    "Derivative of temperature-dependent coefficients of Toth isotherm wrt.
     temperature";

public
  Modelica.SIunits.Pressure p_adsorpt
    "Equilibrium pressure";
  Modelica.SIunits.Pressure p_adsorpt_inv
    "Equilibrium pressure calculates with inverse of Toth isotherm";

  Modelica.SIunits.Temperature T_adsorpt
    "Equilibrium temperature";

  Real x_adsorpt(unit="1")
    "Equilibrium loading";

  Real dx_adsorpt_dp(unit="kg/(kg.Pa)")
    "Derivative of loading wrt. pressure";
  Real dx_adsorpt_dp_num(unit="kg/(kg.Pa)")
    "Derivative of loading wrt. pressure";

  Real dx_adsorpt_dT(unit="kg/(kg.K)")
    "Derivative of loading wrt. temperature";
  Real dx_adsorpt_dT_num(unit="kg/(kg.K)")
    "Derivative of loading wrt. temperature";

  Real dp_adsorpt_dx(unit="kg.Pa/kg")
    "Derivative of equilibrium pressure of adsorpt wrt. loading";
  Real dp_adsorpt_dx_num(unit="kg.Pa/kg")
    "Derivative of equilibrium pressure of adsorpt wrt. loading";

  Real dp_adsorpt_dT(unit="Pa/K")
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";
  Real dp_adsorpt_dT_num(unit="Pa/K")
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";

equation
  //
  // Calculate temperature-dependent coefficients
  //
  c_Toth = {c[1] * Modelica.Math.exp(c[3]/T_adsorpt),
            c[2] * Modelica.Math.exp(c[3]/T_adsorpt),
            c[4] + c[5]/T_adsorpt}
    "Temperature-dependent coefficients of Toth isotherm";
  c_Toth_pdT = {c[1] * Modelica.Math.exp(c[3]/(T_adsorpt+1e-4)),
                c[2] * Modelica.Math.exp(c[3]/(T_adsorpt+1e-4)),
                c[4] + c[5]/(T_adsorpt+1e-4)}
    "Temperature-dependent coefficients of Toth isotherm: T + 1e-4K";
  c_Toth_mdT = {c[1] * Modelica.Math.exp(c[3]/(T_adsorpt-1e-4)),
                c[2] * Modelica.Math.exp(c[3]/(T_adsorpt-1e-4)),
                c[4] + c[5]/(T_adsorpt-1e-4)}
    "Temperature-dependent coefficients of Toth isotherm: T - 1e-4K";

  dc_Toth_dT = {-c[3] / (T_adsorpt^2) * c_Toth[1],
                -c[3] / (T_adsorpt^2) * c_Toth[2],
                -c[5] / (T_adsorpt^2)}
    "Temperature-dependent coefficients of Toth isotherm";

  //
  // Calculate equilibrium properties
  //
  p_adsorpt = 1 + time*0.85e4
    "Define equilibrium pressure";
  T_adsorpt = 293.15 + 0.5 * time
    "Equilibrium temperature";
  x_adsorpt = SurfaceApproach.TothIsotherm(p_adsorpt, c_Toth)
    "Calculate equilibrium loading";

  p_adsorpt_inv = SurfaceApproach.TothIsotherm_inverse(x_adsorpt, c_Toth)
    "Calculate equilibrium pressure with inverse of Toth isotherm";

  //
  // Calculate equilibrium properties
  //
  dx_adsorpt_dp =
    SurfaceApproach.TothIsotherm__dxdp(p_adsorpt, c_Toth)
    "Derivative of loading wrt. pressure";
  dx_adsorpt_dp_num =
    (SurfaceApproach.TothIsotherm(p_adsorpt+1e-4, c_Toth) -
     SurfaceApproach.TothIsotherm(p_adsorpt-1e-4, c_Toth)) / (2e-4)
    "Derivative of loading wrt. pressure";

  dx_adsorpt_dT =
    SurfaceApproach.TothIsotherm__dxdT(p_adsorpt, c_Toth, dc_Toth_dT)
    "Derivative of loading wrt. temperature";
  dx_adsorpt_dT_num =
    (SurfaceApproach.TothIsotherm(p_adsorpt, c_Toth_pdT) -
     SurfaceApproach.TothIsotherm(p_adsorpt, c_Toth_mdT)) / (2e-4)
    "Derivative of loading wrt. temperature";

  dp_adsorpt_dx =
    SurfaceApproach.TothIsotherm_inverse__dpdx(x_adsorpt, c_Toth)
    "Derivative of equilibrium pressure of adsorpt wrt. loading";
  dp_adsorpt_dx_num =
    (SurfaceApproach.TothIsotherm_inverse(x_adsorpt+1e-6, c_Toth) -
     SurfaceApproach.TothIsotherm_inverse(x_adsorpt-1e-6, c_Toth)) /
     (2e-6)
    "Derivative of equilibrium pressure of adsorpt wrt. loading";

  dp_adsorpt_dT =
    SurfaceApproach.TothIsotherm_inverse__dpdT(x_adsorpt, c_Toth, dc_Toth_dT)
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";
  dp_adsorpt_dT_num =
    (SurfaceApproach.TothIsotherm_inverse(x_adsorpt, c_Toth_pdT) -
     SurfaceApproach.TothIsotherm_inverse(x_adsorpt, c_Toth_mdT)) /
     (2e-4)
    "Derivative of equilibrium pressure of adsorpt wrt. temperature";

  //
  // Annotations
  //
  annotation (experiment(StopTime=100), Documentation(revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end TestToth;
