within SorpLib.Media.Function.SingleComponent.Equilibrium.VolumetricApproach.Tester;
model TestCharCurve_Arctan "Tester for arctan-approach"
  extends Modelica.Icons.Example;

  //
  // Definition of parameters
  //
  final parameter Real[4] c={5.072313e-4,1.305531e5,-8.492403e4,4.128962e-6}
    "Coefficients for characteristic curve";

  parameter Modelica.SIunits.Temperature T_adsorpt = 273.15 + 60
    "Temperature of adsorpt";
  parameter Real beta_adsorpt(final unit="1/K") = 0.000523293
    "Liquid isobaric expansion coefficient";

  //
  // Definition of models
  //
  CharCurve_Arctan_Summary summary_classical(
    final T_adsorpt=T_adsorpt,
    final beta_adsorpt=beta_adsorpt,
    final A_input=A,
    final W_input=0,
    final type_charCurve=Internals.Types.Isotherm.Classical,
    final c=c)
    "Summary model using classical approach for characteristic curve"
    annotation (Placement(transformation(extent={{-10,20},{10,40}})));
  CharCurve_Arctan_Summary summary_inverse(
    final T_adsorpt=T_adsorpt,
    final beta_adsorpt=beta_adsorpt,
    final c=c,
    final A_input=0,
    final W_input=summary_classical.W_output,
    final type_charCurve=Internals.Types.Isotherm.Inverse)
    "Summary model using inverse approach for characteristic curve"
    annotation (Placement(transformation(extent={{-10,-40},{10,-20}})));

  //
  // Definition of variables
  //
  Modelica.SIunits.SpecificEnergy A
    "Adsorption potential";
  Modelica.SIunits.SpecificEnergy A_inv
    "Adsorption potential calculated with inverse function";

  Modelica.SIunits.SpecificVolume W
    "Specific volume of adsorbate";

  Real dW_dA(unit="m3/J")
    "Derivative of specific volume of adsorbate W wrt. to adsorption 
     potential A";
  Real dW_dA_num(unit="m3/J")
    "Numerical derivative of specific volume of adsorbate W wrt. to adsorption 
     potential A";

  Real dA_dW(unit="J/m3")
    "Derivative of adsorption potential A wrt. to specific volume of 
     adsorbate W";
  Real dA_dW_num(unit="J/m3")
    "Numerical derivative of adsorption potential A wrt. to specific volume of
     adsorbate W";

equation
  //
  // Calculate equilibrium properties
  //
  A = 1e2 + time*1e4
    "Adsorption potential";
  W = VolumetricApproach.CharCurve_Arctan(A, c)
    "Specific volume of adsorbate";

  A_inv = VolumetricApproach.CharCurve_Arctan_inverse(W, c)
    "Adsorption potential calculated with inverse function";

  dW_dA =CharCurve_Arctan__dWdA(A, c)
    "Derivative of specific volume of adsorbate W wrt. to adsorption 
     potential A";
  dW_dA_num =
    (VolumetricApproach.CharCurve_Arctan(A + 0.5, c) -
     VolumetricApproach.CharCurve_Arctan(A - 0.5, c)) / 1
    "Numerical derivative of specific volume of adsorbate W wrt. to adsorption 
     potential A";

  dA_dW =CharCurve_Arctan__dAdW(W, c)
    "Derivative of adsorption potential A wrt. to specific volume of 
     adsorbate W";
  dA_dW_num =
    (VolumetricApproach.CharCurve_Arctan_inverse(W + 1e-12, c) -
     VolumetricApproach.CharCurve_Arctan_inverse(W - 1e-12, c)) / (2e-12)
    "Numerical derivative of adsorption potential A wrt. to specific volume of
     adsorbate W";

  //
  // Annotations
  //
  annotation (experiment(StopTime=100), Documentation(revisions="<html>
<ul>
  <li>
  December 10, 2020, by Mirko Engelpracht:<br/>
  First implementation after restructuring of the library.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end TestCharCurve_Arctan;
