within SorpLib.Media.WorkingPair.BaseClasses;
partial model PartialWorkingPair "Base model for all working pair models"

  //
  // Definition of inputs
  //
  input Modelica.SIunits.Temperature T_adsorpt
    "Temperature of adsorpt (i.e., equilibrium temperature)"
    annotation (Dialog(tab = "General", group = "Inputs"));

  //
  // Definition of parameters
  //
  parameter Internals.Types.Isotherm type_isotherm=
    Internals.Types.Isotherm.Classical
    "Isotherm type"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                Evaluate=true,
                HideResult = true);

  parameter Modelica.SIunits.SpecificHeatCapacity c_sorbent = 1000
    "Constant heat capacity of dry sorbent"
    annotation (Dialog(tab = "General", group = "Sorbent"),
                HideResult = true);

  parameter Boolean calculate_caloric_properties=true
    "= true, to calculate caloric properties (e.g. u_adsorpt, h_adsorpt, ...)"
    annotation (Dialog(tab="Media", group="General"),
                Evaluate=true,
                HideResult = true);

  parameter Boolean use_pInitial = false
    "= true, if initial value for pressure is used for calculations"
    annotation (Dialog(group = "Initial Values", tab = "Initialisation"),
                Evaluate=true,
                HideResult = true);

  parameter Internals.Types.AdsorptionEnthalpy type_AdsorptionEnthalpy=
    Internals.Types.AdsorptionEnthalpy.Bonding
    "Calculation approach for adsorption enthalpy"
    annotation (Dialog(tab="Advanced",group="Adsorption Enthalpy",
                enable=calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);

  parameter Internals.Types.SpecificEnthalpySorptive type_SpecEnthalpySorptive=
    Internals.Types.SpecificEnthalpySorptive.Media_pT
    "Calculation approach for specific enthalpy of sorptive (i.e., gas or vapor
     phase)"
    annotation (Dialog(tab="Advanced",group="Sorptive",
                enable = calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);

  parameter Internals.Types.SpecificEnthalpyAdsorpt type_SpecEnthalpyAdsorpt=
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Calculation approach for specific enthalpy of adsorpt (e.g., definition of
     adsorption enthalpy or detailed model)"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);
  parameter Internals.Types.SpecificHeatCapacityAdsorpt type_SpecHeatCapacityAdsorpt=
    Internals.Types.SpecificHeatCapacityAdsorpt.SatLiq
    "Calculation approach for specific heat capacity of adsorpt"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);
  parameter Boolean use_SpecVolumeAdsorpt = true
    "= true, if term p_adsorpt*v_adsorpt shall be considered when calculating
     internal energy of adsorpt"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);
  final parameter Internals.Types.SpecificVolumeAdsorpt type_SpecVolumeAdsorpt=
    Internals.Types.SpecificVolumeAdsorpt.SatLiq
    "Calculation approach for specific volume of adsorpt"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = calculate_caloric_properties and use_SpecVolumeAdsorpt),
                Evaluate=true,
                HideResult = true);

  parameter Boolean avoid_events = true
    "= true, if events are avoid by using noEvent()-operator"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                Evaluate=true,
                HideResult = true);
  parameter Boolean use_stateLimiter = true
    "= true, if limiter is used for pressure of media object *_pT"
    annotation (Dialog(tab = "Advanced", group = "Numerics",
                enable = (type_SpecEnthalpySorptive==
                Internals.Types.SpecificEnthalpySorptive.Media_pT) and
                calculate_caloric_properties),
                Evaluate=true,
                HideResult = true);

  //
  // Definition of constants
  //
protected
  constant Internals.Types.MediaModel type_Media=
    Internals.Types.MediaModel.Modelica_VLE
    "Type required to check consistency of media models"
    annotation (HideResult=true);
  constant Internals.Types.WPairModel type_WPair=
    Internals.Types.WPairModel.SingleAdsorption
    "Type required to check consistency of working pair models"
    annotation (HideResult=true);

  //
  // Annotations
  //
  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
          Ellipse(
          extent={{-100,100},{100,-100}},
          lineColor={0,0,0},
          fillColor={217,67,180},
          fillPattern=FillPattern.Solid), Text(
          extent={{-64,42},{76,-118}},
          lineColor={0,0,0},
          fillColor={217,67,180},
          fillPattern=FillPattern.Solid,
          textString="WP",
          textStyle={TextStyle.Bold})}),
                              Diagram(coordinateSystem(preserveAspectRatio=false)),
    Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialWorkingPair;
