within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleMediaModel "Base model for all models calculating fluid properties required by 
  adsorption equilibrium models of single component adsorption"
  extends SorpLib.Media.WorkingPair.BaseClasses.PartialMediaModel;

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.SpecificHeatCapacity cp_adsorpt_fixed = 4180
    "Constant heat capacity of adsorpt"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = (type_SpecHeatCapacityAdsorpt==
                Internals.Types.SpecificHeatCapacityAdsorpt.Constant) and
                calculate_caloric_properties),
                HideResult = true);

  //
  // Definition of inputs
  //
  input Modelica.SIunits.Pressure p_adsorpt
    "Pressure of adsorpt (i.e., equilibrium pressure)"
    annotation (Dialog(tab = "General", group = "Inputs"));

  //
  // Definition of outputs
  //
  Modelica.Blocks.Interfaces.RealOutput h_sorptive(final unit="J/kg")
    "Specific enthalpy of sorptive: Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput v_adsorpt(final unit="m3/kg")
    "Specific volume of adsorpt: Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput cp_adsorpt(final unit="J/(kg.K)")
    "Specific heat capacity of adsorpt: Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));

  //
  // Definition of internal variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput h_sorptive_dummy(final unit="J/kg")=
    -1 if not calculate_caloric_properties
    "Specific enthalpy of sorptive: Dummy value if caloric properties are not
     required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput cp_adsorpt_dummy(final unit="J/(kg.K)")=
    -1 if not calculate_caloric_properties
    "Specific heat capacity of sorptive: Dummy value if caloric properties are 
     not required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput cp_adsorpt_constant(final unit="J/(kg.K)")=
    cp_adsorpt_fixed if calculate_caloric_properties and
    (type_SpecHeatCapacityAdsorpt==
    Internals.Types.SpecificHeatCapacityAdsorpt.Constant)
    "Specific heat capacity of sorptive: Constant value"
    annotation (HideResult = true);

equation
  //
  // Connections for conditional removed variables / models
  //
  connect(h_sorptive, h_sorptive_dummy);
  connect(cp_adsorpt, cp_adsorpt_dummy);
  connect(cp_adsorpt, cp_adsorpt_constant);

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSingleMediaModel;
