within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleMediaModel_Volumetric "Base model for all models calculating fluid properties required by 
   adsorption equilibrium models of single component adsorption based on the 
   volumetric approach"
  extends PartialSingleMediaModel(
    type_SpecEnthalpyAdsorpt=
      Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy,
    type_AdsorptionEnthalpy= if (type_SpecEnthalpyAdsorpt==
      Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy) then
      Internals.Types.AdsorptionEnthalpy.Bonding else
      Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.TemperatureDifference dT_AdsEnt_CC=1e-4
    "Temperature difference required to calculate adsorption enthalpy with
     Clausius-Clapeyron approach using the finite difference approache"
     annotation (Dialog(tab="Advanced", group="Adsorpt",
                 enable = type_SpecEnthalpyAdsorpt==
                 Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy),
                 HideResult=true);

  //
  // Definition of outputs
  //
  Modelica.Blocks.Interfaces.RealOutput p_sat_adsorpt(final unit="Pa")
    "Saturation pressure at T_Adsorpt: Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput h_sat_liq_adsorpt(final unit="J/kg")
    "Specific bubble enthalpy: Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput beta_sat_liq_adsorpt(final unit="1/K")
    "Bubble isobaric expansion coefficient:Depends on calculation approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));

  Modelica.Blocks.Interfaces.RealOutput p_sat_mdT_adsorpt(final unit="Pa")
    "Saturation pressure at T_adsorpt-dT_AdsEnt_CC: Depends on calculation 
     approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput p_sat_pdT_adsorpt(final unit="Pa")
    "Saturation pressure at T_adsorpt+dT_AdsEnt_CC: Depends on calculation 
     approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput d_sat_liq_mdT_adsorpt(
    final unit="kg/m3")
    "Saturated liquid density at T_adsorpt-dT_AdsEnt_CC: Depends on calculation 
     approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));
  Modelica.Blocks.Interfaces.RealOutput d_sat_liq_pdT_adsorpt(
    final unit="kg/m3")
    "Saturated liquid density at T_adsorpt+dT_AdsEnt_CC:Depends on calculation 
     approach"
    annotation (Dialog(tab = "General", group = "Outputs", enable=false));

  //
  // Definition of internal variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput h_sat_liq_adsorpt_dummy(
    final unit="J/kg")=
    -1 if not calculate_caloric_properties or type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Specific bubble enthalpy: Dummy value if caloric properties are 
     not required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput beta_sat_liq_adsorpt_dummy(
    final unit="1/K")=
    -1 if not calculate_caloric_properties or type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Bubble isobaric expansion coefficient: Dummy value if caloric properties 
     are not required"
    annotation (HideResult = true);

  Modelica.Blocks.Interfaces.RealInput p_sat_mdT_adsorpt_dummy(
    final unit="Pa") = -1 if not calculate_caloric_properties or not
    type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Saturation pressure at T_adsorpt-dT_AdsEnt_CC: Dummy value if not required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput p_sat_pdT_adsorpt_dummy(
    final unit="Pa") = -1 if not calculate_caloric_properties or not
    type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Saturation pressure at T_adsorpt+dT_AdsEnt_CC: Dummy value if not required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput d_sat_liq_mdT_adsorpt_dummy(
    final unit="kg/m3") = -1 if not calculate_caloric_properties or not
    type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Saturated liquid density at T_adsorpt-dT_AdsEnt_CC: Dummy value if not 
     required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput d_sat_liq_pdT_adsorpt_dummy(
    final unit="kg/m3") = -1 if not calculate_caloric_properties or not
    type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy
    "Saturated liquid density at T_adsorpt+dT_AdsEnt_CC: Dummy value if not 
     required"
    annotation (HideResult = true);

equation
  //
  // Connections for conditional removed variables / models
  //
  connect(h_sat_liq_adsorpt,h_sat_liq_adsorpt_dummy);
  connect(beta_sat_liq_adsorpt,beta_sat_liq_adsorpt_dummy);

  connect(p_sat_mdT_adsorpt,p_sat_mdT_adsorpt_dummy);
  connect(p_sat_pdT_adsorpt,p_sat_pdT_adsorpt_dummy);
  connect(d_sat_liq_mdT_adsorpt,d_sat_liq_mdT_adsorpt_dummy);
  connect(d_sat_liq_pdT_adsorpt,d_sat_liq_pdT_adsorpt_dummy);

  //
  // Annotations
  //
  annotation (Icon(graphics={
        Text(
          extent={{-52,-68},{48,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={215,215,215},
          fillPattern=FillPattern.Solid,
          textStyle={TextStyle.Bold},
          textString="Volumetric Approach")}), Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSingleMediaModel_Volumetric;
