within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleWPair
  "Base model for all working pairs models describing single component adsorption"
  extends SorpLib.Media.WorkingPair.BaseClasses.PartialWorkingPair(final
      type_WPair=Internals.Types.WPairModel.SingleAdsorption);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.MolarMass M_adsorpt = 0.018015267
    "Molar mass of adsorpt / sorptive"
    annotation (Dialog(tab = "General", group = "Adsorpt"),
                Evaluate=true,
                HideResult=true);
protected
  final parameter Modelica.SIunits.SpecificHeatCapacity R_adsorpt=
    Modelica.Constants.R / M_adsorpt
    "Specific gas constant of adsorpt / sorptive"
    annotation (Dialog(tab = "General", group = "Adsorpt"),
                Evaluate=true,
                HideResult=true);

public
  parameter Modelica.SIunits.SpecificHeatCapacity cp_adsorpt_fixed = 4180
    "Constant heat capacity of adsorpt"
    annotation (Dialog(tab="Advanced",group="Adsorpt",
                enable = (type_SpecHeatCapacityAdsorpt==
                Internals.Types.SpecificHeatCapacityAdsorpt.Constant) and
                calculate_caloric_properties),
                HideResult = true);

  parameter Modelica.SIunits.Pressure p_min = 10
    "Minimal pressure used as state variable for *_pT media models"
    annotation (Dialog(tab = "Advanced", group = "Numerics",
                enable = (type_SpecEnthalpySorptive==
                Internals.Types.SpecificEnthalpySorptive.Media_pT) and
                calculate_caloric_properties
                and use_stateLimiter),
                HideResult = true);

  //
  // Definition of inputs
  //
  input Real x_adsorpt(final unit="kg/kg")
    "Loading of adsorpt (i.e., equilirium loading in kg_adsorpt/kg_sorbent)"
    annotation (Dialog(tab = "General", group = "Inputs"));

  //
  // Definition of outputs
  //
  output Modelica.SIunits.Pressure p_adsorpt(start=50)
    "Pressure of adsorpt"
    annotation (Dialog(tab = "General", group = "Outputs", enable = false));

  output Real[2] du_avg_dT_x(unit="J/(kg.K)")=if
    calculate_caloric_properties then {c_sorbent,  x_adsorpt*cp_adsorpt} else
    {-1, -1}
    "Partial derivatives of average internal energies (i.e., sorbent and 
     x_adsorpt*u_avg_adsorpt) wrt. temperature T_adsorpt at constant loading
     x_adsorpt"
    annotation (Dialog(tab = "General", group = "Outputs", enable = false),
                HideResult = not calculate_caloric_properties);

  output Modelica.SIunits.SpecificInternalEnergy[2] du_avg_dx_T=if
    calculate_caloric_properties then {0, u_adsorpt} else
    {-1, -1}
    "Partial derivatives of average internal energies (i.e., sorbent and 
     x_adsorpt*u_avg_adsorpt) wrt. loading x_adsorpt at constant temperature 
     T_adsorpt"
    annotation (Dialog(tab = "General", group = "Outputs", enable = false),
                HideResult = not calculate_caloric_properties);

  output Modelica.SIunits.SpecificEnthalpy h_sorptive
    "Specific enthalpy of sorptive"
    annotation (Dialog(tab = "General", group = "Outputs", enable = false),
                HideResult = not calculate_caloric_properties);

  //
  // Definition of parameters
  //
  parameter Modelica.SIunits.Pressure pInitial = 5000
    "Initial value for pressure of adsorpt"
    annotation (Dialog(tab = "Initialisation", group = "Initial Values",
                enable = use_pInitial),
                HideResult = true);

  //
  // Definition of variables
  //
  Modelica.SIunits.SpecificVolume v_adsorpt(min=-1)
    "Specific volume of adsorpt"
    annotation (HideResult = not calculate_caloric_properties or
                not use_SpecVolumeAdsorpt);
  Modelica.SIunits.SpecificEnthalpy h_adsorpt
    "Specific enthalpy of adsorpt"
    annotation (HideResult = not calculate_caloric_properties);
  Modelica.SIunits.SpecificInternalEnergy u_adsorpt
    "Internal energy of adsorpt"
    annotation (HideResult = not calculate_caloric_properties);

  Modelica.SIunits.SpecificHeatCapacity cp_adsorpt
    "Specific heat capacity of adsorpt"
    annotation (HideResult = not calculate_caloric_properties);

  Modelica.SIunits.SpecificEnthalpy dh_ads
    "Adsorption enthalpy"
    annotation (HideResult = not calculate_caloric_properties);

  Modelica.SIunits.SpecificEnthalpy dh_bond
    "Bonding enthalpy"
    annotation (HideResult = true);

initial equation
  //
  // Check if start values are required
  //
  if use_pInitial then
    p_adsorpt = pInitial;
  end if;

equation
  //
  // Calculate internal energy of adsorpt and its derivative
  //
  if use_SpecVolumeAdsorpt then
    // Do not neglect specific volume of adsorpt
    //
    u_adsorpt = if calculate_caloric_properties then
      h_adsorpt - p_adsorpt*v_adsorpt else -1
      "Internal energy of adsorpt";

  else
    // Neglect specific volume of adsorpt
    //
    u_adsorpt = if calculate_caloric_properties then
      h_adsorpt else -1
      "Internal energy of adsorpt";

  end if;

  //
  // Annotations
  //
  annotation (Icon(graphics={             Text(
          extent={{-78,106},{82,-34}},
          lineColor={0,0,0},
          fillColor={217,67,180},
          fillPattern=FillPattern.Solid,
          textStyle={TextStyle.Bold},
          textString="Single")}), Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  Major reivisions (e.g., object-oriented approach, functionaliy) due to restructuring of the library.
  </li>
  <li>
  November 17, 2017, by Uwe Bau:<br/> 
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>
This partial model contains all necessary variables and the basic equations to describe equilibrium data
</p>
<p>
The partial model needs to be extended by the specific functions for each working pair
</p>
<h4>Main equations</h4>
<p align=\"center\"><i>u<sub>ads</sub> = h<sub>ads</sub> - p<sub>ad</sub> / <code>&rho;</code><sub>ad</sub> </i></p>
<p>where <i>u</i><sub>ads</sub> is the specific internal energy of the adsorbate, <i>h</i><sub>ads</sub> is the specific enthalpy of the adsorbate of the adsorbate, <i>p</i><sub>ad</sub> is the
pressure and <i><code>&rho;</code></i><sub>ad</sub> is the density of the adsorbate</p>
<p align=\"center\"><i>dudT_x = c<sub>sor</sub> + x * c<sub>ad</sub> , </i></p>
<p>where <i>dudT_x</i> is the specific heat capacity of adsorbent and adsorbate, <i>c</i><sub>sor</sub> is the specific heat capacity of the adsorbent, <i>x</i> is the loading and 
<i>c</i><sub>ad</sub> is the specific heat capacity of the adsorbate (approximated by the specific heat capacity of liquid water).</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<ul>
<li>N&uacute;&ntilde;ez, T. Charakterisierung und Bewertung von Adsorbentien f&uuml;r W&auml;rmetransformationsanwendungen. PhD Thesis. Freiburg, 2001. </li>
</ul>
</html>"));
end PartialSingleWPair;
