within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleWPair_SorpPropLib
  "Base model for all working pairs models describing single component 
  adsorption based on SorpPropLib"
  extends PartialSingleWPair(
    final type_SpecEnthalpyAdsorpt=
      Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy,
    final type_AdsorptionEnthalpy=
      Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron);

  //
  // Definition of parameters
  //
  parameter String path_db=
    "D:/Engelpracht/01-Modelica/01-Bib/SorpPropLib/Resources/Data/sorpproplib.json"
    "Path to database, i.e., JSON-file of database."
    annotation (Dialog(tab="General", group = "Equilibrium Model"));

  parameter String name_sorbent = "zeolite pellet"
    "Name of sorbent, e.g., 'zeolite pellet'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));
  parameter String name_sorbent_sub_type = "13X"
    "Sub-type of sorbent, e.g., '13X'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));
  parameter String name_refrigerant = "Water"
    "Name of refrigerant, e.g., 'Water'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));

  parameter String func_isotherm = "Toth"
    "Name of functional approach for isotherm, e.g., 'Toth'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));
  parameter Integer func_isotherm_ID = 1
    "ID of functional approach for isotherm, e.g., '1'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));

  parameter String func_vapor_pressure = "VaporPressure_EoS1"
    "Name of functional approach for vapor pressure, e.g., 'VaporPressure_EoS1'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));
  parameter Integer func_vapor_pressure_ID = 1
    "ID of functional approach for vapor pressure, e.g., '1'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));

  parameter String func_saturated_liquid_density = "SaturatedLiquidDensity_EoS1"
    "Name of functional approach for saturated liquid density, e.g. 'SaturatedLiquidDensity_EoS1'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));
  parameter Integer func_saturated_liquid_density_ID = 1
    "ID of functional approach for saturated liquid density, e.g., '1'"
    annotation (Dialog(tab="General", group = "Equilibrium Model"));

  replaceable model MediaModel = PartialSingleMediaModel_Surface
    constrainedby PartialSingleMediaModel_Surface
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true);

  //
  // Instanziation of models
  //
  MediaModel mediaModels(
    final p_adsorpt=if use_stateLimiter then max(p_min, p_adsorpt) else
      p_adsorpt,
    final T_adsorpt=T_adsorpt,
    final calculate_caloric_properties=calculate_caloric_properties,
    final type_AdsorptionEnthalpy=type_AdsorptionEnthalpy,
    final type_SpecEnthalpySorptive=type_SpecEnthalpySorptive,
    final type_SpecEnthalpyAdsorpt=type_SpecEnthalpyAdsorpt,
    final type_SpecHeatCapacityAdsorpt=type_SpecHeatCapacityAdsorpt,
    final cp_adsorpt_fixed=cp_adsorpt_fixed,
    final use_SpecVolumeAdsorpt=use_SpecVolumeAdsorpt,
    final type_SpecVolumeAdsorpt=type_SpecVolumeAdsorpt,
    final avoid_events=avoid_events)
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}})));

  SorpPropLib.WorkingPair.WPair extObj=SorpPropLib.WorkingPair.WPair(path_db,
        name_sorbent, name_sorbent_sub_type, name_refrigerant,
        func_isotherm, func_isotherm_ID,
        func_vapor_pressure, func_vapor_pressure_ID,
        func_saturated_liquid_density, func_saturated_liquid_density_ID)
    "Initialize external object of working pair by calling its class";

equation
  //
  // Assertion to check for invalid parameter selections
  //
  assert(type_SpecEnthalpyAdsorpt<>
    Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy,
    "Error in working pair model: Specific adsorption enthalpy cannot be 
     derived from specific bonding and specific bubble enthalpies since 
     specific bonding enthalpy is not defined for equilibrium models based on 
     the surface approach!");

  //
  // Passing through fluid properties from the media models
  //
  h_sorptive = mediaModels.h_sorptive
    "Specific enthalpy of sorptive";
  v_adsorpt = mediaModels.v_adsorpt
    "Specific volume of adsorpt";
  cp_adsorpt = mediaModels.cp_adsorpt
    "Specific heat capacity of adsorpt";

  //
  // Check form of isotherm
  //
  if type_isotherm == Internals.Types.Isotherm.Classical then
    //
    // Classical form (i.e., x(p,T))
    //
    x_adsorpt = SorpPropLib.WorkingPair.WPairFunctions.Adsorption.General.w_pT(
      p_adsorpt,
      T_adsorpt,
      extObj=extObj)
      "Calculate equlibrium loading of adsorpt";

  else
    //
    // Inverse form (i.e., p(x,T))
    //
    p_adsorpt = SorpPropLib.WorkingPair.WPairFunctions.Adsorption.General.p_wT(
      x_adsorpt,
      T_adsorpt,
      extObj=extObj)
      "Calculate equlibrium pressure of adsorpt";

  end if;

  //
  // Calculate specific enthalpy of adsorpt
  //
  if type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy then
    // Use definition of adsorption enthalpy to calculate specific enthalpy of
    // adsorpt
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive - dh_ads else -1
      "Calculation of specific enthalpy of adsorpt";

  else
    // Currently, no other calculation approach is implemented
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive - dh_ads else -1
      "Calculation of specific enthalpy of adsorpt";

  end if;

  //
  // Calculate specific adsorption enthalpy
  //
  dh_bond = -1
    "Calculation of specific bonding enthalpy: Not defined for equilibrium
     models based on the surface approach";

  dh_ads =if calculate_caloric_properties then
    Function.SingleComponent.AdsorptionEnthalpy.dhAds_ClausiusClapeyron(
    R_adsorpt,
    p_adsorpt,
    T_adsorpt,
    SorpPropLib.WorkingPair.WPairFunctions.Adsorption.General.dw_dT_pT(
      p_adsorpt, T_adsorpt, extObj=extObj),
    SorpPropLib.WorkingPair.WPairFunctions.Adsorption.General.dw_dp_pT(
      p_adsorpt, T_adsorpt, extObj=extObj)) else -1
    "Calculation of specific adsorption enthalpy";

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  August 19, 2021, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSingleWPair_SorpPropLib;
