within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleWPair_Surface "Base model for all working pairs models describing single component 
  adsorption based on surface approach (e.g., Langmuir, Toth, Sips)"
  extends PartialSingleWPair(
    final type_SpecEnthalpyAdsorpt=
      Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy,
    final type_AdsorptionEnthalpy=
      Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron);

  //
  // Definition of parameters
  //
  parameter Integer no_coefficients = 1
    "Number of coefficients required by isotherm equation"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                Evaluate=true,
                HideResult = true);
  replaceable function func_x_pT =
    Function.SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm
    constrainedby Function.SingleComponent.BaseClasses.PartialSurfaceFunction
    "Classical form of isotherm: Calculates x_adsorpt from p_adsorpt and T_adsorpt"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_p_xT =
    Function.SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm_inverse
    constrainedby
    Function.SingleComponent.BaseClasses.PartialSurfaceFunction_inverse
    "Inverse form of isotherm: Calculates p_adsorpt from x_adsorpt and T_adsorpt"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_dx_dp_pT =
    Function.SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm__dxdp
    constrainedby Function.SingleComponent.BaseClasses.PartialSurfaceFunction
    "Classical form of isotherm: Calculates partial derivative of x_adsorpt wrt.
     p_adsorpt from p_adsorpt and T_adsorpt"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_dx_dT_pT =
    Function.SingleComponent.Equilibrium.SurfaceApproach.TothIsotherm__dxdT
    constrainedby
    Function.SingleComponent.BaseClasses.PartialSurfaceFunction_dT
    "Classical form of isotherm: Calculates partial derivative of x_adsorpt wrt.
     T_adsorpt from p_adsorpt and T_adsorpt"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);

  replaceable model MediaModel = PartialSingleMediaModel_Surface
    constrainedby PartialSingleMediaModel_Surface
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true);

  //
  // Instanziation of models
  //
  MediaModel mediaModels(
    final p_adsorpt=if use_stateLimiter then max(p_min, p_adsorpt) else
      p_adsorpt,
    final T_adsorpt=T_adsorpt,
    final calculate_caloric_properties=calculate_caloric_properties,
    final type_AdsorptionEnthalpy=type_AdsorptionEnthalpy,
    final type_SpecEnthalpySorptive=type_SpecEnthalpySorptive,
    final type_SpecEnthalpyAdsorpt=type_SpecEnthalpyAdsorpt,
    final type_SpecHeatCapacityAdsorpt=type_SpecHeatCapacityAdsorpt,
    final cp_adsorpt_fixed=cp_adsorpt_fixed,
    final use_SpecVolumeAdsorpt=use_SpecVolumeAdsorpt,
    final type_SpecVolumeAdsorpt=type_SpecVolumeAdsorpt,
    final avoid_events=avoid_events)
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}})));

  //
  // Definition of variables
  //
protected
  Real[no_coefficients] c_isotherm
    "Temperature-dependent coefficients of isotherm"
    annotation (HideResult = true);
  Real[no_coefficients] dc_isotherm_dT
    "Derivative of temperature-dependent coefficients of isotherm wrt.
     temperature"
    annotation (HideResult = true);

equation
  //
  // Assertion to check for invalid parameter selections
  //
  assert(type_SpecEnthalpyAdsorpt<>
    Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy,
    "Error in working pair model: Specific adsorption enthalpy cannot be 
     derived from specific bonding and specific bubble enthalpies since 
     specific bonding enthalpy is not defined for equilibrium models based on 
     the surface approach!");

  //
  // Passing through fluid properties from the media models
  //
  h_sorptive = mediaModels.h_sorptive
    "Specific enthalpy of sorptive";
  v_adsorpt = mediaModels.v_adsorpt
    "Specific volume of adsorpt";
  cp_adsorpt = mediaModels.cp_adsorpt
    "Specific heat capacity of adsorpt";

  //
  // Check form of isotherm
  //
  if type_isotherm == Internals.Types.Isotherm.Classical then
    //
    // Classical form (i.e., x(p,T))
    //
    x_adsorpt = func_x_pT(p_adsorpt, c_isotherm)
      "Calculate equlibrium loading of adsorpt";

  else
    //
    // Inverse form (i.e., p(x,T))
    //
    p_adsorpt = func_p_xT(x_adsorpt, c_isotherm)
      "Calculate equlibrium pressure of adsorpt";

  end if;

  //
  // Calculate specific enthalpy of adsorpt
  //
  if type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy then
    // Use definition of adsorption enthalpy to calculate specific enthalpy of
    // adsorpt
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive - dh_ads else -1
      "Calculation of specific enthalpy of adsorpt";

  else
    // Currently, no other calculation approach is implemented
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive - dh_ads else -1
      "Calculation of specific enthalpy of adsorpt";

  end if;

  //
  // Calculate specific adsorption enthalpy
  //
  dh_bond = -1
    "Calculation of specific bonding enthalpy: Not defined for equilibrium
     models based on the surface approach";

  dh_ads =if calculate_caloric_properties then
    Function.SingleComponent.AdsorptionEnthalpy.dhAds_ClausiusClapeyron(
    R_adsorpt,
    p_adsorpt,
    T_adsorpt,
    func_dx_dT_pT(p_adsorpt, c_isotherm, dc_isotherm_dT),
    func_dx_dp_pT(p_adsorpt, c_isotherm)) else -1
    "Calculation of specific adsorption enthalpy";

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end PartialSingleWPair_Surface;
