within SorpLib.Media.WorkingPair.SingleComponent.BaseClasses;
partial model PartialSingleWPair_Volumetric "Base model for all working pairs models describing single component 
  adsorption based on volumetric approach (e.g., Dubinin)"
  extends PartialSingleWPair(
    type_SpecEnthalpyAdsorpt=
      Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy,
    final type_AdsorptionEnthalpy= if (type_SpecEnthalpyAdsorpt==
      Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy) then
      Internals.Types.AdsorptionEnthalpy.Bonding else
      Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron);

  //
  // Definition of parameters
  //
  parameter Real[:] c
    "Coefficients of characteristic curve"
    annotation (Dialog(tab = "General", group = "Equilibrium Model"),
                HideResult=true,
                Evaluate=true);
  replaceable function func_W_A =
    Function.SingleComponent.Equilibrium.VolumetricApproach.CharCurve_Arctan
    constrainedby Function.SingleComponent.BaseClasses.PartiaVolumetricFunction
    "Classical form of characteristic curve: Calculates W from A"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_A_W =
    Function.SingleComponent.Equilibrium.VolumetricApproach.CharCurve_Arctan_inverse
    constrainedby
    Function.SingleComponent.BaseClasses.PartiaVolumetricFunction_inverse
    "Inverse form of characteristic curve: Calculates A from W"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_dW_dA_A =
    Function.SingleComponent.Equilibrium.VolumetricApproach.CharCurve_Arctan__dWdA
    constrainedby Function.SingleComponent.BaseClasses.PartiaVolumetricFunction
    "Classical form of characteristic curve: Calculates partial darivative of W
     wrt. A from A"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable function func_dA_dW_W =
    Function.SingleComponent.Equilibrium.VolumetricApproach.CharCurve_Arctan__dAdW
    constrainedby
    Function.SingleComponent.BaseClasses.PartiaVolumetricFunction_inverse
    "Inverse form of characteristic curve: Calculates partial derivative of A 
     wrt. W from W"
    annotation (Dialog(tab="General", group="Equilibrium Model"),
                choicesAllMatching=true,
                Evaluate=true,
                HideResult = true);
  replaceable model CharCurve =
    Function.SingleComponent.Equilibrium.VolumetricApproach.CharCurve_Arctan_Summary
    constrainedby Function.SingleComponent.BaseClasses.PartialCharCurve_Summary
    "Equilibrium model that summarizes important quantities of characteristic
     curve and that is required if adsorption enthalpy is calculated
     by Clausius-Clapeyron approach using the finite difference approach"
    annotation (Dialog(tab = "General", group = "Equilibrium Model"),
                HideResult=true,
                Evaluate=true);

  replaceable model MediaModel = PartialSingleMediaModel_Volumetric
    constrainedby PartialSingleMediaModel_Volumetric
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true);

  parameter Modelica.SIunits.SpecificEnergy A_start = 1e5
    "Start value for adsorption potential"
    annotation (Dialog(tab = "Initialisation", group = "Start Values"),
                HideResult=true);
  parameter Modelica.SIunits.SpecificVolume W_start = 5e-5
    "Start value for specific volume of adsorbate"
    annotation (Dialog(tab = "Initialisation", group = "Start Values"),
                HideResult=true);

  parameter Modelica.SIunits.TemperatureDifference dT_AdsEnt_CC=1e-4
    "Temperature difference required to calculate adsorption enthalpy with
     Clausius-Clapeyron approach using the finite difference approache"
     annotation (Dialog(tab="Advanced", group="Adsorpt",
                 enable = type_SpecEnthalpyAdsorpt==
                 Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy),
                 HideResult=true);

  parameter Boolean use_CharCurveLimiter = true
    "= true, if limiter is used for characteristic curve (e.g., A and W)"
    annotation (Dialog(tab = "Advanced", group = "Numerics"),
                HideResult=true,
                Evaluate=true);
  parameter Modelica.SIunits.SpecificEnergy A_min = 0
    "Minimal adsorption potential"
    annotation (Dialog(tab = "Advanced", group = "Numerics",
                enable = use_CharCurveLimiter),
                HideResult=true);
  parameter Modelica.SIunits.SpecificVolume W_max = 1e-4
    "Maximal specific volume of adsorbate"
    annotation (Dialog(tab = "Advanced", group = "Numerics",
                enable = use_CharCurveLimiter),
                HideResult=true);

  //
  // Definition of variables
  //
  Modelica.SIunits.SpecificEnergy A
    "Adsorption potential";
  Modelica.SIunits.SpecificVolume W
    "Specific volume of adsorbate";

protected
  Modelica.SIunits.Pressure p_sat_adsorpt
    "Saturation pressure at temperature T_adsorpt"
    annotation (HideResult=true);
  Modelica.SIunits.SpecificEnthalpy h_sat_liq_adsorpt
    "Specific bubble enthalpy of at temperature T_adsorpt"
    annotation (HideResult=true);
  Real beta_sat_liq_adsorpt(final unit="1/K")
    "Bubble isobaric expansion coefficient of adsorpt at temperature T_adsorpt"
    annotation (HideResult=true);

  Modelica.SIunits.SpecificEnergy A_(start=A_start)
    "Adsorption potential that is not limited"
    annotation (HideResult=true);
  Modelica.SIunits.SpecificVolume W_(start=W_start)
    "Specific volume of adsorbate that is not limited"
    annotation (HideResult=true);

  Modelica.SIunits.Pressure p_sat_mdT_adsorpt
    "Saturation pressure at temperature T_adsorpt-dT_AdsEnt_CC"
    annotation (HideResult=true);
  Modelica.SIunits.Pressure p_sat_pdT_adsorpt
    "Saturation pressure at temperature T_adsorpt+dT_AdsEnt_CC"
    annotation (HideResult=true);
  Modelica.SIunits.Density d_sat_liq_mdT_adsorpt(min=-1)
    "Saturated liquid density at temperature T_adsorpt-dT_AdsEnt_CC"
    annotation (HideResult=true);
  Modelica.SIunits.Density d_sat_liq_pdT_adsorpt(min=-1)
    "Saturated liquid density at temperature T_adsorpt+dT_AdsEnt_CC"
    annotation (HideResult=true);

  //
  // Instanziation of models
  //
public
  MediaModel mediaModels(
    final p_adsorpt=if use_stateLimiter then max(p_min, p_adsorpt) else
      p_adsorpt,
    final T_adsorpt=T_adsorpt,
    final calculate_caloric_properties=calculate_caloric_properties,
    final type_AdsorptionEnthalpy=type_AdsorptionEnthalpy,
    final type_SpecEnthalpySorptive=type_SpecEnthalpySorptive,
    final type_SpecEnthalpyAdsorpt=type_SpecEnthalpyAdsorpt,
    final type_SpecHeatCapacityAdsorpt=type_SpecHeatCapacityAdsorpt,
    final cp_adsorpt_fixed=cp_adsorpt_fixed,
    final use_SpecVolumeAdsorpt=use_SpecVolumeAdsorpt,
    final type_SpecVolumeAdsorpt=type_SpecVolumeAdsorpt,
    final dT_AdsEnt_CC=dT_AdsEnt_CC,
    final avoid_events=avoid_events)
    "Model containing media models to calculate fluid properties required by 
     adsorption equilibrium models"
    annotation (Placement(transformation(extent={{-20,-20},{20,20}})));

protected
  CharCurve charCurve_mdT(
    final type_charCurve=Internals.Types.Isotherm.Inverse,
    final c=c,
    final beta_adsorpt=0,
    final A_input=0,
    final T_adsorpt=T_adsorpt-dT_AdsEnt_CC,
    final W_input=if use_CharCurveLimiter then min(x_adsorpt/
      d_sat_liq_mdT_adsorpt, W_max) else x_adsorpt/d_sat_liq_mdT_adsorpt,
    final AInitial=A_start,
    final WInitial=W_start,
    final calculate_dh_bond=false) if calculate_caloric_properties and
    type_AdsorptionEnthalpy ==
    Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron
    "Properties of characteristic curve calculated with x_adsorpt and 
     T_adsorpt-dT_AdsEnt_CC"
    annotation (Placement(transformation(extent={{-40,-90},{-20,-70}})),
                HideResult=true);

  CharCurve charCurve_pdT(
    final type_charCurve=Internals.Types.Isotherm.Inverse,
    final c=c,
    final beta_adsorpt=0,
    final A_input=0,
    final T_adsorpt=T_adsorpt+dT_AdsEnt_CC,
    final W_input=if use_CharCurveLimiter then min(x_adsorpt/
      d_sat_liq_pdT_adsorpt, W_max) else x_adsorpt/d_sat_liq_pdT_adsorpt,
    final AInitial=A_start,
    final WInitial=W_start,
    final calculate_dh_bond=false) if calculate_caloric_properties and
    type_AdsorptionEnthalpy ==
    Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron
    "Properties of characteristic curve calculated with x_adsorpt and 
     T_adsorpt+dT_AdsEnt_CC"
    annotation (Placement(transformation(extent={{22,-90},{42,-70}})),
                HideResult=true);

  //
  // Definition of internal variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput dh_ads_dummy(final unit="J/kg")=
    -1 if not calculate_caloric_properties
    "Specific adsorption enthalpy: Dummy value if caloric properties are 
     not required"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput dh_ads_Bonding(final unit="J/kg")=
    Function.SingleComponent.AdsorptionEnthalpy.dhAds_Bonding(h_sorptive,
      h_adsorpt) if calculate_caloric_properties and
    type_AdsorptionEnthalpy == Internals.Types.AdsorptionEnthalpy.Bonding
    "Specific adsorption enthalpy: Calculated with bonding enthalpy"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput dh_ads_CC(final unit="J/kg")=
    Function.SingleComponent.AdsorptionEnthalpy.dhAds_ClausiusClapeyron_num(
    R_adsorpt,
    p_sat_mdT_adsorpt * exp(-charCurve_mdT.A_output/R_adsorpt/
      (T_adsorpt-dT_AdsEnt_CC)),
    p_sat_pdT_adsorpt * exp(-charCurve_pdT.A_output/R_adsorpt/
      (T_adsorpt+dT_AdsEnt_CC)),
    T_adsorpt-dT_AdsEnt_CC,
    T_adsorpt+dT_AdsEnt_CC) if calculate_caloric_properties and
    type_AdsorptionEnthalpy == Internals.Types.AdsorptionEnthalpy.ClausiusClapeyron
    "Specific adsorption enthalpy: Calculated with Clausius-Clapyron approach"
    annotation (HideResult = true);

  Modelica.Blocks.Interfaces.RealOutput dh_ads_(final unit="J/kg")
    "Specific adsorption enthalpy: Depends on calculation approach"
    annotation (HideResult = true);

equation
  //
  // Pass through values that are calculated using selectable calculation
  // approaches
  //
  h_sorptive = mediaModels.h_sorptive
    "Specific enthalpy of sorptive";

  v_adsorpt = mediaModels.v_adsorpt
    "Specific volume of adsorpt";
  cp_adsorpt = mediaModels.cp_adsorpt
    "Specific heat capacity of adsorpt";

  p_sat_adsorpt = mediaModels.p_sat_adsorpt
    "Saturation pressure at temperature T_adsorpt";
  h_sat_liq_adsorpt = mediaModels.h_sat_liq_adsorpt
    "Specific bubble enthalpy";
  beta_sat_liq_adsorpt = mediaModels.beta_sat_liq_adsorpt
    "Bubble isobaric expansion valve";

  dh_ads = dh_ads_
    "Specific adsorption enthalpy";

  p_sat_mdT_adsorpt = mediaModels.p_sat_mdT_adsorpt
    "Saturation pressure at T_adsorpt-dT_AdsEnt_CC";
  p_sat_pdT_adsorpt = mediaModels.p_sat_pdT_adsorpt
    "Saturation pressure at T_adsorpt+dT_AdsEnt_CC";
  d_sat_liq_mdT_adsorpt = mediaModels.d_sat_liq_mdT_adsorpt
    "Saturated liquid density at T_adsorpt-dT_AdsEnt_CC";
  d_sat_liq_pdT_adsorpt = mediaModels.d_sat_liq_pdT_adsorpt
    "Saturated liquid density at T_adsorpt+dT_AdsEnt_CC";

  //
  // Check form of characteristic curve
  //
  if type_isotherm == Internals.Types.Isotherm.Classical then
    //
    // Classical form of characteristic curve: W(A) with A(p,T).
    // Thus, p and T are known and x shall be calculated1
    //
    W = func_W_A(A, c)
      "Specific volume of adsorbat";

    dh_bond = if calculate_caloric_properties then
      -A + T_adsorpt*beta_sat_liq_adsorpt*W / func_dW_dA_A(A, c) else -1
      "Bonding enthalpy";

  else
    //
    // Inverse form of characteristic curve: A(W)
    //
    A = func_A_W(W, c)
      "Adsorption potential";

    dh_bond = if calculate_caloric_properties then
      -A + T_adsorpt*beta_sat_liq_adsorpt*W*func_dA_dW_W(W, c) else -1
      "Bonding enthalpy";

  end if;

  //
  // Apply limiter to avoid numerical issues if necessary
  //
  if type_isotherm == Internals.Types.Isotherm.Classical then
    //
    // Classical form of characteristic curve: W(A) with A(p,T).
    // Thus, p and T are known and x shall be calculated1
    //
    A = if use_CharCurveLimiter then max(A_, A_min) else A_
      "Limited adsorption potential";
    W = W_
      "Due to limitation of adsorption potential, specific volume of adsorbat
       is also limited";

  else
    //
    // Inverse form of characteristic curve: A(W) with W(x,T).
    // Thus, x and T are known and p shall be calculated1
    //
    W = if use_CharCurveLimiter then min(W_, W_max) else W_
      "Limited specific volume of adsorbat";
    A = A_
      "Due to limitation of specific volume of adsorbat, adsorption potential
       is also limited";

  end if;

  //
  // Calculate specific enthalpy of adsorpt
  //
  if type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.AdsEnthalpy then
    // Use definition of adsorption enthalpy to calculate specific enthalpy of
    // adsorpt
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive - dh_ads else -1
      "Calculation of specific enthalpy of adsorpt";

  elseif type_SpecEnthalpyAdsorpt==
    Internals.Types.SpecificEnthalpyAdsorpt.BondEnthalpy then
    // Use definition of bonding enthalpy to calculate specific enthalpy of
    // adsorpt
    //
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sat_liq_adsorpt + dh_bond else -1
      "Calculation of specific enthalpy of adsorpt";

  else
    // Currently, no other calculation approach is implemented
    //
    h_adsorpt = if calculate_caloric_properties then
      h_sorptive + dh_bond else -1
      "Calculation of specific enthalpy of adsorpt";

  end if;

  //
  // Calculate volumetric specific variables
  //
  if type_isotherm == Internals.Types.Isotherm.Classical then
    //
    // Classical form of characteristic curve: W(A) with A(p,T).
    // Thus, p and T are known and x shall be calculated1
    //
    p_adsorpt = p_sat_adsorpt * exp(-A_/R_adsorpt/T_adsorpt)
      "Equilibrium pressure of adsorpt";
    //abs(p_adsorpt/p_sat_adsorpt) = exp(-A_/R_adsorpt/T_adsorpt)
    //  "Equilibrium pressure of adsorpt";
    W = x_adsorpt*v_adsorpt
      "Specific volume of adsorbat";

  else
    //
    // Inverse form of characteristic curve: A(W) with W(x,T).
    // Thus, x and T are known and p shall be calculated1
    //
    W_ = x_adsorpt*v_adsorpt
      "Specific volume of adsorbat";
    p_adsorpt = p_sat_adsorpt * exp(-A/R_adsorpt/T_adsorpt)
      "Equilibrium pressure of adsorpt";
    //abs(p_adsorpt/p_sat_adsorpt) = exp(-A/R_adsorpt/T_adsorpt)
    //  "Equilibrium pressure of adsorpt";

  end if;

  //
  // Connections for conditional removed variables / models
  //
  connect(dh_ads_,dh_ads_dummy);
  connect(dh_ads_,dh_ads_Bonding);
  connect(dh_ads_,dh_ads_CC);

  //
  // Annotations
  //
  annotation (Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  Major reivisions (e.g., object-oriented approach, functionaliy) due to restructuring of the library.
  </li>
  <li>
  November 17, 2017, by Uwe Bau:<br/> 
  Tidy up implementation and enhance documentation for publication of library.
  </li>
</ul>
</html>", info="<html>
<p>This partial model provides the basic equations to describe equilibrium data for the working pair according to the Dubinin (1967) model.</p><p>The partial model needs to be extended by the specific functions based on the characteristic curve: W = f(A) and h<sub>bond</sub>. </p>
<h4>Main equations</h4>
<p align=\"center\"><i>W = x / <code>&rho;</code><sub>ad</sub>(T) , </i></p>
<p>where <i>W</i> is the specific volume of the adsorbate, <i>x</i> is the loading, <code><i>&rho;</i></code><sub>ad</sub> is the density of the adsorbate, and <i>T</i> is the temperature. </p>
<p align=\"center\"><i>p = p<sub>s</sub>(T) exp(-A / (R T)) , </i></p>
<p>where <i>A</i> is the adsorption potential, <i>p</i><sub>s</sub>(<i>T</i>) is the saturation pressure at temperature <i>T</i>, and <i>R</i> is the specific gas constant of water.</p>
<p><br><i>h</i><sub>ads</sub> describes the specific enthalpy of the adsorbate given the specific enthalpy of water as reference state. </p>
<p align=\"center\"><i>h<sub>ads</sub> = h<sub>l</sub> + h<sub>bond</sub> , </i></p>
<p>where <i>h</i><sub>l</sub> is specific enthalpy of liquid water and <i>h</i><sub>bond</sub> is the specific bonding enthalpy. </p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<ul>
<li>Dubinin, M.M. Adsorption in micropores. J. Colloid Interface Sci., 1967, 23(4), 487-499. </li>
<li>N&uacute;&ntilde;ez, T. Charakterisierung und Bewertung von Adsorbentien f&uuml;r W&auml;rmetransformationsanwendungen. PhD Thesis. Freiburg, 2001. </li>
</ul>
</html>"));
end PartialSingleWPair_Volumetric;
