within SorpLib.Media.WorkingPair.SingleComponent.MediaModels;
model SingleMediaModel_Surface_TIL_Gas
  "Media model required by adsorption equilibrium models of single component 
   adsorption based on the surface approach and using the TILMedia library to 
   calculate fluid properties"
  extends BaseClasses.PartialSingleMediaModel_Surface(
    use_SpecVolumeAdsorpt=false,
    type_SpecEnthalpySorptive=
      Internals.Types.SpecificEnthalpySorptive.Media_pT,
    type_SpecHeatCapacityAdsorpt=
      Internals.Types.SpecificHeatCapacityAdsorpt.GasVapor);

  //
  // Definition of parameters
  //
  replaceable parameter Media.TILMediaDefinition.Gas.TILMedia_H2O gasType
    constrainedby TILMedia.GasTypes.BaseGas
    "Gas type"
    annotation (Dialog(tab = "Media", group = "Media Model"),
                choicesAllMatching=true,
                Evaluate=true);

  //
  // Definition of media models
  //
  TILMedia.Gas_pT media_sorptive(
    final gasType=gasType,
    final computeTransportProperties=false,
    final p=p_adsorpt,
    final T=T_adsorpt) if
    calculate_caloric_properties and ((type_SpecEnthalpySorptive==
    Internals.Types.SpecificEnthalpySorptive.Media_pT) or
    (type_SpecHeatCapacityAdsorpt==
    Internals.Types.SpecificHeatCapacityAdsorpt.GasVapor))
    "Fluid properties of sorptive"
    annotation (Placement(transformation(extent={{-10,30},{10,50}})));

  //
  // Definition of internal variables
  //
protected
  Modelica.Blocks.Interfaces.RealInput h_sorptive_media_pT(final unit="J/kg")=
    media_sorptive.h if calculate_caloric_properties and
    (type_SpecEnthalpySorptive==
    Internals.Types.SpecificEnthalpySorptive.Media_pT)
    "Specific enthalpy of sorptive: Calculated with media object using states
     p_adsorpt and T_adsorpt"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput h_sorptive_invalid(final unit="J/kg")=
    -1 if calculate_caloric_properties and (type_SpecEnthalpySorptive<>
    Internals.Types.SpecificEnthalpySorptive.Media_pT)
    "Specific enthalpy of sorptive: Invalid choice for current media model"
    annotation (HideResult = true);

  Modelica.Blocks.Interfaces.RealInput v_adsorpt_invalid(final unit="m3/kg")=
    -1 if calculate_caloric_properties and use_SpecVolumeAdsorpt and
    (type_SpecVolumeAdsorpt==Internals.Types.SpecificVolumeAdsorpt.SatLiq)
    "Specific volume of sorptive: Dummy value if caloric properties are not
     required"
    annotation (HideResult = true);

  Modelica.Blocks.Interfaces.RealInput cp_adsorpt_vapor(final unit="J/(kg.K)")=
    media_sorptive.cp if calculate_caloric_properties and
    (type_SpecHeatCapacityAdsorpt==
    Internals.Types.SpecificHeatCapacityAdsorpt.GasVapor)
    "Specific heat capacity of sorptive: Specific heat capacity of vapor phase"
    annotation (HideResult = true);
  Modelica.Blocks.Interfaces.RealInput cp_adsorpt_invalid(final unit="J/(kg.K)")=
    -1 if calculate_caloric_properties and (type_SpecHeatCapacityAdsorpt<>
    Internals.Types.SpecificHeatCapacityAdsorpt.Constant and
    type_SpecHeatCapacityAdsorpt<>
    Internals.Types.SpecificHeatCapacityAdsorpt.GasVapor)
    "Specific heat capacity of sorptive: Invalid choice for current media model"
    annotation (HideResult = true);

equation
  //
  // Assertion to check for invalid parameter selections
  //
  assert(type_SpecEnthalpySorptive<>
    Internals.Types.SpecificEnthalpySorptive.Media_VLE,
    "Error in working pair model: Specific enthalpy of sorptive cannot be
     approximated via specific dew enthalpy since gas media model does not
     support VLE properties!");
  assert(not use_SpecVolumeAdsorpt or type_SpecVolumeAdsorpt<>
    Internals.Types.SpecificVolumeAdsorpt.SatLiq,
    "Error in working pair model: Specific volume of adsorpt cannot be
     approximated via specific bubble volume since gas media model does not
     support VLE properties!");
  assert(type_SpecHeatCapacityAdsorpt<>
    Internals.Types.SpecificHeatCapacityAdsorpt.SatLiq,
    "Error in working pair model: Specific heat capacity of adsorpt cannot be
     approximated via specific bubble heat capacity since gas media model does
     not support VLE properties!");

  //
  // Connections for conditional removed variables / models
  //
  connect(h_sorptive, h_sorptive_media_pT);
  connect(h_sorptive, h_sorptive_invalid);

  connect(v_adsorpt, v_adsorpt_invalid);

  connect(cp_adsorpt, cp_adsorpt_vapor);
  connect(cp_adsorpt, cp_adsorpt_invalid);

  //
  // Annotations
  //
  annotation (Icon(graphics={
        Text(
          extent={{-40,-80},{40,-100}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={215,215,215},
          fillPattern=FillPattern.Solid,
          textStyle={TextStyle.Bold},
          textString="TILMedia | Gas")}),
                    Diagram(graphics={                         Line(
          points={{0,20},{0,-20}},
          color={0,0,0},
          arrow={Arrow.Open,Arrow.Open}), Text(
          extent={{-20,10},{20,-10}},
          lineColor={0,0,0},
          textString="Equilibrium",
          origin={-10,0},
          rotation=90)}),
    Documentation(revisions="<html>
<ul>
  <li>
  December 14, 2020, by Mirko Engelpracht:<br/>
  First implementation.
  </li>
</ul>
</html>", info="<html>
<p>
A short introduction.
</p>
<h4>Main equations</h4>
<p>
Most important equations (e.g., mass and energy balances).
</p>
<h4>Assumptions and limitations</h4>
<p>
Main assumptions and limitations (e.g., ideal mixture).
</p>
<h4>Typical use</h4>
<p>
Typical use of model (e.g., to calculate mass transfer within sorbent).
</p>
<h4>Important parameters and options</h4>
<p>
Main parameters and options of the model (e.g., description of most important flags).
</p>
<h4>Dynamics</h4>
<p>
Describe which states and dynamics are present in the model
and which parameters may be used to influence them.
This need not be added in partial classes.
</p>
<h4>Validation</h4>
<p>
Describe whether the validation was done using
analytical validation, comparative model validation
or empirical validation.
</p>
<h4>References</h4>
<p>
xxx
</p>
</html>"));
end SingleMediaModel_Surface_TIL_Gas;
